@@ -143,8 +143,6 @@
 
 #include "temporal/time.h"
 
-typedef uint64_t microseconds_t;
-
 #include "about.h"
 #include "editing.h"
 #include "enums_convert.h"
@@ -158,6 +156,7 @@ typedef uint64_t microseconds_t;
 #include "big_clock_window.h"
 #include "big_transport_window.h"
 #include "bundle_manager.h"
+#include "dsp_stats_window.h"
 #include "duplicate_routes_dialog.h"
 #include "debug.h"
 #include "engine_dialog.h"
@@ -184,9 +183,11 @@ typedef uint64_t microseconds_t;
 #include "opts.h"
 #include "pingback.h"
 #include "plugin_dspload_window.h"
+#include "plugin_manager_ui.h"
 #include "processor_box.h"
 #include "public_editor.h"
 #include "rc_option_editor.h"
+#include "recorder_ui.h"
 #include "route_time_axis.h"
 #include "route_params_ui.h"
 #include "save_as_dialog.h"
@@ -209,6 +210,7 @@ typedef uint64_t microseconds_t;
 #include "virtual_keyboard_window.h"
 #include "add_video_dialog.h"
 #include "transcode_video_dialog.h"
+#include "plugin_selector.h"
 
 #include "pbd/i18n.h"
 
@@ -299,13 +301,13 @@ ARDOUR_UI::ARDOUR_UI (int *argcp, char **argvp[], const char* localedir)
 	, main_window_visibility (0)
 	, editor (0)
 	, mixer (0)
+	, recorder (0)
 	, nsm (0)
 	, _was_dirty (false)
 	, _mixer_on_top (false)
 	, _shared_popup_menu (0)
 	, startup_fsm (0)
 	, secondary_clock_spacer (0)
-	, auto_input_button (ArdourButton::led_default_elements)
 	, latency_disable_button (ArdourButton::led_default_elements)
 	, time_info_box (0)
 	, auto_return_button (ArdourButton::led_default_elements)
@@ -334,7 +336,9 @@ ARDOUR_UI::ARDOUR_UI (int *argcp, char **argvp[], const char* localedir)
 	, export_video_dialog (X_("video-export"), _("Video Export Dialog"))
 	, lua_script_window (X_("script-manager"), _("Script Manager"))
 	, idleometer (X_("idle-o-meter"), _("Idle'o'Meter"))
+	, plugin_manager_ui (X_("plugin-manager"), _("Plugin Manager"))
 	, plugin_dsp_load_window (X_("plugin-dsp-load"), _("Plugin DSP Load"))
+	, dsp_statistics_window (X_("dsp-statistics"), _("Performance Meters"))
 	, transport_masters_window (X_("transport-masters"), _("Transport Masters"))
 	, session_option_editor (X_("session-options-editor"), _("Properties"), boost::bind (&ARDOUR_UI::create_session_option_editor, this))
 	, add_video_dialog (X_("add-video"), _("Add Video"), boost::bind (&ARDOUR_UI::create_add_video_dialog, this))
@@ -352,11 +356,13 @@ ARDOUR_UI::ARDOUR_UI (int *argcp, char **argvp[], const char* localedir)
 	, have_disk_speed_dialog_displayed (false)
 	, _status_bar_visibility (X_("status-bar"))
 	, _feedback_exists (false)
+	, _ambiguous_latency (false)
 	, _log_not_acknowledged (LogLevelNone)
 	, duplicate_routes_dialog (0)
-	, editor_visibility_button (S_("Window|Editor"))
-	, mixer_visibility_button (S_("Window|Mixer"))
-	, prefs_visibility_button (S_("Window|Preferences"))
+	, editor_visibility_button (S_("Window|Edit"))
+	, mixer_visibility_button (S_("Window|Mix"))
+	, prefs_visibility_button (S_("Window|Prefs"))
+	, recorder_visibility_button (S_("Window|Rec"))
 {
 	Gtkmm2ext::init (localedir);
 
@@ -501,7 +507,9 @@ ARDOUR_UI::ARDOUR_UI (int *argcp, char **argvp[], const char* localedir)
 		export_video_dialog.set_state (*ui_xml, 0);
 		lua_script_window.set_state (*ui_xml, 0);
 		idleometer.set_state (*ui_xml, 0);
+		plugin_manager_ui.set_state (*ui_xml, 0);
 		plugin_dsp_load_window.set_state (*ui_xml, 0);
+		dsp_statistics_window.set_state (*ui_xml, 0);
 		transport_masters_window.set_state (*ui_xml, 0);
 	}
 
@@ -528,7 +536,9 @@ ARDOUR_UI::ARDOUR_UI (int *argcp, char **argvp[], const char* localedir)
 	WM::Manager::instance().register_window (&audio_port_matrix);
 	WM::Manager::instance().register_window (&midi_port_matrix);
 	WM::Manager::instance().register_window (&idleometer);
+	WM::Manager::instance().register_window (&plugin_manager_ui);
 	WM::Manager::instance().register_window (&plugin_dsp_load_window);
+	WM::Manager::instance().register_window (&dsp_statistics_window);
 	WM::Manager::instance().register_window (&transport_masters_window);
 
 	/* session-sensitive windows */
@@ -634,14 +644,13 @@ the audio backend and save the session."), PROGRAM_NAME);
 void
 ARDOUR_UI::post_engine ()
 {
-	/* Things to be done once (and once ONLY) after we have a backend running in the AudioEngine
-	 */
+	/* Things to be done once (and once ONLY) after we have a backend running in the AudioEngine */
+
 #ifdef AUDIOUNIT_SUPPORT
-	std::string au_msg;
-	if (AUPluginInfo::au_get_crashlog(au_msg)) {
-		popup_error(_("Audio Unit Plugin Scan Failed. Automatic AU scanning has been disabled. Please see the log window for further details."));
-		error << _("Audio Unit Plugin Scan Failed:") << endmsg;
-		info << au_msg << endmsg;
+	string aucrsh = Glib::build_filename (ARDOUR::user_cache_directory(), "au_crash");
+	if (Glib::file_test (aucrsh, Glib::FILE_TEST_EXISTS)) {
+		popup_error (_("Indexing Audio Unit Plugin Failed.\nAutomatic AU scanning has been disabled\n(check with 'auval', then re-enable scanning the in preferences)."));
+		::g_unlink (aucrsh.c_str());
 	}
 #endif
 
@@ -833,6 +842,11 @@ ARDOUR_UI::~ARDOUR_UI ()
 
 	stop_video_server();
 
+	/* unsubscribe from AudioEngine::Stopped */
+	if (recorder) {
+		recorder->cleanup ();
+	}
+
 	if (getenv ("ARDOUR_RUNNING_UNDER_VALGRIND")) {
 		// don't bother at 'real' exit. the OS cleans up for us.
 		delete big_clock; big_clock = 0;
@@ -842,6 +856,7 @@ ARDOUR_UI::~ARDOUR_UI ()
 		delete time_info_box; time_info_box = 0;
 		delete meterbridge; meterbridge = 0;
 		delete luawindow; luawindow = 0;
+		delete recorder; recorder = 0;
 		delete editor; editor = 0;
 		delete mixer; mixer = 0;
 		delete rc_option_editor; rc_option_editor = 0; // failed to wrap object warning
@@ -851,8 +866,12 @@ ARDOUR_UI::~ARDOUR_UI ()
 		delete main_window_visibility;
 		FastMeter::flush_pattern_cache ();
 		ArdourFader::flush_pattern_cache ();
+	} else if (mixer) {
+		/* drop references to any PluginInfoPtr */
+		delete mixer->plugin_selector ();
 	}
 
+
 #ifndef NDEBUG
 	/* Small trick to flush main-thread event pool.
 	 * Other thread-pools are destroyed at pthread_exit(),
@@ -1069,9 +1088,6 @@ If you still wish to quit, please use the\n\n\
 
 	halt_connection.disconnect ();
 	AudioEngine::instance()->stop ();
-#ifdef WINDOWS_VST_SUPPORT
-	fst_stop_threading();
-#endif
 	quit ();
 }
 
@@ -1670,7 +1686,7 @@ ARDOUR_UI::transport_goto_wallclock ()
 		samples += tmnow.tm_min * (60 * sample_rate);
 		samples += tmnow.tm_sec * sample_rate;
 
-		_session->request_locate (samples, RollIfAppropriate);
+		_session->request_locate (samples);
 
 		/* force displayed area in editor to start no matter
 		   what "follow playhead" setting is.
@@ -1819,7 +1835,7 @@ ARDOUR_UI::transport_roll ()
 	}
 
 	if (!rolling) {
-		_session->request_transport_speed (1.0f);
+		_session->request_roll ();
 	}
 }
 
@@ -1897,7 +1913,7 @@ ARDOUR_UI::toggle_roll (bool with_abort, bool roll_out_of_bounded_mode)
 					_session->set_requested_return_sample (range.front().start);  //force an auto-return here
 				}
 			}
-			_session->request_transport_speed (1.0f);
+			_session->request_roll ();
 		}
 	}
 }
@@ -1978,54 +1994,111 @@ ARDOUR_UI::transport_rec_count_in ()
 }
 
 void
-ARDOUR_UI::transport_ffwd_rewind (int option, int dir)
+ARDOUR_UI::transport_ffwd_rewind (bool fwd)
 {
 	if (!_session) {
 		return;
 	}
+	// incrementally increase speed by semitones
+	// (keypress auto-repeat is 100ms)
+	const float maxspeed = Config->get_shuttle_max_speed();
+	float semitone_ratio = exp2f (1.0f/12.0f);
+	const float octave_down = powf (1.f / semitone_ratio, 12.f);
+	float transport_speed = _session->actual_speed();
+	float speed;
 
-	/* engine speed is always positive, so multiply by transport
-	 * (-1, 0, 1) to get directional value
-	 */
+	if (Config->get_rewind_ffwd_like_tape_decks()) {
 
-	const float current_transport_speed = _session->engine_speed () * _session->transport_speed ();
-	float target_speed = current_transport_speed;
+		if (fwd) {
+			if (transport_speed <= 0) {
+				_session->request_transport_speed (1.0);
+				_session->request_roll (TRS_UI);
+				return;
+			}
+		} else {
+			if (transport_speed >= 0) {
+				_session->request_transport_speed (-1.0);
+				_session->request_roll (TRS_UI);
+				return;
+			}
+		}
+
+
+	} else {
+
+		if (fabs (transport_speed) <= 0.1) {
+
+			/* close to zero, maybe flip direction */
+
+			if (fwd) {
+				if (transport_speed <= 0) {
+					_session->request_transport_speed (1.0);
+					_session->request_roll (TRS_UI);
+				}
+			} else {
+				if (transport_speed >= 0) {
+					_session->request_transport_speed (-1.0);
+					_session->request_roll (TRS_UI);
+				}
+			}
+
+			/* either we've just started, or we're moving as slowly as we
+			 * ever should
+			 */
+
+			return;
+		}
+
+		if (fwd) {
+			if (transport_speed < 0.f) {
+				if (fabs (transport_speed) < octave_down) {
+					/* we need to move the speed back towards zero */
+					semitone_ratio = powf (1.f / semitone_ratio, 4.f);
+				} else {
+					semitone_ratio = 1.f / semitone_ratio;
+				}
+			} else {
+				if (fabs (transport_speed) < octave_down) {
+					/* moving very slowly, use 4 semitone steps */
+					semitone_ratio = powf (semitone_ratio, 4.f);
+				}
+			}
+		} else {
+			if (transport_speed > 0.f) {
+				/* we need to move the speed back towards zero */
+
+				if (transport_speed < octave_down) {
+					semitone_ratio = powf (1.f / semitone_ratio, 4.f);
+				} else {
+					semitone_ratio = 1.f / semitone_ratio;
+				}
+			} else {
+				if (fabs (transport_speed) < octave_down) {
+					/* moving very slowly, use 4 semitone steps */
+					semitone_ratio = powf (semitone_ratio, 4.f);
+				}
+			}
+		}
 
-	switch (option) {
-	case 0:
-		target_speed = dir * 1.0f;
-		break;
-	case 1:
-		target_speed = dir * 4.0f;
-		break;
-	case -1:
-		target_speed = dir * 0.5f;
-		break;
 	}
 
-	/* if wanting to move forward/backward and current speed is at or above current
-	   speed (i.e. same direction, and moving), then speed up.
-	*/
+	speed = semitone_ratio * transport_speed;
+	speed = std::max (-maxspeed, std::min (maxspeed, speed));
+	_session->request_transport_speed (speed);
+	_session->request_roll (TRS_UI);
 
-	const bool speed_up = (dir > 0 && current_transport_speed >= target_speed) || (dir < 0 && current_transport_speed <= target_speed);
-
-	if (speed_up) {
-		target_speed = current_transport_speed * 1.5f;
-	}
-
-	_session->request_transport_speed (target_speed);
 }
 
 void
-ARDOUR_UI::transport_rewind (int option)
+ARDOUR_UI::transport_rewind ()
 {
-	transport_ffwd_rewind (option, -1);
+	transport_ffwd_rewind (false);
 }
 
 void
-ARDOUR_UI::transport_forward (int option)
+ARDOUR_UI::transport_forward ()
 {
-	transport_ffwd_rewind (option, 1);
+	transport_ffwd_rewind (true);
 }
 
 void
@@ -2206,10 +2279,6 @@ ARDOUR_UI::save_template_dialog_response (int response, SaveTemplateDialog* d)
 void
 ARDOUR_UI::save_template ()
 {
-	if (!check_audioengine (_main_window)) {
-		return;
-	}
-
 	const std::string desc = SessionMetadata::Metadata()->description ();
 	SaveTemplateDialog* d = new SaveTemplateDialog (_session->name (), desc);
 	d->signal_response().connect (sigc::bind (sigc::mem_fun (*this, &ARDOUR_UI::save_template_dialog_response), d));
@@ -2627,6 +2696,8 @@ ARDOUR_UI::cleanup_peakfiles ()
 	// - setup peakfiles (background thread)
 	_session->cleanup_peakfiles ();
 
+	ArdourWaveView::WaveView::clear_cache ();
+
 	// re-add waves to ARV
 	for (list<RegionView*>::iterator i = views.begin(); i != views.end(); ++i) {
 		AudioRegionView* arv = dynamic_cast<AudioRegionView*> (*i);
@@ -2889,6 +2960,10 @@ what you would like to do.\n"), PROGRAM_NAME));
 	image->show();
 	hbox->show();
 
+	if (Splash::exists ()) {
+		Splash::instance()->hide ();
+	}
+
 	switch (dialog.run ()) {
 	case RESPONSE_ACCEPT:
 		return 1;
