@@ -73,6 +73,7 @@
 #include "plugin_scan_dialog.h"
 #include "public_editor.h"
 #include "splash.h"
+#include "ui_config.h"
 
 #include "pbd/i18n.h"
 
@@ -124,7 +125,7 @@ ARDOUR_UI::setup_profile ()
 int
 ARDOUR_UI::missing_file (Session*s, std::string str, DataType type)
 {
-	MissingFileDialog dialog (s, str, type);
+	MissingFileDialog dialog (_main_window, s, str, type);
 
 	dialog.show ();
 	dialog.present ();
@@ -309,6 +310,24 @@ ARDOUR_UI::editor_settings () const
 	return node;
 }
 
+XMLNode*
+ARDOUR_UI::recorder_settings () const
+{
+	XMLNode* node = 0;
+
+	if (_session) {
+		node = _session->instant_xml(X_("Recorder"));
+	} else {
+		node = Config->instant_xml(X_("Recorder"));
+	}
+
+	if (!node) {
+		node = new XMLNode (X_("Recorder"));
+	}
+
+	return node;
+}
+
 XMLNode*
 ARDOUR_UI::keyboard_settings () const
 {
@@ -404,11 +423,7 @@ ARDOUR_UI::nsm_init ()
 		}
 	}
 	if (i == 5000) {
-#if 0 // USE AFTER STRINGFREEZE 6.0
 		error << _("NSM server did not announce itself. Continuing without NSM.") << endmsg;
-#else
-		error << _("NSM server did not announce itself") << endmsg;
-#endif
 		delete nsm;
 		nsm = 0;
 		return 0;
@@ -490,7 +505,11 @@ ARDOUR_UI::starting ()
 
 	app->ShouldLoad.connect (sigc::mem_fun (*this, &ARDOUR_UI::load_from_application_api));
 
-	if (ARDOUR_COMMAND_LINE::check_announcements) {
+	if (ARDOUR_COMMAND_LINE::check_announcements
+#ifndef MIXBUS
+	    && UIConfiguration::instance().get_check_announcements ()
+#endif
+	   ) {
 		check_announcements ();
 	}
 
@@ -522,7 +541,7 @@ ARDOUR_UI::starting ()
 
 
 		/* allow signals to be handled, ShouldLoad() from flush-pending */
-		Splash::instance()->pop_front();
+		Splash::instance()->exists(); // create splash
 		flush_pending ();
 
 		if (!startup_fsm) {
@@ -554,8 +573,6 @@ ARDOUR_UI::load_session_from_startup_fsm ()
 	const BusProfile bus_profile = startup_fsm->bus_profile;
 	const bool   session_was_not_named = (!startup_fsm->session_name_edited && ARDOUR_COMMAND_LINE::session_name.empty());
 
-	std::cout  << " loading from " << session_path << " as " << session_name << " templ " << session_template << " is_new " << session_is_new << " bp " << bus_profile.master_out_channels << std::endl;
-
 	if (session_is_new) {
 
 		if (build_session (session_path, session_name, session_template, bus_profile, true, session_was_not_named)) {
@@ -799,7 +816,7 @@ ARDOUR_UI::load_from_application_api (const std::string& path)
 		 * 3) no audio/MIDI setup required
 		 */
 
-		Splash::instance()->pop_front();
+		Splash::instance()->exists(); // create splash
 		startup_fsm->start ();
 	}
 }
