@@ -136,10 +136,10 @@ class MidiExportDialog;
 class MixerStrip;
 class MouseCursors;
 class NoteBase;
-class PlaylistSelector;
 class PluginSelector;
 class ProgressReporter;
 class QuantizeDialog;
+class RegionPeakCursor;
 class RhythmFerret;
 class RulerDialog;
 class Selection;
@@ -188,8 +188,8 @@ public:
 
 	Editing::SnapMode  snap_mode () const;
 	Editing::GridType  grid_type () const;
+	bool  grid_type_is_musical (Editing::GridType) const;
 	bool  grid_musical () const;
-	bool  grid_nonmusical () const;
 
 	void undo (uint32_t n = 1);
 	void redo (uint32_t n = 1);
@@ -310,7 +310,9 @@ public:
 	void export_region ();
 
 	/* export for analysis only */
-	void measure_master_loudness (bool);
+	void loudness_assistant (bool);
+	void loudness_assistant_marker ();
+	void measure_master_loudness (samplepos_t start, samplepos_t end, bool);
 
 	bool process_midi_export_dialog (MidiExportDialog& dialog, boost::shared_ptr<ARDOUR::MidiRegion> midi_region);
 
@@ -328,12 +330,9 @@ public:
 
 	/* stuff that AudioTimeAxisView and related classes use */
 
-	PlaylistSelector& playlist_selector() const;
 	void clear_playlist (boost::shared_ptr<ARDOUR::Playlist>);
 
-	void new_playlists (TimeAxisView* v);
-	void copy_playlists (TimeAxisView* v);
-	void clear_playlists (TimeAxisView* v);
+	void clear_grouped_playlists (RouteUI* v);
 
 	void get_onscreen_tracks (TrackViewList&);
 
@@ -385,6 +384,9 @@ public:
 	/* returns the left-most and right-most time that the gui should allow the user to scroll to */
 	std::pair <samplepos_t,samplepos_t> session_gui_extents (bool use_extra = true) const;
 
+	/* RTAV Automation display option */
+	bool show_touched_automation () const;
+
 	/* fades */
 
 	void toggle_region_fades (int dir);
@@ -445,8 +447,6 @@ public:
 		return track_views;
 	}
 
-	int get_regionview_count_from_region_list (boost::shared_ptr<ARDOUR::Region>);
-
 	void do_ptimport(std::string path, ARDOUR::SrcQuality quality);
 
 	void do_import (std::vector<std::string>              paths,
@@ -456,7 +456,9 @@ public:
 	                ARDOUR::MidiTrackNameSource           mts,
 	                ARDOUR::MidiTempoMapDisposition       mtd,
 	                samplepos_t&                          pos,
-	                boost::shared_ptr<ARDOUR::PluginInfo> instrument = boost::shared_ptr<ARDOUR::PluginInfo>());
+	                boost::shared_ptr<ARDOUR::PluginInfo> instrument = boost::shared_ptr<ARDOUR::PluginInfo>(),
+	                bool with_markers = false
+		);
 
 	void do_embed (std::vector<std::string>              paths,
 	               Editing::ImportDisposition            disposition,
@@ -464,7 +466,7 @@ public:
 	               samplepos_t&                          pos,
 	               boost::shared_ptr<ARDOUR::PluginInfo> instrument = boost::shared_ptr<ARDOUR::PluginInfo>());
 
-	void get_regions_corresponding_to (boost::shared_ptr<ARDOUR::Region> region, std::vector<RegionView*>& regions, bool src_comparison);
+	void get_regionview_corresponding_to (boost::shared_ptr<ARDOUR::Region> region, std::vector<RegionView*>& regions);
 
 	void get_regionviews_by_id (PBD::ID const id, RegionSelection & regions) const;
 	void get_per_region_note_selection (std::list<std::pair<PBD::ID, std::set<boost::shared_ptr<Evoral::Note<Temporal::Beats> > > > >&) const;
@@ -571,12 +573,21 @@ public:
 	void split_regions_at (ARDOUR::MusicSample, RegionSelection&);
 	void split_region_at_points (boost::shared_ptr<ARDOUR::Region>, ARDOUR::AnalysisFeatureList&, bool can_ferret, bool select_new = false);
 	RegionSelection get_regions_from_selection_and_mouse (samplepos_t);
+	void do_remove_gaps ();
+	void remove_gaps (samplecnt_t threshold, samplecnt_t leave, bool markers_too);
+
+	void mouse_brush_insert_region (RegionView*, samplepos_t pos);
 
 	void mouse_add_new_tempo_event (samplepos_t where);
 	void mouse_add_new_meter_event (samplepos_t where);
 	void edit_tempo_section (ARDOUR::TempoSection*);
 	void edit_meter_section (ARDOUR::MeterSection*);
 
+	void add_region_marker ();
+	void clear_region_markers ();
+	void remove_region_marker (ARDOUR::CueMarker&);
+	void make_region_markers_global (bool as_cd_markers);
+
 protected:
 	void map_transport_state ();
 	void map_position_change (samplepos_t);
@@ -596,8 +607,6 @@ private:
 	// to keep track of the playhead position for control_scroll
 	boost::optional<samplepos_t> _control_scroll_target;
 
-	PlaylistSelector* _playlist_selector;
-
 	TimeInfoBox*      _time_info_box;
 
 	typedef std::pair<TimeAxisView*,XMLNode*> TAVState;
@@ -674,8 +683,10 @@ private:
 	void add_new_location (ARDOUR::Location*);
 	ArdourCanvas::Container* add_new_location_internal (ARDOUR::Location*);
 	void location_gone (ARDOUR::Location*);
-	void remove_marker (ArdourCanvas::Item&, GdkEvent*);
-	gint really_remove_marker (ARDOUR::Location* loc);
+	void remove_marker (ArdourCanvas::Item&);
+	void remove_marker (ArdourMarker*);
+	gint really_remove_global_marker (ARDOUR::Location* loc);
+	gint really_remove_region_marker (ArdourMarker*);
 	void goto_nth_marker (int nth);
 	void trigger_script (int nth);
 	void trigger_script_by_name (const std::string script_name);
@@ -705,7 +716,7 @@ private:
 
 		void set_show_lines (bool);
 		void set_selected (bool);
-		void canvas_height_set (double);
+		void set_entered (bool);
 		void setup_lines ();
 
 		void set_name (const std::string&);
@@ -762,14 +773,22 @@ private:
 	RegionView* regionview_from_region (boost::shared_ptr<ARDOUR::Region>) const;
 	RouteTimeAxisView* rtav_from_route (boost::shared_ptr<ARDOUR::Route>) const;
 
-	void mapover_tracks (sigc::slot<void,RouteTimeAxisView&,uint32_t> sl, TimeAxisView*, PBD::PropertyID) const;
 	void mapover_tracks_with_unique_playlists (sigc::slot<void,RouteTimeAxisView&,uint32_t> sl, TimeAxisView*, PBD::PropertyID) const;
-
-	/* functions to be passed to mapover_tracks(), possibly with sigc::bind()-supplied arguments */
 	void mapped_get_equivalent_regions (RouteTimeAxisView&, uint32_t, RegionView*, std::vector<RegionView*>*) const;
-	void mapped_use_new_playlist (RouteTimeAxisView&, uint32_t, std::vector<boost::shared_ptr<ARDOUR::Playlist> > const &);
-	void mapped_use_copy_playlist (RouteTimeAxisView&, uint32_t, std::vector<boost::shared_ptr<ARDOUR::Playlist> > const &);
-	void mapped_clear_playlist (RouteTimeAxisView&, uint32_t);
+
+	void mapover_grouped_routes (sigc::slot<void, RouteUI&> sl, RouteUI*, PBD::PropertyID) const;
+	void mapover_armed_routes (sigc::slot<void, RouteUI&> sl) const;
+	void mapover_selected_routes (sigc::slot<void, RouteUI&> sl) const;
+	void mapover_all_routes (sigc::slot<void, RouteUI&> sl) const;
+
+	void mapped_select_playlist_matching (RouteUI&, boost::weak_ptr<ARDOUR::Playlist> pl);
+	void mapped_use_new_playlist (RouteUI&, std::string name, std::string gid, bool copy, std::vector<boost::shared_ptr<ARDOUR::Playlist> > const &);
+	void mapped_clear_playlist (RouteUI&);
+
+	void new_playlists_for_all_tracks(bool copy);
+	void new_playlists_for_grouped_tracks(RouteUI* v, bool copy);
+	void new_playlists_for_selected_tracks(bool copy);
+	void new_playlists_for_armed_tracks(bool copy);
 
 	void button_selection (ArdourCanvas::Item* item, GdkEvent* event, ItemType item_type);
 	bool button_release_can_deselect;
@@ -848,6 +867,8 @@ private:
 	friend class VerboseCursor;
 	VerboseCursor* _verbose_cursor;
 
+	RegionPeakCursor* _region_peak_cursor;
+
 	void parameter_changed (std::string);
 	void ui_parameter_changed (std::string);
 
@@ -966,7 +987,9 @@ private:
 		bbt_show_quarters,
 		bbt_show_eighths,
 		bbt_show_sixteenths,
-		bbt_show_thirtyseconds
+		bbt_show_thirtyseconds,
+		bbt_show_sixtyfourths,
+		bbt_show_onetwentyeighths
 	};
 
 	BBTRulerScale bbt_ruler_scale;
@@ -974,7 +997,6 @@ private:
 	uint32_t bbt_bars;
 	gint bbt_nmarks;
 	uint32_t bbt_bar_helper_on;
-	uint32_t bbt_accent_modulo;
 	void compute_bbt_ruler_scale (samplepos_t lower, samplepos_t upper);
 
 	ArdourCanvas::Ruler* timecode_ruler;
@@ -1027,11 +1049,9 @@ private:
 	int get_videotl_bar_height () const { return videotl_bar_height; }
 	void toggle_region_video_lock ();
 
-	friend class EditorCursor;
+	EditorCursor* playhead_cursor () const { return _playhead_cursor; }
+	EditorCursor* snapped_cursor () const { return _snapped_cursor; }
 
-	EditorCursor* snapped_cursor;
-
-	EditorCursor* playhead_cursor;
 	samplepos_t playhead_cursor_sample () const;
 
 	samplepos_t get_region_boundary (samplepos_t pos, int32_t dir, bool with_selection, bool only_onscreen);
@@ -1101,7 +1121,7 @@ private:
 
 	bool track_canvas_map_handler (GdkEventAny*);
 
-	bool edit_controls_button_release (GdkEventButton*);
+	bool edit_controls_button_event (GdkEventButton*);
 	Gtk::Menu* edit_controls_left_menu;
 	Gtk::Menu* edit_controls_right_menu;
 
@@ -1373,6 +1393,7 @@ private:
 	                     int                                   target_regions,
 	                     int                                   target_tracks,
 	                     boost::shared_ptr<ARDOUR::Track>&     track,
+	                     std::string const&                    pgroup_id,
 	                     bool                                  replace,
 	                     boost::shared_ptr<ARDOUR::PluginInfo> instrument = boost::shared_ptr<ARDOUR::PluginInfo>());
 
@@ -1385,6 +1406,7 @@ private:
 	                    int                                   target_regions,
 	                    int                                   target_tracks,
 	                    boost::shared_ptr<ARDOUR::Track>&     track,
+	                    std::string const&                    pgroup_id,
 	                    boost::shared_ptr<ARDOUR::PluginInfo> instrument = boost::shared_ptr<ARDOUR::PluginInfo>());
 
 	int add_sources (std::vector<std::string>              paths,
@@ -1395,6 +1417,7 @@ private:
 	                 int                                   target_regions,
 	                 int                                   target_tracks,
 	                 boost::shared_ptr<ARDOUR::Track>&     track,
+	                 std::string const&                    pgroup_id,
 	                 bool                                  add_channel_suffix,
 	                 boost::shared_ptr<ARDOUR::PluginInfo> instrument = boost::shared_ptr<ARDOUR::PluginInfo>());
 
@@ -1404,7 +1427,8 @@ private:
 	                                 samplepos_t&                          pos,
 	                                 Editing::ImportMode                   mode,
 	                                 boost::shared_ptr<ARDOUR::Track>&     existing_track,
-	                                 const std::string&                    new_track_name,
+	                                 std::string const&                    new_track_name,
+	                                 std::string const&                    pgroup_id,
 	                                 boost::shared_ptr<ARDOUR::PluginInfo> instrument);
 
 	boost::shared_ptr<ARDOUR::AudioTrack> get_nth_selected_audio_track (int nth) const;
@@ -1453,6 +1477,7 @@ private:
 	void import_audio (bool as_tracks);
 	void do_import (std::vector<std::string> paths, bool split, bool as_tracks);
 	void import_smf_tempo_map (Evoral::SMF const &, samplepos_t pos);
+	void import_smf_markers (Evoral::SMF &, samplepos_t pos);
 	void move_to_start ();
 	void move_to_end ();
 	void center_playhead ();
@@ -1466,6 +1491,7 @@ private:
 	void scroll_tracks_up ();
 	void set_mark ();
 	void clear_markers ();
+	void clear_xrun_markers ();
 	void clear_ranges ();
 	void clear_locations ();
 	void unhide_markers ();
@@ -1512,7 +1538,12 @@ private:
 	bool select_new_marker;
 
 	void toggle_all_existing_automation ();
+
 	void toggle_layer_display ();
+	void layer_display_stacked ();
+	void layer_display_overlaid ();
+
+	void launch_playlist_selector ();
 
 	void reverse_selection ();
 	void edit_envelope ();
@@ -1589,8 +1620,6 @@ private:
 	bool can_remove_control_point (ArdourCanvas::Item*);
 	void remove_control_point (ArdourCanvas::Item*);
 
-	void mouse_brush_insert_region (RegionView*, samplepos_t pos);
-
 	/* Canvas event handlers */
 
 	bool canvas_scroll_event (GdkEventScroll* event, bool from_canvas);
@@ -1671,6 +1700,10 @@ private:
 
 	void initialize_canvas ();
 
+	/* playlist internal ops */
+
+	bool stamp_new_playlist (std::string title, std::string &name, std::string &pgroup, bool copy);
+
 	/* display control */
 
 	/// true if the editor should follow the playhead, otherwise false
@@ -1767,8 +1800,6 @@ private:
 
 	void redisplay_grid (bool immediate_redraw);
 
-	uint32_t bbt_beat_subdivision;
-
 	/* toolbar */
 
 	Gtk::ToggleButton editor_mixer_button;
@@ -1909,6 +1940,11 @@ private:
 
 	bool audio_region_selection_covers (samplepos_t where);
 
+	/* playhead and edit cursor */
+
+	EditorCursor* _playhead_cursor;
+	EditorCursor* _snapped_cursor;
+
 	/* transport range select process */
 
 	ArdourCanvas::Rectangle* cd_marker_bar_drag_rect;
@@ -2134,7 +2170,7 @@ private:
 	/* tracking step changes of track height */
 
 	TimeAxisView* current_stepping_trackview;
-	ARDOUR::microseconds_t last_track_height_step_timestamp;
+	PBD::microseconds_t last_track_height_step_timestamp;
 	gint track_height_step_timeout();
 	sigc::connection step_timeout;
 
@@ -2303,6 +2339,11 @@ private:
 	void follow_mixer_selection ();
 	bool _following_mixer_selection;
 
+	/* RTAV Automation display option */
+	void toggle_show_touched_automation ();
+	void set_show_touched_automation (bool);
+	bool _show_touched_automation;
+
 	int time_fx (ARDOUR::RegionList&, float val, bool pitching);
 	void note_edit_done (int, EditNoteDialog*);
 	void toggle_sound_midi_notes ();
@@ -2342,6 +2383,8 @@ private:
 	void toggle_reg_sens (Glib::RefPtr<Gtk::ActionGroup> group, char const* name, char const* label, sigc::slot<void> slot);
 	void radio_reg_sens (Glib::RefPtr<Gtk::ActionGroup> action_group, Gtk::RadioAction::Group& radio_group, char const* name, char const* label, sigc::slot<void> slot);
 
+	void remove_gap_marker_callback (samplepos_t at, samplecnt_t distance);
+
 	friend class Drag;
 	friend class RegionCutDrag;
 	friend class RegionDrag;
