@@ -313,6 +313,33 @@ Editor::import_smf_tempo_map (Evoral::SMF const & smf, samplepos_t pos)
 	_session->tempo_map() = new_map;
 }
 
+void
+Editor::import_smf_markers (Evoral::SMF & smf, samplepos_t pos)
+{
+	if (!_session) {
+		return;
+	}
+
+	smf.load_markers ();
+
+	Evoral::SMF::Markers const & markers = smf.markers();
+
+	if (markers.empty()) {
+		return;
+	}
+
+	/* XXX in nutempo2 just add location using Beats */
+
+	for (Evoral::SMF::Markers::const_iterator m = markers.begin(); m != markers.end(); ++m) {
+		// Temporal::Beats beat_pos (m->time_pulses / (double) smf.ppqn() / 4.0);
+		double beat = m->time_pulses / (double) smf.ppqn();
+		samplepos_t samplepos = pos + _session->tempo_map().sample_at_beat (beat);
+		Location* loc = new Location (*_session, samplepos, samplepos_t (0), m->text, Location::IsMark, 0);
+		_session->locations()->add (loc);
+	}
+
+}
+
 void
 Editor::do_import (vector<string>          paths,
                    ImportDisposition       disposition,
@@ -321,28 +348,52 @@ Editor::do_import (vector<string>          paths,
                    MidiTrackNameSource     midi_track_name_source,
                    MidiTempoMapDisposition smf_tempo_disposition,
                    samplepos_t&            pos,
-                   ARDOUR::PluginInfoPtr   instrument)
+                   ARDOUR::PluginInfoPtr   instrument,
+                   bool                    with_markers)
 {
 	boost::shared_ptr<Track> track;
 	vector<string> to_import;
 	int nth = 0;
 	bool use_timestamp = (pos == -1);
+	std::string const& pgroup_id = Playlist::generate_pgroup_id ();
 
-	if (smf_tempo_disposition == SMFTempoUse) {
-		/* Find the first MIDI file with a tempo map, and import it
-		   before we do anything else.
-		*/
+	/* XXX nutempo2: we will import markers using music (beat) time, which
+	   will make any imported tempo map irrelevant. Not doing that (in 6.7,
+	   before nutempo2) is much more complicated because we don't know
+	   which file may have the tempo map, and if we're importing that
+	   it will change the marker positions. So for now, there's an implicit
+	   limitation that if you import more than 1 MIDI file and the first
+	   has markers but the second has the tempo map, the markers could be
+	   in the wrong position.
+	*/
+
+	if (with_markers || (smf_tempo_disposition == SMFTempoUse)) {
+
+		bool tempo_map_done = false;
 
 		for (vector<string>::iterator a = paths.begin(); a != paths.end(); ++a) {
+
 			Evoral::SMF smf;
+
 			if (smf.open (*a)) {
 				continue;
 			}
-			if (smf.num_tempos() > 0) {
-				import_smf_tempo_map (smf, pos);
-				smf.close ();
-				break;
+
+			/* Find the first MIDI file with a tempo map, and import it
+			   before we do anything else.
+			*/
+			if (!tempo_map_done && smf_tempo_disposition == SMFTempoUse) {
+				if (smf.num_tempos() > 0) {
+					import_smf_tempo_map (smf, pos);
+					tempo_map_done = true;
+				}
 			}
+
+			if (with_markers) {
+
+				import_smf_markers (smf, pos);
+			}
+
 			smf.close ();
 		}
 	}
@@ -372,7 +423,7 @@ Editor::do_import (vector<string>          paths,
 
 		if (!cancel) {
 			ipw.show ();
-			import_sndfiles (paths, disposition, mode, quality, pos, 1, 1, track, false, instrument);
+			import_sndfiles (paths, disposition, mode, quality, pos, 1, 1, track, pgroup_id, false, instrument);
 			import_status.clear();
 		}
 
@@ -418,7 +469,7 @@ Editor::do_import (vector<string>          paths,
 					track = get_nth_selected_audio_track (nth++);
 				}
 
-				import_sndfiles (to_import, disposition, mode, quality, pos, 1, -1, track, replace, instrument);
+				import_sndfiles (to_import, disposition, mode, quality, pos, 1, -1, track, pgroup_id, replace, instrument);
 				import_status.clear();
 				break;
 
@@ -427,7 +478,7 @@ Editor::do_import (vector<string>          paths,
 				to_import.clear ();
 				to_import.push_back (*a);
 
-				import_sndfiles (to_import, disposition, mode, quality, pos, -1, -1, track, replace, instrument);
+				import_sndfiles (to_import, disposition, mode, quality, pos, -1, -1, track, pgroup_id, replace, instrument);
 				import_status.clear();
 				break;
 
@@ -436,7 +487,7 @@ Editor::do_import (vector<string>          paths,
 				to_import.clear ();
 				to_import.push_back (*a);
 
-				import_sndfiles (to_import, disposition, mode, quality, pos, 1, 1, track, replace, instrument);
+				import_sndfiles (to_import, disposition, mode, quality, pos, 1, 1, track, pgroup_id, replace, instrument);
 				import_status.clear();
 				break;
 
@@ -451,7 +502,11 @@ Editor::do_import (vector<string>          paths,
 }
 
 void
-Editor::do_embed (vector<string> paths, ImportDisposition import_as, ImportMode mode, samplepos_t& pos, ARDOUR::PluginInfoPtr instrument)
+Editor::do_embed (vector<string> paths,
+                  ImportDisposition import_as,
+                  ImportMode mode,
+                  samplepos_t& pos,
+                  ARDOUR::PluginInfoPtr instrument)
 {
 	boost::shared_ptr<Track> track;
 	bool check_sample_rate = true;
@@ -459,6 +514,7 @@ Editor::do_embed (vector<string> paths, ImportDisposition import_as, ImportMode
 	bool multi = paths.size() > 1;
 	int nth = 0;
 	bool use_timestamp = (pos == -1);
+	std::string const& pgroup_id = Playlist::generate_pgroup_id ();
 
 	switch (import_as) {
 	case Editing::ImportDistinctFiles:
@@ -476,7 +532,7 @@ Editor::do_embed (vector<string> paths, ImportDisposition import_as, ImportMode
 				track = get_nth_selected_audio_track (nth++);
 			}
 
-			if (embed_sndfiles (to_embed, multi, check_sample_rate, import_as, mode, pos, 1, -1, track, instrument) < -1) {
+			if (embed_sndfiles (to_embed, multi, check_sample_rate, import_as, mode, pos, 1, -1, track, pgroup_id, instrument) < -1) {
 				/* error, bail out */
 				return;
 			}
@@ -494,7 +550,7 @@ Editor::do_embed (vector<string> paths, ImportDisposition import_as, ImportMode
 			to_embed.clear ();
 			to_embed.push_back (*a);
 
-			if (embed_sndfiles (to_embed, multi, check_sample_rate, import_as, mode, pos, -1, -1, track, instrument) < -1) {
+			if (embed_sndfiles (to_embed, multi, check_sample_rate, import_as, mode, pos, -1, -1, track, pgroup_id, instrument) < -1) {
 				/* error, bail out */
 				return;
 			}
@@ -502,7 +558,7 @@ Editor::do_embed (vector<string> paths, ImportDisposition import_as, ImportMode
 		break;
 
 	case Editing::ImportMergeFiles:
-		if (embed_sndfiles (paths, multi, check_sample_rate, import_as, mode, pos, 1, 1, track, instrument) < -1) {
+		if (embed_sndfiles (paths, multi, check_sample_rate, import_as, mode, pos, 1, 1, track, pgroup_id, instrument) < -1) {
 			/* error, bail out */
 			return;
 		}
@@ -519,7 +575,7 @@ Editor::do_embed (vector<string> paths, ImportDisposition import_as, ImportMode
 			to_embed.clear ();
 			to_embed.push_back (*a);
 
-			if (embed_sndfiles (to_embed, multi, check_sample_rate, import_as, mode, pos, 1, 1, track, instrument) < -1) {
+			if (embed_sndfiles (to_embed, multi, check_sample_rate, import_as, mode, pos, 1, 1, track, pgroup_id, instrument) < -1) {
 				/* error, bail out */
 				return;
 			}
@@ -537,6 +593,7 @@ Editor::import_sndfiles (vector<string>            paths,
                          int                       target_regions,
                          int                       target_tracks,
                          boost::shared_ptr<Track>& track,
+                         std::string const&        pgroup_id,
                          bool                      replace,
                          ARDOUR::PluginInfoPtr     instrument)
 {
@@ -588,7 +645,7 @@ Editor::import_sndfiles (vector<string>            paths,
 			import_status.mode,
 			import_status.target_regions,
 			import_status.target_tracks,
-			track, false, instrument
+			track, pgroup_id, false, instrument
 			);
 
 		/* update position from results */
@@ -609,6 +666,7 @@ Editor::embed_sndfiles (vector<string>            paths,
                         int                       target_regions,
                         int                       target_tracks,
                         boost::shared_ptr<Track>& track,
+                        std::string const&        pgroup_id,
                         ARDOUR::PluginInfoPtr     instrument)
 {
 	boost::shared_ptr<AudioFileSource> source;
@@ -725,7 +783,7 @@ Editor::embed_sndfiles (vector<string>            paths,
 	}
 
 	if (!sources.empty()) {
-		return add_sources (paths, sources, pos, disposition, mode, target_regions, target_tracks, track, true, instrument);
+		return add_sources (paths, sources, pos, disposition, mode, target_regions, target_tracks, track, pgroup_id, true, instrument);
 	}
 
 	return 0;
@@ -740,6 +798,7 @@ Editor::add_sources (vector<string>            paths,
                      int                       target_regions,
                      int                       target_tracks,
                      boost::shared_ptr<Track>& track,
+                     std::string const&        pgroup_id,
                      bool                      /*add_channel_suffix*/,
                      ARDOUR::PluginInfoPtr     instrument)
 {
@@ -936,7 +995,7 @@ Editor::add_sources (vector<string>            paths,
 			}
 		}
 
-		finish_bringing_in_material (*r, input_chan, output_chan, pos, mode, track, track_names[n], instrument);
+		finish_bringing_in_material (*r, input_chan, output_chan, pos, mode, track, track_names[n], pgroup_id, instrument);
 
 		rlen = (*r)->length();
 
@@ -965,10 +1024,11 @@ int
 Editor::finish_bringing_in_material (boost::shared_ptr<Region> region,
                                      uint32_t                  in_chans,
                                      uint32_t                  out_chans,
-                                     samplepos_t&               pos,
+                                     samplepos_t&              pos,
                                      ImportMode                mode,
                                      boost::shared_ptr<Track>& existing_track,
-                                     const string&             new_track_name,
+                                     string const&             new_track_name,
+                                     string const&             pgroup_id,
                                      ARDOUR::PluginInfoPtr     instrument)
 {
 	boost::shared_ptr<AudioRegion> ar = boost::dynamic_pointer_cast<AudioRegion>(region);
@@ -997,10 +1057,17 @@ Editor::finish_bringing_in_material (boost::shared_ptr<Region> region,
 		boost::shared_ptr<Playlist> playlist = existing_track->playlist();
 		boost::shared_ptr<Region> copy (RegionFactory::create (region, region->properties()));
 		playlist->clear_changes ();
+		playlist->clear_owned_changes ();
 		playlist->add_region (copy, pos);
-		if (Config->get_edit_mode() == Ripple)
+		if (Config->get_edit_mode() == Ripple) {
 			playlist->ripple (pos, copy->length(), copy);
 
+			/* recusive diff of rippled regions */
+			vector<Command*> cmds;
+			playlist->rdiff (cmds);
+			_session->add_commands (cmds);
+		}
+
 		_session->add_command (new StatefulDiffCommand (playlist));
 		break;
 	}
@@ -1014,10 +1081,11 @@ Editor::finish_bringing_in_material (boost::shared_ptr<Region> region,
 				if (at.empty()) {
 					return -1;
 				}
-				if (Config->get_strict_io ()) {
-					for (list<boost::shared_ptr<AudioTrack> >::iterator i = at.begin(); i != at.end(); ++i) {
+				for (list<boost::shared_ptr<AudioTrack> >::iterator i = at.begin(); i != at.end(); ++i) {
+					if (Config->get_strict_io ()) {
 						(*i)->set_strict_io (true);
 					}
+					(*i)->playlist()->set_pgroup_id (pgroup_id);
 				}
 
 				existing_track = at.front();
@@ -1036,7 +1104,13 @@ Editor::finish_bringing_in_material (boost::shared_ptr<Region> region,
 					return -1;
 				}
 
-				// TODO set strict_io from preferences
+				for (list<boost::shared_ptr<MidiTrack> >::iterator i = mt.begin(); i != mt.end(); ++i) {
+					if (Config->get_strict_io ()) {
+						(*i)->set_strict_io (true);
+					}
+					(*i)->playlist()->set_pgroup_id (pgroup_id);
+				}
+
 				existing_track = mt.front();
 			}
 
