@@ -197,8 +197,8 @@ public:
 		return _preview_video;
 	}
 
-	/** @return minimum number of samples (in x) and pixels (in y) that should be considered a movement */
-	virtual std::pair<ARDOUR::samplecnt_t, int> move_threshold () const {
+	/** @return minimum number of pixels (x, y) that should be considered a movement */
+	virtual std::pair<int, int> move_threshold () const {
 		return std::make_pair (1, 1);
 	}
 
@@ -231,6 +231,8 @@ public:
 		_preview_video = false;
 	}
 
+	int grab_button() const { return _grab_button; }
+
 protected:
 
 	double grab_x () const {
@@ -309,6 +311,7 @@ private:
 	double      _snap_delta_music;
 	CursorContext::Handle _cursor_ctx; ///< cursor change context
 	bool _constraint_pressed; ///< if the keyboard indicated constraint modifier was pressed on start_grab()
+	int _grab_button;
 };
 
 class RegionDrag;
@@ -362,19 +365,52 @@ protected:
 
 	friend class DraggingView;
 
+protected:
+	typedef std::set<boost::shared_ptr<ARDOUR::Playlist> > PlaylistSet;
+	void add_stateful_diff_commands_for_playlists (PlaylistSet const &);
+
 private:
 
 	void region_going_away (RegionView *);
 	PBD::ScopedConnection death_connection;
 };
 
+/** Drag the Contents rather than the bounds of a region (i.e. Slip) */
+class RegionSlipContentsDrag : public RegionDrag
+{
+public:
+
+	RegionSlipContentsDrag (Editor *, ArdourCanvas::Item *, RegionView *, std::list<RegionView*> const &);
+	virtual ~RegionSlipContentsDrag () {}
+
+	virtual void start_grab (GdkEvent *, Gdk::Cursor *);
+	virtual void motion (GdkEvent *, bool);
+	virtual void finished (GdkEvent *, bool);
+	virtual void aborted (bool);
+};
+
+/** Brush on a region to repeat it */
+class RegionBrushDrag : public RegionDrag
+{
+public:
+	RegionBrushDrag (Editor *, ArdourCanvas::Item *, RegionView *, std::list<RegionView*> const &);
+	virtual ~RegionBrushDrag () {}
+
+	virtual void start_grab (GdkEvent *, Gdk::Cursor *);
+	virtual void motion (GdkEvent *, bool);
+	virtual void finished (GdkEvent *, bool);
+	virtual void aborted (bool);
+private:
+	typedef std::set<samplepos_t> SamplePositionSet;
+	SamplePositionSet _already_pasted;
+};
 
 /** Drags involving region motion from somewhere */
 class RegionMotionDrag : public RegionDrag
 {
 public:
 
-	RegionMotionDrag (Editor *, ArdourCanvas::Item *, RegionView *, std::list<RegionView*> const &, bool);
+	RegionMotionDrag (Editor *, ArdourCanvas::Item *, RegionView *, std::list<RegionView*> const &);
 	virtual ~RegionMotionDrag () {}
 
 	virtual void start_grab (GdkEvent *, Gdk::Cursor *);
@@ -392,7 +428,6 @@ protected:
 	double compute_x_delta (GdkEvent const *, ARDOUR::MusicSample *);
 	virtual bool y_movement_allowed (int, double, int skip_invisible = 0) const;
 
-	bool _brushing;
 	bool _ignore_video_lock;
 	ARDOUR::MusicSample _last_position; ///< last position of the thing being dragged
 	double _total_x_delta;
@@ -411,27 +446,29 @@ private:
 class RegionMoveDrag : public RegionMotionDrag
 {
 public:
-	RegionMoveDrag (Editor *, ArdourCanvas::Item *, RegionView *, std::list<RegionView*> const &, bool, bool);
+	RegionMoveDrag (Editor *, ArdourCanvas::Item *, RegionView *, std::list<RegionView*> const &, bool);
 	virtual ~RegionMoveDrag () {}
 
 	void motion (GdkEvent *, bool);
 	void finished (GdkEvent *, bool);
 	void aborted (bool);
 
+	void clear_draggingview_list ();
+
 	bool regions_came_from_canvas () const {
 		return true;
 	}
 
-	std::pair<ARDOUR::samplecnt_t, int> move_threshold () const {
-		return std::make_pair (4, 4);
+	std::pair<int, int> move_threshold () const {
+		if (_copy) {
+			return std::make_pair (6, 4);
+		} else {
+			return std::make_pair (2, 4);
+		}
 	}
 
 	void setup_pointer_sample_offset ();
 
-protected:
-	typedef std::set<boost::shared_ptr<ARDOUR::Playlist> > PlaylistSet;
-	void add_stateful_diff_commands_for_playlists (PlaylistSet const &);
-
 private:
 	void finished_no_copy (
 		bool const,
@@ -565,6 +602,10 @@ public:
 	void finished (GdkEvent *, bool);
 	void aborted (bool);
 
+	bool allow_vertical_autoscroll () const {
+		return false;
+	}
+
 private:
 	MidiRegionView*     region;
 	bool                relative;
@@ -585,6 +626,11 @@ public:
 	void aborted (bool);
 
 	void setup_pointer_sample_offset ();
+
+	bool allow_vertical_autoscroll () const {
+		return false;
+	}
+
 private:
 
 	double total_dx (GdkEvent * event) const; // total movement in quarter notes
@@ -615,6 +661,10 @@ public:
 		return mode == Editing::MouseDraw || mode == Editing::MouseContent;
 	}
 
+	bool allow_vertical_autoscroll () const {
+		return false;
+	}
+
 	bool y_movement_matters () const {
 		return false;
 	}
@@ -623,8 +673,8 @@ private:
 	double y_to_region (double) const;
 	ARDOUR::samplecnt_t grid_samples (samplepos_t) const;
 
-	/** @return minimum number of samples (in x) and pixels (in y) that should be considered a movement */
-	virtual std::pair<ARDOUR::samplecnt_t, int> move_threshold () const {
+	/** @return minimum number of pixels (x, y) that should be considered a movement */
+	virtual std::pair<int, int> move_threshold () const {
 		return std::make_pair (0, 0);
 	}
 
@@ -656,8 +706,8 @@ private:
 	double y_to_region (double) const;
 	ARDOUR::samplecnt_t grid_samples (samplepos_t) const;
 
-	/** @return minimum number of samples (in x) and pixels (in y) that should be considered a movement */
-	virtual std::pair<ARDOUR::samplecnt_t, int> move_threshold () const {
+	/** @return minimum number of pixels (x, y) that should be considered a movement */
+	virtual std::pair<int, int> move_threshold () const {
 		return std::make_pair (0, 0);
 	}
 
@@ -681,6 +731,10 @@ public:
 		return false;
 	}
 
+	bool allow_vertical_autoscroll () const {
+		return false;
+	}
+
 	void setup_pointer_sample_offset ();
 
 private:
@@ -734,8 +788,7 @@ class TrimDrag : public RegionDrag
 public:
 	enum Operation {
 		StartTrim,
-		EndTrim,
-		ContentsTrim,
+		EndTrim
 	};
 
 	TrimDrag (Editor *, ArdourCanvas::Item *, RegionView*, std::list<RegionView*> const &, bool preserve_fade_anchor = false);
@@ -749,6 +802,10 @@ public:
 		return false;
 	}
 
+	bool allow_vertical_autoscroll () const {
+		return false;
+	}
+
 	void setup_pointer_sample_offset ();
 
 private:
@@ -957,6 +1014,10 @@ public:
 		return false;
 	}
 
+	bool allow_vertical_autoscroll () const {
+		return false;
+	}
+
 	void setup_pointer_sample_offset ();
 };
 
@@ -975,6 +1036,10 @@ public:
 		return false;
 	}
 
+	bool allow_vertical_autoscroll () const {
+		return false;
+	}
+
 	void setup_pointer_sample_offset ();
 };
 
@@ -1031,6 +1096,11 @@ public:
 
 	bool active (Editing::MouseMode m);
 
+	bool allow_vertical_autoscroll () const {
+		return false;
+	}
+
+
 private:
 
 	ControlPoint* _point;
@@ -1054,6 +1124,10 @@ public:
 	void finished (GdkEvent *, bool);
 	void aborted (bool);
 
+	bool allow_vertical_autoscroll () const {
+		return false;
+	}
+
 private:
 
 	AutomationLine* _line;
@@ -1098,8 +1172,9 @@ public:
 	void finished (GdkEvent *, bool);
 	void aborted (bool);
 
-	std::pair<ARDOUR::samplecnt_t, int> move_threshold () const {
-		return std::make_pair (8, 1);
+	/** @return minimum number of pixels (x, y) that should be considered a movement */
+	std::pair<int, int> move_threshold () const {
+		return std::make_pair (1, 1);
 	}
 
 	void do_select_things (GdkEvent *, bool);
@@ -1256,7 +1331,8 @@ public:
 	void finished (GdkEvent *, bool);
 	void aborted (bool);
 
-	std::pair<ARDOUR::samplecnt_t, int> move_threshold () const {
+	/** @return minimum number of pixels (x, y) that should be considered a movement */
+	std::pair<int, int> move_threshold () const {
 		return std::make_pair (4, 4);
 	}
 
@@ -1320,8 +1396,8 @@ public:
 		return false;
 	}
 
-	virtual std::pair<ARDOUR::samplecnt_t, int> move_threshold () const {
-		return std::make_pair (4, 4);
+	virtual std::pair<int, int> move_threshold () const {
+		return std::make_pair (1, 4);
 	}
 
 private:
@@ -1329,4 +1405,32 @@ private:
 	bool start;
 };
 
+class RegionMarkerDrag : public Drag
+{
+  public:
+	RegionMarkerDrag (Editor*, RegionView*, ArdourCanvas::Item*);
+	~RegionMarkerDrag ();
+
+	void start_grab (GdkEvent *, Gdk::Cursor* c = 0);
+	void motion (GdkEvent *, bool);
+	void finished (GdkEvent *, bool);
+	void aborted (bool);
+
+	bool allow_vertical_autoscroll () const {
+		return false;
+	}
+
+	bool y_movement_matters () const {
+		return false;
+	}
+
+	void setup_pointer_sample_offset ();
+
+  private:
+	RegionView* rv;
+	ArdourMarker* view;
+	ARDOUR::CueMarker model;
+	ARDOUR::CueMarker dragging_model;
+};
+
 #endif /* __gtk2_ardour_editor_drag_h_ */
