@@ -81,6 +81,7 @@
 #include "edit_note_dialog.h"
 #include "mouse_cursors.h"
 #include "editor_cursors.h"
+#include "region_peak_cursor.h"
 #include "verbose_cursor.h"
 #include "note.h"
 
@@ -312,7 +313,8 @@ Editor::mouse_mode_toggled (MouseMode m)
 	if (_session && mouse_mode == MouseAudition) {
 		/* stop transport and reset default speed to avoid oddness with
 		   auditioning */
-		_session->request_transport_speed (0.0, true);
+		_session->request_stop ();
+		_session->reset_transport_speed ();
 	}
 
 	const bool was_internal = internal_editing();
@@ -742,14 +744,19 @@ Editor::button_press_handler_1 (ArdourCanvas::Item* item, GdkEvent* event, ItemT
 
 	switch (item_type) {
 	case PlayheadCursorItem:
-		_drags->set (new CursorDrag (this, *playhead_cursor, true), event);
+		_drags->set (new CursorDrag (this, *_playhead_cursor, true), event);
 		return true;
 
 	case MarkerItem:
 		if (Keyboard::modifier_state_equals (event->button.state, Keyboard::ModifierMask(Keyboard::PrimaryModifier|Keyboard::TertiaryModifier))) {
 			hide_marker (item, event);
 		} else {
-			_drags->set (new MarkerDrag (this, item), event);
+			ArdourMarker* marker = static_cast<ArdourMarker*> (item->get_data ("marker"));
+			if (marker->type() == ArdourMarker::RegionCue) {
+				_drags->set (new RegionMarkerDrag (this, marker->region_view(), item), event);
+			} else {
+				_drags->set (new MarkerDrag (this, item), event);
+			}
 		}
 		return true;
 
@@ -805,7 +812,7 @@ Editor::button_press_handler_1 (ArdourCanvas::Item* item, GdkEvent* event, ItemT
 	case BBTRulerItem:
 		if (!Keyboard::modifier_state_equals (event->button.state, Keyboard::PrimaryModifier)
 		    && !ArdourKeyboard::indicates_constraint (event->button.state)) {
-			_drags->set (new CursorDrag (this, *playhead_cursor, false), event);
+			_drags->set (new CursorDrag (this, *_playhead_cursor, false), event);
 		} else if (ArdourKeyboard::indicates_constraint (event->button.state)
 		           && Keyboard::modifier_state_contains (event->button.state, Keyboard::PrimaryModifier)) {
 			_drags->set (new TempoTwistDrag (this, item), event);
@@ -822,14 +829,14 @@ Editor::button_press_handler_1 (ArdourCanvas::Item* item, GdkEvent* event, ItemT
 		} else if (Keyboard::modifier_state_equals (event->button.state, Keyboard::PrimaryModifier)) {
 			_drags->set (new RangeMarkerBarDrag (this, item, RangeMarkerBarDrag::CreateRangeMarker), event);
 		} else {
-			_drags->set (new CursorDrag (this, *playhead_cursor, false), event);
+			_drags->set (new CursorDrag (this, *_playhead_cursor, false), event);
 		}
 		return true;
 		break;
 
 	case CdMarkerBarItem:
 		if (!Keyboard::modifier_state_equals (event->button.state, Keyboard::PrimaryModifier)) {
-			_drags->set (new CursorDrag (this, *playhead_cursor, false), event);
+			_drags->set (new CursorDrag (this, *_playhead_cursor, false), event);
 		} else {
 			_drags->set (new RangeMarkerBarDrag (this, item, RangeMarkerBarDrag::CreateCDMarker), event);
 		}
@@ -838,7 +845,7 @@ Editor::button_press_handler_1 (ArdourCanvas::Item* item, GdkEvent* event, ItemT
 
 	case TransportMarkerBarItem:
 		if (!Keyboard::modifier_state_equals (event->button.state, Keyboard::PrimaryModifier)) {
-			_drags->set (new CursorDrag (this, *playhead_cursor, false), event);
+			_drags->set (new CursorDrag (this, *_playhead_cursor, false), event);
 		} else {
 			_drags->set (new RangeMarkerBarDrag (this, item, RangeMarkerBarDrag::CreateTransportMarker), event);
 		}
@@ -1054,7 +1061,11 @@ Editor::button_press_handler_1 (ArdourCanvas::Item* item, GdkEvent* event, ItemT
 				}
 
 				/* click on a normal region view */
-				if (ArdourKeyboard::indicates_copy (event->button.state)) {
+				if (Keyboard::modifier_state_equals (event->button.state, ArdourKeyboard::slip_contents_modifier ())) {
+					if (!clicked_regionview->region()->locked() && (Config->get_edit_mode() != Lock)) {
+						_drags->add (new RegionSlipContentsDrag (this, item, clicked_regionview, selection->regions.by_layer()));
+					}
+				} else if (ArdourKeyboard::indicates_copy (event->button.state)) {
 					add_region_copy_drag (item, event, clicked_regionview);
 				} else if (Keyboard::the_keyboard().key_is_down (GDK_b)) {
 					add_region_brush_drag (item, event, clicked_regionview);
@@ -1667,7 +1678,7 @@ Editor::button_release_handler (ArdourCanvas::Item* item, GdkEvent* event, ItemT
 			break;
 
 		case MarkerItem:
-			remove_marker (*item, event);
+			remove_marker (*item);
 			break;
 
 		case RegionItem:
@@ -1787,7 +1798,7 @@ Editor::button_release_handler (ArdourCanvas::Item* item, GdkEvent* event, ItemT
 				}
 			} else if (_session) {
 				/* make sure we stop */
-				_session->request_transport_speed (0.0);
+				_session->request_stop ();
 			}
 			break;
 
@@ -1912,7 +1923,7 @@ Editor::enter_handler (ArdourCanvas::Item* item, GdkEvent* event, ItemType item_
 			break;
 		}
 		entered_marker = marker;
-		marker->set_color_rgba (UIConfiguration::instance().color ("entered marker"));
+		marker->set_entered (true);
 		break;
 
 	case MeterMarkerItem:
@@ -2013,8 +2024,6 @@ Editor::leave_handler (ArdourCanvas::Item* item, GdkEvent*, ItemType item_type)
 	ArdourMarker *marker;
 	TempoMarker *t_marker;
 	MeterMarker *m_marker;
-	Location *loc;
-	bool is_start;
 	bool ret = true;
 
 	if (!_enter_stack.empty()) {
@@ -2042,9 +2051,7 @@ Editor::leave_handler (ArdourCanvas::Item* item, GdkEvent*, ItemType item_type)
 			break;
 		}
 		entered_marker = 0;
-		if ((loc = find_location_from_marker (marker, is_start)) != 0) {
-			location_flags_changed (loc);
-		}
+		marker->set_entered (false);
 		break;
 
 	case MeterMarkerItem:
@@ -2132,7 +2139,7 @@ Editor::scrub (samplepos_t sample, double current_x)
 				scrub_reverse_distance = 0;
 
 				delta = 0.01 * (last_scrub_x - current_x);
-				_session->request_transport_speed_nonzero (_session->transport_speed() - delta);
+				_session->request_transport_speed_nonzero (_session->actual_speed() - delta);
 			}
 
 		} else {
@@ -2151,7 +2158,7 @@ Editor::scrub (samplepos_t sample, double current_x)
 				scrub_reverse_distance = 0;
 
 				delta = 0.01 * (current_x - last_scrub_x);
-				_session->request_transport_speed_nonzero (_session->transport_speed() + delta);
+				_session->request_transport_speed_nonzero (_session->actual_speed() + delta);
 			}
 		}
 
@@ -2216,13 +2223,28 @@ Editor::motion_handler (ArdourCanvas::Item* /*item*/, GdkEvent* event, bool from
 		//drags change the snapped_cursor location, because we are snapping the thing being dragged, not the actual mouse cursor
 		return _drags->motion_handler (event, from_autoscroll);
 	} else {
-		//the snapped_cursor shows where an operation (like Split) is going to occur
 		bool ignored;
+		bool peaks_visible = false;
 		MusicSample where (0, 0);
 		if (mouse_sample (where.sample, ignored)) {
+
+			/* display peaks */
+			if (mouse_mode == MouseContent || ArdourKeyboard::indicates_snap (event->motion.state)) {
+				AudioRegionView* arv = dynamic_cast<AudioRegionView*>(entered_regionview);
+				if (arv) {
+					_region_peak_cursor->set (arv, where.sample, samples_per_pixel);
+					peaks_visible = true;
+				}
+			}
+
+			/* the snapped_cursor shows where an operation (like Split) is going to occur */
 			snap_to_with_modifier (where, event);
 			set_snapped_cursor_position (where.sample);
 		}
+
+		if (!peaks_visible) {
+			_region_peak_cursor->hide ();
+		}
 	}
 
 	return false;
@@ -2564,7 +2586,7 @@ Editor::add_region_drag (ArdourCanvas::Item* item, GdkEvent*, RegionView* region
 			_drags->add (new RegionRippleDrag (this, item, region_view, selection->regions.by_layer()));
 			break;
 		default:
-			_drags->add (new RegionMoveDrag (this, item, region_view, selection->regions.by_layer(), false, false));
+			_drags->add (new RegionMoveDrag (this, item, region_view, selection->regions.by_layer(), false));
 			break;
 
 	}
@@ -2579,7 +2601,7 @@ Editor::add_region_copy_drag (ArdourCanvas::Item* item, GdkEvent*, RegionView* r
 		return;
 	}
 
-	_drags->add (new RegionMoveDrag (this, item, region_view, selection->regions.by_layer(), false, true));
+	_drags->add (new RegionMoveDrag (this, item, region_view, selection->regions.by_layer(), true));
 }
 
 void
@@ -2595,7 +2617,8 @@ Editor::add_region_brush_drag (ArdourCanvas::Item* item, GdkEvent*, RegionView*
 		return;
 	}
 
-	_drags->add (new RegionMoveDrag (this, item, region_view, selection->regions.by_layer(), true, false));
+	std::list<RegionView*> empty;
+	_drags->add (new RegionBrushDrag (this, item, region_view, empty));
 }
 
 /** Start a grab where a time range is selected, track(s) are selected, and the
@@ -2659,7 +2682,7 @@ Editor::start_selection_grab (ArdourCanvas::Item* /*item*/, GdkEvent* event)
 
 	commit_reversible_command ();
 
-	_drags->set (new RegionMoveDrag (this, latest_regionviews.front()->get_canvas_group(), latest_regionviews.front(), latest_regionviews, false, false), event);
+	_drags->set (new RegionMoveDrag (this, latest_regionviews.front()->get_canvas_group(), latest_regionviews.front(), latest_regionviews, false), event);
 }
 
 void
