@@ -25,7 +25,6 @@
 #include <algorithm>
 #include <cstdlib>
 
-#include "pbd/stacktrace.h"
 #include "pbd/unwind.h"
 
 #include "ardour/control_protocol_manager.h"
@@ -222,7 +221,7 @@ Editor::set_selected_track_as_side_effect (Selection::Operation op)
 	switch (op) {
 	case Selection::Toggle:
 		if (selection->selected (clicked_axisview)) {
-			if (group && group->is_active()) {
+			if (group && group->is_active() && group->enabled_property(ARDOUR::Properties::group_select.property_id)) {
 				for (TrackViewList::iterator i = track_views.begin(); i != track_views.end (); ++i) {
 					if ((*i)->route_group() == group) {
 						selection->remove(*i);
@@ -232,7 +231,7 @@ Editor::set_selected_track_as_side_effect (Selection::Operation op)
 				selection->remove (clicked_axisview);
 			}
 		} else {
-			if (group && group->is_active()) {
+			if (group && group->is_active() && group->enabled_property(ARDOUR::Properties::group_select.property_id)) {
 				for (TrackViewList::iterator i = track_views.begin(); i != track_views.end (); ++i) {
 					if ((*i)->route_group() == group) {
 						selection->add(*i);
@@ -245,7 +244,7 @@ Editor::set_selected_track_as_side_effect (Selection::Operation op)
 		break;
 
 	case Selection::Add:
-		if (group && group->is_active()) {
+		if (group && group->is_active() && group->enabled_property(ARDOUR::Properties::group_select.property_id)) {
 			for (TrackViewList::iterator i  = track_views.begin(); i != track_views.end (); ++i) {
 				if ((*i)->route_group() == group) {
 					selection->add(*i);
@@ -258,7 +257,7 @@ Editor::set_selected_track_as_side_effect (Selection::Operation op)
 
 	case Selection::Set:
 		selection->clear();
-		if (group && group->is_active()) {
+		if (group && group->is_active() && group->enabled_property(ARDOUR::Properties::group_select.property_id)) {
 			for (TrackViewList::iterator i  = track_views.begin(); i != track_views.end (); ++i) {
 				if ((*i)->route_group() == group) {
 					selection->add(*i);
@@ -407,37 +406,77 @@ Editor::get_onscreen_tracks (TrackViewList& tvl)
  */
 
 void
-Editor::mapover_tracks (sigc::slot<void, RouteTimeAxisView&, uint32_t> sl, TimeAxisView* basis, PBD::PropertyID prop) const
+Editor::mapover_grouped_routes (sigc::slot<void, RouteUI&> sl, RouteUI* basis, PBD::PropertyID prop) const
 {
-	RouteTimeAxisView* route_basis = dynamic_cast<RouteTimeAxisView*> (basis);
+	set<RouteUI*> routes;
 
-	if (route_basis == 0) {
-		return;
-	}
+	routes.insert(basis);
 
-	set<RouteTimeAxisView*> tracks;
-	tracks.insert (route_basis);
-
-	RouteGroup* group = route_basis->route()->route_group();
+	RouteGroup* group = basis->route()->route_group();
 
 	if (group && group->enabled_property(prop) && group->enabled_property (Properties::active.property_id)) {
 
 		/* the basis is a member of an active route group, with the appropriate
-		   properties; find other members */
+		 * properties; find other members */
 
 		for (TrackViewList::const_iterator i = track_views.begin(); i != track_views.end(); ++i) {
-			RouteTimeAxisView* v = dynamic_cast<RouteTimeAxisView*> (*i);
-			if (v && v->route()->route_group() == group) {
-				tracks.insert (v);
+			RouteUI* v = dynamic_cast<RouteUI*> (*i);
+			if ( v && (v->route() != basis->route()) && v->route()->route_group() == group) {
+				routes.insert (v);
 			}
 		}
 	}
 
 	/* call the slots */
-	uint32_t const sz = tracks.size ();
+	for (set<RouteUI*>::iterator i = routes.begin(); i != routes.end(); ++i) {
+		sl (**i);
+	}
+}
 
-	for (set<RouteTimeAxisView*>::iterator i = tracks.begin(); i != tracks.end(); ++i) {
-		sl (**i, sz);
+void
+Editor::mapover_armed_routes (sigc::slot<void, RouteUI&> sl) const
+{
+	set<RouteUI*> routes;
+	for (TrackViewList::const_iterator i = track_views.begin(); i != track_views.end(); ++i) {
+		RouteUI* v = dynamic_cast<RouteUI*> (*i);
+		if (v && v->route()->is_track()) {
+			if ( v->track()->rec_enable_control()->get_value()) {
+				routes.insert (v);
+			}
+		}
+	}
+	for (set<RouteUI*>::iterator i = routes.begin(); i != routes.end(); ++i) {
+		sl (**i);
+	}
+}
+
+void
+Editor::mapover_selected_routes (sigc::slot<void, RouteUI&> sl) const
+{
+	set<RouteUI*> routes;
+	for (TrackSelection::iterator i = selection->tracks.begin(); i != selection->tracks.end(); ++i) {
+		RouteTimeAxisView* r = dynamic_cast<RouteTimeAxisView*> (*i);
+		if (r) {
+			routes.insert (r);
+		}
+	}
+	for (set<RouteUI*>::iterator i = routes.begin(); i != routes.end(); ++i) {
+		sl (**i);
+	}
+}
+
+void
+Editor::mapover_all_routes (sigc::slot<void, RouteUI&> sl) const
+{
+	set<RouteUI*> routes;
+	for (TrackViewList::const_iterator i = track_views.begin(); i != track_views.end(); ++i) {
+		RouteTimeAxisView* r = dynamic_cast<RouteTimeAxisView*> (*i);
+		if (r) {
+			routes.insert (r);
+		}
+	}
+	for (set<RouteUI*>::iterator i = routes.begin(); i != routes.end(); ++i) {
+		sl (**i);
 	}
 }
 
@@ -560,44 +599,6 @@ Editor::get_equivalent_regions (RegionSelection & basis, PBD::PropertyID prop) c
 	return equivalent;
 }
 
-int
-Editor::get_regionview_count_from_region_list (boost::shared_ptr<Region> region)
-{
-	int region_count = 0;
-
-	for (TrackViewList::iterator i = track_views.begin(); i != track_views.end(); ++i) {
-
-		RouteTimeAxisView* tatv;
-
-		if ((tatv = dynamic_cast<RouteTimeAxisView*> (*i)) != 0) {
-
-			boost::shared_ptr<Playlist> pl;
-			vector<boost::shared_ptr<Region> > results;
-			RegionView* marv;
-			boost::shared_ptr<Track> tr;
-
-			if ((tr = tatv->track()) == 0) {
-				/* bus */
-				continue;
-			}
-
-			if ((pl = (tr->playlist())) != 0) {
-				pl->get_region_list_equivalent_regions (region, results);
-			}
-
-			for (vector<boost::shared_ptr<Region> >::iterator ir = results.begin(); ir != results.end(); ++ir) {
-				if ((marv = tatv->view()->find_view (*ir)) != 0) {
-					region_count++;
-				}
-			}
-
-		}
-	}
-
-	return region_count;
-}
-
-
 bool
 Editor::set_selected_regionview_from_click (bool press, Selection::Operation op)
 {
@@ -958,11 +959,11 @@ Editor::set_selection (std::list<Selectable*> s, Selection::Operation op)
 void
 Editor::set_selected_regionview_from_region_list (boost::shared_ptr<Region> region, Selection::Operation op)
 {
-	vector<RegionView*> all_equivalent_regions;
+	vector<RegionView*> regionviews;
 
-	get_regions_corresponding_to (region, all_equivalent_regions, region->whole_file());
+	get_regionview_corresponding_to (region, regionviews);
 
-	if (all_equivalent_regions.empty()) {
+	if (regionviews.empty()) {
 		return;
 	}
 
@@ -971,16 +972,16 @@ Editor::set_selected_regionview_from_region_list (boost::shared_ptr<Region> regi
 	switch (op) {
 	case Selection::Toggle:
 		/* XXX this is not correct */
-		selection->toggle (all_equivalent_regions);
+		selection->toggle (regionviews);
 		break;
 	case Selection::Set:
-		selection->set (all_equivalent_regions);
+		selection->set (regionviews);
 		break;
 	case Selection::Extend:
-		selection->add (all_equivalent_regions);
+		selection->add (regionviews);
 		break;
 	case Selection::Add:
-		selection->add (all_equivalent_regions);
+		selection->add (regionviews);
 		break;
 	}
 
@@ -1689,7 +1690,9 @@ Editor::select_all_internal_edit (Selection::Operation)
 {
 	bool selected = false;
 
-	for (RegionSelection::iterator i = selection->regions.begin(); i != selection->regions.end(); ++i) {
+	RegionSelection copy (selection->regions);
+
+	for (RegionSelection::iterator i = copy.begin(); i != copy.end(); ++i) {
 		MidiRegionView* mrv = dynamic_cast<MidiRegionView*>(*i);
 		if (mrv) {
 			mrv->select_all_notes ();
