@@ -150,8 +150,8 @@ MidiListEditor::MidiListEditor (Session* s, boost::shared_ptr<MidiRegion> r, boo
 
 	region->midi_source(0)->model()->ContentsChanged.connect (content_connections, invalidator (*this),
 	                                                          boost::bind (&MidiListEditor::redisplay_model, this), gui_context());
-	region->RegionPropertyChanged.connect (content_connections, invalidator (*this),
-	                                       boost::bind (&MidiListEditor::redisplay_model, this), gui_context());
+	region->PropertyChanged.connect (content_connections, invalidator (*this),
+	                                 boost::bind (&MidiListEditor::redisplay_model, this), gui_context());
 
 	buttons.attach (sound_notes_button, 0, 1, 0, 1);
 	Glib::RefPtr<Gtk::Action> act = ActionManager::get_action ("Editor", "sound-midi-notes");
