@@ -168,8 +168,6 @@ MidiTimeAxisView::set_route (boost::shared_ptr<Route> rt)
 
 	_view->apply_color (ARDOUR_UI_UTILS::gdk_color_to_rgba (color()), StreamView::RegionColor);
 
-	subplugin_menu.set_name ("ArdourContextMenu");
-
 	_note_range_changed_connection.disconnect();
 
 	if (!gui_property ("note-range-min").empty ()) {
@@ -181,8 +179,6 @@ MidiTimeAxisView::set_route (boost::shared_ptr<Route> rt)
 	_view->ContentsHeightChanged.connect (
 		sigc::mem_fun (*this, &MidiTimeAxisView::contents_height_changed));
 
-	ignore_toggle = false;
-
 	if (is_midi_track()) {
 		_note_mode = midi_track()->note_mode();
 	}
@@ -639,11 +635,22 @@ MidiTimeAxisView::append_extra_display_menu_items ()
 	items.push_back (MenuElem (_("Patch Selector..."),
 				sigc::mem_fun(*this, &RouteUI::select_midi_patch)));
 
+	items.push_back (CheckMenuElem (_("Restore Patch")));
+	Gtk::CheckMenuItem* cmi = dynamic_cast<Gtk::CheckMenuItem *> (&items.back());
+	cmi->set_active (midi_track ()->restore_pgm_on_load ());
+	cmi->signal_activate().connect (sigc::mem_fun (*this, &MidiTimeAxisView::toggle_restore_pgm_on_load));
+
 	items.push_back (MenuElem (_("Color Mode"), *build_color_mode_menu ()));
 
 	items.push_back (SeparatorElem ());
 }
 
+void
+MidiTimeAxisView::toggle_restore_pgm_on_load ()
+{
+	midi_track ()->set_restore_pgm_on_load (!midi_track ()->restore_pgm_on_load ());
+}
+
 void
 MidiTimeAxisView::toggle_channel_selector ()
 {
