@@ -40,10 +40,10 @@
 #include "pbd/convert.h"
 #include "pbd/enumwriter.h"
 #include "pbd/replace_all.h"
-#include "pbd/stacktrace.h"
 #include "pbd/unwind.h"
 
 #include "ardour/amp.h"
+#include "ardour/async_midi_port.h"
 #include "ardour/audio_track.h"
 #include "ardour/audioengine.h"
 #include "ardour/internal_send.h"
@@ -117,6 +117,8 @@ MixerStrip::MixerStrip (Mixer_UI& mx, Session* sess, bool in_mixer)
 	, mute_solo_table (1, 2)
 	, master_volume_table (2, 2)
 	, bottom_button_table (1, 3)
+	, input_button (true)
+	, output_button (false)
 	, monitor_section_button (0)
 	, midi_input_enable_button (0)
 	, _plugin_insert_cnt (0)
@@ -153,6 +155,8 @@ MixerStrip::MixerStrip (Mixer_UI& mx, Session* sess, boost::shared_ptr<Route> rt
 	, mute_solo_table (1, 2)
 	, master_volume_table (1, 2)
 	, bottom_button_table (1, 3)
+	, input_button (true)
+	, output_button (false)
 	, monitor_section_button (0)
 	, midi_input_enable_button (0)
 	, _plugin_insert_cnt (0)
@@ -174,9 +178,6 @@ MixerStrip::init ()
 	_entered_mixer_strip= 0;
 	group_menu = 0;
 	route_ops_menu = 0;
-	ignore_comment_edit = false;
-	ignore_toggle = false;
-	comment_area = 0;
 	_width_owner = 0;
 
 	/* the length of this string determines the width of the mixer strip when it is set to `wide' */
@@ -196,14 +197,8 @@ MixerStrip::init ()
 	set_tooltip (&hide_button, _("Hide this mixer strip"));
 
 	input_button_box.set_spacing(2);
-
-	input_button.set_text (_("Input"));
-	input_button.set_name ("mixer strip button");
 	input_button_box.pack_start (input_button, true, true);
 
-	output_button.set_text (_("Output"));
-	output_button.set_name ("mixer strip button");
-
 	bottom_button_table.attach (gpm.meter_point_button, 2, 3, 0, 1);
 
 	hide_button.set_events (hide_button.get_events() & ~(Gdk::ENTER_NOTIFY_MASK|Gdk::LEAVE_NOTIFY_MASK));
@@ -325,7 +320,7 @@ MixerStrip::init ()
 	global_vpacker.pack_start (width_hide_box, Gtk::PACK_SHRINK);
 	global_vpacker.pack_start (name_button, Gtk::PACK_SHRINK);
 	global_vpacker.pack_start (input_button_box, Gtk::PACK_SHRINK);
-	global_vpacker.pack_start (_invert_button_box, Gtk::PACK_SHRINK);
+	global_vpacker.pack_start (invert_button_box, Gtk::PACK_SHRINK);
 	global_vpacker.pack_start (processor_box, true, true);
 	global_vpacker.pack_start (panners, Gtk::PACK_SHRINK);
 	global_vpacker.pack_start (rec_mon_table, Gtk::PACK_SHRINK);
@@ -351,6 +346,7 @@ MixerStrip::init ()
 		scrollbar.ensure_style();
 		Gtk::Requisition requisition(scrollbar.size_request ());
 		scrollbar_height = requisition.height;
+		scrollbar_height += 3; // track_display_frame border/shadow
 	}
 	spacer.set_size_request (-1, scrollbar_height);
 	global_vpacker.pack_end (spacer, false, false);
@@ -370,17 +366,6 @@ MixerStrip::init ()
 	_packed = false;
 	_embedded = false;
 
-	input_button.signal_button_press_event().connect (sigc::mem_fun(*this, &MixerStrip::input_press), false);
-	input_button.signal_button_release_event().connect (sigc::mem_fun(*this, &MixerStrip::input_release), false);
-	input_button.signal_size_allocate().connect (sigc::mem_fun (*this, &MixerStrip::input_button_resized));
-
-	input_button.set_text_ellipsize (Pango::ELLIPSIZE_MIDDLE);
-	output_button.set_text_ellipsize (Pango::ELLIPSIZE_MIDDLE);
-
-	output_button.signal_button_press_event().connect (sigc::mem_fun(*this, &MixerStrip::output_press), false);
-	output_button.signal_button_release_event().connect (sigc::mem_fun(*this, &MixerStrip::output_release), false);
-	output_button.signal_size_allocate().connect (sigc::mem_fun (*this, &MixerStrip::output_button_resized));
-
 	number_label.signal_button_press_event().connect (sigc::mem_fun(*this, &MixerStrip::number_button_button_press), false);
 
 	name_button.signal_button_press_event().connect (sigc::mem_fun(*this, &MixerStrip::name_button_button_press), false);
@@ -389,14 +374,6 @@ MixerStrip::init ()
 
 	_width = (Width) -1;
 
-	/* start off as a passthru strip. we'll correct this, if necessary,
-	   in update_diskstream_display().
-	*/
-
-	/* start off as a passthru strip. we'll correct this, if necessary,
-	   in update_diskstream_display().
-	*/
-
 	if (is_midi_track()) {
 		set_name ("MidiTrackStripBase");
 	} else {
@@ -411,16 +388,12 @@ MixerStrip::init ()
 
 	set_flags (get_flags() | Gtk::CAN_FOCUS);
 
-	AudioEngine::instance()->PortConnectedOrDisconnected.connect (
-		*this, invalidator (*this), boost::bind (&MixerStrip::port_connected_or_disconnected, this, _1, _3), gui_context ()
-		);
-
 	/* Add the widgets under visibility control to the VisibilityGroup; the names used here
 	   must be the same as those used in RCOptionEditor so that the configuration changes
 	   are recognised when they occur.
 	*/
 	_visibility.add (&input_button_box, X_("Input"), _("Input"), false);
-	_visibility.add (&_invert_button_box, X_("PhaseInvert"), _("Phase Invert"), false);
+	_visibility.add (&invert_button_box, X_("PhaseInvert"), _("Phase Invert"), false);
 	_visibility.add (&rec_mon_table, X_("RecMon"), _("Record & Monitor"), false, boost::bind (&MixerStrip::override_rec_mon_visibility, this));
 	_visibility.add (&solo_iso_table, X_("SoloIsoLock"), _("Solo Iso / Lock"), false);
 	_visibility.add (&output_button, X_("Output"), _("Output"), false);
@@ -724,6 +697,9 @@ MixerStrip::set_route (boost::shared_ptr<Route> rt)
 		}
 	}
 
+	input_button.set_route (route (), this);
+	output_button.set_route (route (), this);
+
 	gpm.meter_point_button.set_text (meter_point_string (_route->meter_point()));
 
 	delete route_ops_menu;
@@ -761,7 +737,7 @@ MixerStrip::set_route (boost::shared_ptr<Route> rt)
 		panners.hide_all ();
 	}
 
-	update_diskstream_display ();
+	route_color_changed ();
 	update_input_display ();
 	update_output_display ();
 
@@ -889,329 +865,6 @@ MixerStrip::set_packed (bool yn)
 	set_gui_property ("visible", _packed);
 }
 
-
-struct RouteCompareByName {
-	bool operator() (boost::shared_ptr<Route> a, boost::shared_ptr<Route> b) {
-		return a->name().compare (b->name()) < 0;
-	}
-};
-
-gint
-MixerStrip::output_release (GdkEventButton *ev)
-{
-	switch (ev->button) {
-	case 3:
-		edit_output_configuration ();
-		break;
-	}
-
-	return false;
-}
-
-gint
-MixerStrip::output_press (GdkEventButton *ev)
-{
-	using namespace Menu_Helpers;
-	if (!ARDOUR_UI_UTILS::engine_is_running ()) {
-		return true;
-	}
-
-	MenuList& citems = output_menu.items();
-	switch (ev->button) {
-
-	case 3:
-		return false;  //wait for the mouse-up to pop the dialog
-
-	case 1:
-	{
-		output_menu.set_name ("ArdourContextMenu");
-		citems.clear ();
-		output_menu_bundles.clear ();
-
-		citems.push_back (MenuElem (_("Disconnect"), sigc::mem_fun (*(static_cast<RouteUI*>(this)), &RouteUI::disconnect_output)));
-
-		citems.push_back (SeparatorElem());
-		uint32_t const n_with_separator = citems.size ();
-
-		ARDOUR::BundleList current = _route->output()->bundles_connected ();
-
-		boost::shared_ptr<ARDOUR::BundleList> b = _session->bundles ();
-
-		/* guess the user-intended main type of the route output */
-		DataType intended_type = guess_main_type(false);
-
-		/* try adding the master bus first */
-		boost::shared_ptr<Route> master = _session->master_out();
-		if (master) {
-			maybe_add_bundle_to_output_menu (master->input()->bundle(), current, intended_type);
-		}
-
-		/* then other routes inputs */
-		RouteList copy = _session->get_routelist ();
-		copy.sort (RouteCompareByName ());
-		for (ARDOUR::RouteList::const_iterator i = copy.begin(); i != copy.end(); ++i) {
-			maybe_add_bundle_to_output_menu ((*i)->input()->bundle(), current, intended_type);
-		}
-
-		/* then try adding user bundles, often labeled/grouped physical inputs */
-		for (ARDOUR::BundleList::iterator i = b->begin(); i != b->end(); ++i) {
-			if (boost::dynamic_pointer_cast<UserBundle> (*i)) {
-				maybe_add_bundle_to_output_menu (*i, current, intended_type);
-			}
-		}
-
-		/* then all other bundles, including physical outs or other sofware */
-		for (ARDOUR::BundleList::iterator i = b->begin(); i != b->end(); ++i) {
-			if (boost::dynamic_pointer_cast<UserBundle> (*i) == 0) {
-				maybe_add_bundle_to_output_menu (*i, current, intended_type);
-			}
-		}
-
-		if (citems.size() == n_with_separator) {
-			/* no routes added; remove the separator */
-			citems.pop_back ();
-		}
-
-		citems.push_back (SeparatorElem());
-
-		if (!ARDOUR::Profile->get_mixbus()) {
-			bool need_separator = false;
-			for (DataType::iterator i = DataType::begin(); i != DataType::end(); ++i) {
-				if (!_route->output()->can_add_port (*i)) {
-					continue;
-				}
-				need_separator = true;
-				citems.push_back (
-						MenuElem (
-							string_compose (_("Add %1 port"), (*i).to_i18n_string()),
-							sigc::bind (sigc::mem_fun (*this, &MixerStrip::add_output_port), *i)
-							)
-						);
-			}
-			if (need_separator) {
-				citems.push_back (SeparatorElem());
-			}
-		}
-
-		citems.push_back (MenuElem (_("Routing Grid"), sigc::mem_fun (*(static_cast<RouteUI*>(this)), &RouteUI::edit_output_configuration)));
-
-		Gtkmm2ext::anchored_menu_popup(&output_menu, &output_button, "",
-		                               1, ev->time);
-
-		break;
-	}
-
-	default:
-		break;
-	}
-	return TRUE;
-}
-
-gint
-MixerStrip::input_release (GdkEventButton *ev)
-{
-	switch (ev->button) {
-
-	case 3:
-		edit_input_configuration ();
-		break;
-	default:
-		break;
-
-	}
-
-	return false;
-}
-
-
-gint
-MixerStrip::input_press (GdkEventButton *ev)
-{
-	using namespace Menu_Helpers;
-
-	MenuList& citems = input_menu.items();
-	input_menu.set_name ("ArdourContextMenu");
-	citems.clear();
-
-	if (!ARDOUR_UI_UTILS::engine_is_running ()) {
-		return true;
-	}
-
-	if (_session->actively_recording() && is_track() && track()->rec_enable_control()->get_value())
-		return true;
-
-	switch (ev->button) {
-
-	case 3:
-		return false;  //don't handle the mouse-down here.  wait for mouse-up to pop the menu
-
-	case 1:
-	{
-		citems.push_back (MenuElem (_("Disconnect"), sigc::mem_fun (*(static_cast<RouteUI*>(this)), &RouteUI::disconnect_input)));
-
-		citems.push_back (SeparatorElem());
-		uint32_t const n_with_separator = citems.size ();
-
-		input_menu_bundles.clear ();
-
-		ARDOUR::BundleList current = _route->input()->bundles_connected ();
-
-		boost::shared_ptr<ARDOUR::BundleList> b = _session->bundles ();
-
-		/* give user bundles first chance at being in the menu */
-
-		for (ARDOUR::BundleList::iterator i = b->begin(); i != b->end(); ++i) {
-			if (boost::dynamic_pointer_cast<UserBundle> (*i)) {
-				maybe_add_bundle_to_input_menu (*i, current);
-			}
-		}
-
-		for (ARDOUR::BundleList::iterator i = b->begin(); i != b->end(); ++i) {
-			if (boost::dynamic_pointer_cast<UserBundle> (*i) == 0) {
-				maybe_add_bundle_to_input_menu (*i, current);
-			}
-		}
-
-		boost::shared_ptr<ARDOUR::RouteList> routes = _session->get_routes ();
-		RouteList copy = *routes;
-		copy.sort (RouteCompareByName ());
-		for (ARDOUR::RouteList::const_iterator i = copy.begin(); i != copy.end(); ++i) {
-			maybe_add_bundle_to_input_menu ((*i)->output()->bundle(), current);
-		}
-
-		if (citems.size() == n_with_separator) {
-			/* no routes added; remove the separator */
-			citems.pop_back ();
-		}
-
-		citems.push_back (SeparatorElem());
-
-		bool need_separator = false;
-		for (DataType::iterator i = DataType::begin(); i != DataType::end(); ++i) {
-			if (!_route->input()->can_add_port (*i)) {
-				continue;
-			}
-			need_separator = true;
-			citems.push_back (
-				MenuElem (
-					string_compose (_("Add %1 port"), (*i).to_i18n_string()),
-					sigc::bind (sigc::mem_fun (*this, &MixerStrip::add_input_port), *i)
-					)
-				);
-		}
-		if (need_separator) {
-			citems.push_back (SeparatorElem());
-		}
-
-		citems.push_back (MenuElem (_("Routing Grid"), sigc::mem_fun (*(static_cast<RouteUI*>(this)), &RouteUI::edit_input_configuration)));
-
-		Gtkmm2ext::anchored_menu_popup(&input_menu, &input_button, "",
-		                               1, ev->time);
-
-		break;
-	}
-	default:
-		break;
-	}
-	return TRUE;
-}
-
-void
-MixerStrip::bundle_input_chosen (boost::shared_ptr<ARDOUR::Bundle> c)
-{
-	if (ignore_toggle) {
-		return;
-	}
-
-	_route->input()->connect_ports_to_bundle (c, true, this);
-}
-
-void
-MixerStrip::bundle_output_chosen (boost::shared_ptr<ARDOUR::Bundle> c)
-{
-	if (ignore_toggle) {
-		return;
-	}
-
-	_route->output()->connect_ports_to_bundle (c, true, true, this);
-}
-
-void
-MixerStrip::maybe_add_bundle_to_input_menu (boost::shared_ptr<Bundle> b, ARDOUR::BundleList const& /*current*/)
-{
-	using namespace Menu_Helpers;
-
-	if (b->ports_are_outputs() == false || b->nchannels() != _route->n_inputs() || *b == *_route->output()->bundle()) {
-		return;
-	}
-
-	list<boost::shared_ptr<Bundle> >::iterator i = input_menu_bundles.begin ();
-	while (i != input_menu_bundles.end() && b->has_same_ports (*i) == false) {
-		++i;
-	}
-
-	if (i != input_menu_bundles.end()) {
-		return;
-	}
-
-	input_menu_bundles.push_back (b);
-
-	MenuList& citems = input_menu.items();
-	citems.push_back (MenuElemNoMnemonic (b->name (), sigc::bind (sigc::mem_fun(*this, &MixerStrip::bundle_input_chosen), b)));
-}
-
-void
-MixerStrip::maybe_add_bundle_to_output_menu (boost::shared_ptr<Bundle> b, ARDOUR::BundleList const& /*current*/,
-                                             DataType type)
-{
-	using namespace Menu_Helpers;
-
-	/* The bundle should be an input one, but not ours */
-	if (b->ports_are_inputs() == false || *b == *_route->input()->bundle()) {
-		return;
-	}
-
-	/* Don't add the monitor input unless we are Master */
-	boost::shared_ptr<Route> monitor = _session->monitor_out();
-	if ((!_route->is_master()) && monitor && b->has_same_ports (monitor->input()->bundle()))
-		return;
-
-	/* It should either match exactly our outputs (if |type| is DataType::NIL)
-	 * or have the same number of |type| channels than our outputs. */
-	if (type == DataType::NIL) {
-		if(b->nchannels() != _route->n_outputs())
-			return;
-	} else {
-		if (b->nchannels().n(type) != _route->n_outputs().n(type))
-			return;
-	}
-
-	/* Avoid adding duplicates */
-	list<boost::shared_ptr<Bundle> >::iterator i = output_menu_bundles.begin ();
-	while (i != output_menu_bundles.end() && b->has_same_ports (*i) == false) {
-		++i;
-	}
-	if (i != output_menu_bundles.end()) {
-		return;
-	}
-
-	/* Now add the bundle to the menu */
-	output_menu_bundles.push_back (b);
-
-	MenuList& citems = output_menu.items();
-	citems.push_back (MenuElemNoMnemonic (b->name (), sigc::bind (sigc::mem_fun(*this, &MixerStrip::bundle_output_chosen), b)));
-}
-
-void
-MixerStrip::update_diskstream_display ()
-{
-	if (is_track() && input_selector) {
-		input_selector->hide_all ();
-	}
-
-	route_color_changed ();
-}
-
 void
 MixerStrip::connect_to_pan ()
 {
@@ -1256,301 +909,10 @@ MixerStrip::update_panner_choices ()
 	panners.set_available_panners(PannerManager::instance().PannerManager::get_available_panners(in, out));
 }
 
-DataType
-MixerStrip::guess_main_type(bool for_input, bool favor_connected) const
-{
-	/* The heuristic follows these principles:
-	 *  A) If all ports that the user connected are of the same type, then he
-	 *     very probably intends to use the IO with that type. A common subcase
-	 *     is when the IO has only ports of the same type (connected or not).
-	 *  B) If several types of ports are connected, then we should guess based
-	 *     on the likeliness of the user wanting to use a given type.
-	 *     We assume that the DataTypes are ordered from the most likely to the
-	 *     least likely when iterating or comparing them with "<".
-	 *  C) If no port is connected, the same logic can be applied with all ports
-	 *     instead of connected ones. TODO: Try other ideas, for instance look at
-	 *     the last plugin output when |for_input| is false (note: when StrictIO
-	 *     the outs of the last plugin should be the same as the outs of the route
-	 *     modulo the panner which forwards non-audio anyway).
-	 * All of these constraints are respected by the following algorithm that
-	 * just returns the most likely datatype found in connected ports if any, or
-	 * available ports if any (since if all ports are of the same type, the most
-	 * likely found will be that one obviously). */
-
-	boost::shared_ptr<IO> io = for_input ? _route->input() : _route->output();
-
-	/* Find most likely type among connected ports */
-	if (favor_connected) {
-		DataType type = DataType::NIL; /* NIL is always last so least likely */
-		for (PortSet::iterator p = io->ports().begin(); p != io->ports().end(); ++p) {
-			if (p->connected() && p->type() < type)
-				type = p->type();
-		}
-		if (type != DataType::NIL) {
-			/* There has been a connected port (necessarily non-NIL) */
-			return type;
-		}
-	}
-
-	/* Find most likely type among available ports.
-	 * The iterator stops before NIL. */
-	for (DataType::iterator t = DataType::begin(); t != DataType::end(); ++t) {
-		if (io->n_ports().n(*t) > 0)
-			return *t;
-	}
-
-	/* No port at all, return the most likely datatype by default */
-	return DataType::front();
-}
-
-/*
- * Output port labelling
- *
- * Case 1: Each output has one connection, all connections are to system:playback_%i
- *   out 1 -> system:playback_1
- *   out 2 -> system:playback_2
- *   out 3 -> system:playback_3
- *   Display as: 1/2/3
- *
- * Case 2: Each output has one connection, all connections are to ardour:track_x/in 1
- *   out 1 -> ardour:track_x/in 1
- *   out 2 -> ardour:track_x/in 2
- *   Display as: track_x
- *
- * Case 3: Each output has one connection, all connections are to Jack client "program x"
- *   out 1 -> program x:foo
- *   out 2 -> program x:foo
- *   Display as: program x
- *
- * Case 4: No connections (Disconnected)
- *   Display as: -
- *
- * Default case (unusual routing):
- *   Display as: *number of connections*
- *
- *
- * Tooltips
- *
- * .-----------------------------------------------.
- * | Mixdown                                       |
- * | out 1 -> ardour:master/in 1, jamin:input/in 1 |
- * | out 2 -> ardour:master/in 2, jamin:input/in 2 |
- * '-----------------------------------------------'
- * .-----------------------------------------------.
- * | Guitar SM58                                   |
- * | Disconnected                                  |
- * '-----------------------------------------------'
- */
-
-void
-MixerStrip::update_io_button (bool for_input)
-{
-	ostringstream tooltip;
-	ostringstream label;
-	bool have_label = false;
-
-	uint32_t total_connection_count = 0;
-	uint32_t typed_connection_count = 0;
-	bool each_typed_port_has_one_connection = true;
-
-	DataType dt = guess_main_type(for_input);
-	boost::shared_ptr<IO> io = for_input ? _route->input() : _route->output();
-
-	/* Fill in the tooltip. Also count:
-	 *  - The total number of connections.
-	 *  - The number of main-typed connections.
-	 *  - Whether each main-typed port has exactly one connection. */
-	if (for_input) {
-		tooltip << string_compose (_("<b>INPUT</b> to %1"),
-				Gtkmm2ext::markup_escape_text (_route->name()));
-	} else {
-		tooltip << string_compose (_("<b>OUTPUT</b> from %1"),
-				Gtkmm2ext::markup_escape_text (_route->name()));
-	}
-
-	string arrow = Gtkmm2ext::markup_escape_text(for_input ? " <- " : " -> ");
-	vector<string> port_connections;
-	for (PortSet::iterator port = io->ports().begin();
-	                       port != io->ports().end();
-	                       ++port) {
-		port_connections.clear();
-		port->get_connections(port_connections);
-
-		uint32_t port_connection_count = 0;
-
-		for (vector<string>::iterator i = port_connections.begin();
-		                              i != port_connections.end();
-		                              ++i) {
-			++port_connection_count;
-
-			if (port_connection_count == 1) {
-				tooltip << endl << Gtkmm2ext::markup_escape_text (
-						port->name().substr(port->name().find("/") + 1));
-				tooltip << arrow;
-			} else {
-				tooltip << ", ";
-			}
-
-			tooltip << Gtkmm2ext::markup_escape_text(*i);
-		}
-
-		total_connection_count += port_connection_count;
-		if (port->type() == dt) {
-			typed_connection_count += port_connection_count;
-			each_typed_port_has_one_connection &= (port_connection_count == 1);
-		}
-
-	}
-
-	if (total_connection_count == 0) {
-		tooltip << endl << _("Disconnected");
-	}
-
-	if (typed_connection_count == 0) {
-		label << "-";
-		have_label = true;
-	}
-
-	/* Are all main-typed channels connected to the same route ? */
-	if (!have_label) {
-		boost::shared_ptr<ARDOUR::RouteList> routes = _session->get_routes ();
-		for (ARDOUR::RouteList::const_iterator route = routes->begin();
-		                                       route != routes->end();
-		                                       ++route) {
-			boost::shared_ptr<IO> dest_io =
-				for_input ? (*route)->output() : (*route)->input();
-			if (io->bundle()->connected_to(dest_io->bundle(),
-			                               _session->engine(),
-			                               dt, true)) {
-				label << Gtkmm2ext::markup_escape_text ((*route)->name());
-				have_label = true;
-				break;
-			}
-		}
-	}
-
-	/* Are all main-typed channels connected to the same (user) bundle ? */
-	if (!have_label) {
-		boost::shared_ptr<ARDOUR::BundleList> bundles = _session->bundles ();
-		for (ARDOUR::BundleList::iterator bundle = bundles->begin();
-		                                  bundle != bundles->end();
-		                                  ++bundle) {
-			if (boost::dynamic_pointer_cast<UserBundle> (*bundle) == 0)
-				continue;
-			if (io->bundle()->connected_to(*bundle, _session->engine(),
-			                               dt, true)) {
-				label << Gtkmm2ext::markup_escape_text ((*bundle)->name());
-				have_label = true;
-				break;
-			}
-		}
-	}
-
-	/* Is each main-typed channel only connected to a physical output ? */
-	if (!have_label && each_typed_port_has_one_connection) {
-		ostringstream temp_label;
-		vector<string> phys;
-		string playorcapture;
-		if (for_input) {
-			_session->engine().get_physical_inputs(dt, phys);
-			playorcapture = "capture_";
-		} else {
-			_session->engine().get_physical_outputs(dt, phys);
-			playorcapture = "playback_";
-		}
-		for (PortSet::iterator port = io->ports().begin(dt);
-		                       port != io->ports().end(dt);
-		                       ++port) {
-			string pn = "";
-			for (vector<string>::iterator s = phys.begin();
-			                              s != phys.end();
-			                              ++s) {
-				if (!port->connected_to(*s))
-					continue;
-				pn = AudioEngine::instance()->get_pretty_name_by_name(*s);
-				if (pn.empty()) {
-					string::size_type start = (*s).find(playorcapture);
-					if (start != string::npos) {
-						pn = (*s).substr(start + playorcapture.size());
-					}
-				}
-				break;
-			}
-			if (pn.empty()) {
-				temp_label.str(""); /* erase the failed attempt */
-				break;
-			}
-			if (port != io->ports().begin(dt))
-				temp_label << "/";
-			temp_label << pn;
-		}
-
-		if (!temp_label.str().empty()) {
-			label << temp_label.str();
-			have_label = true;
-		}
-	}
-
-	/* Is each main-typed channel connected to a single and different port with
-	 * the same client name (e.g. another JACK client) ? */
-	if (!have_label && each_typed_port_has_one_connection) {
-		string maybe_client = "";
-		vector<string> connections;
-		for (PortSet::iterator port = io->ports().begin(dt);
-		                       port != io->ports().end(dt);
-		                       ++port) {
-			port_connections.clear();
-			port->get_connections(port_connections);
-			string connection = port_connections.front();
-
-			vector<string>::iterator i = connections.begin();
-			while (i != connections.end() && *i != connection) {
-				++i;
-			}
-			if (i != connections.end())
-				break; /* duplicate connection */
-			connections.push_back(connection);
-
-			connection = connection.substr(0, connection.find(":"));
-			if (maybe_client.empty())
-				maybe_client = connection;
-			if (maybe_client != connection)
-				break;
-		}
-		if (connections.size() == io->n_ports().n(dt)) {
-			label << maybe_client;
-			have_label = true;
-		}
-	}
-
-	/* Odd configuration */
-	if (!have_label) {
-		label << "*" << total_connection_count << "*";
-	}
-
-	if (total_connection_count > typed_connection_count) {
-		label << "\u2295"; /* circled plus */
-	}
-
-	/* Actually set the properties of the button */
-	char * cstr = new char[tooltip.str().size() + 1];
-	strcpy(cstr, tooltip.str().c_str());
-
-	if (for_input) {
-		input_button.set_text (label.str());
-		set_tooltip (&input_button, cstr);
-	} else {
-		output_button.set_text (label.str());
-		set_tooltip (&output_button, cstr);
-	}
-
-	delete [] cstr;
-}
 
 void
 MixerStrip::update_input_display ()
 {
-	update_io_button (true);
 	panners.setup_pan ();
 
 	if (has_audio_outputs ()) {
@@ -1564,7 +926,6 @@ MixerStrip::update_input_display ()
 void
 MixerStrip::update_output_display ()
 {
-	update_io_button (false);
 	gpm.setup_meters ();
 	panners.setup_pan ();
 
@@ -1581,12 +942,6 @@ MixerStrip::fast_update ()
 	gpm.update_meters ();
 }
 
-void
-MixerStrip::diskstream_changed ()
-{
-	Gtkmm2ext::UI::instance()->call_slot (invalidator (*this), boost::bind (&MixerStrip::update_diskstream_display, this));
-}
-
 void
 MixerStrip::io_changed_proxy ()
 {
@@ -1594,23 +949,6 @@ MixerStrip::io_changed_proxy ()
 	Glib::signal_idle().connect_once (sigc::mem_fun (*this, &MixerStrip::update_trim_control));
 }
 
-void
-MixerStrip::port_connected_or_disconnected (boost::weak_ptr<Port> wa, boost::weak_ptr<Port> wb)
-{
-	boost::shared_ptr<Port> a = wa.lock ();
-	boost::shared_ptr<Port> b = wb.lock ();
-
-	if ((a && _route->input()->has_port (a)) || (b && _route->input()->has_port (b))) {
-		update_input_display ();
-		set_width_enum (_width, this);
-	}
-
-	if ((a && _route->output()->has_port (a)) || (b && _route->output()->has_port (b))) {
-		update_output_display ();
-		set_width_enum (_width, this);
-	}
-}
-
 void
 MixerStrip::setup_comment_button ()
 {
@@ -1936,18 +1274,6 @@ MixerStrip::name_changed ()
 	}
 }
 
-void
-MixerStrip::input_button_resized (Gtk::Allocation& alloc)
-{
-	input_button.set_layout_ellipsize_width (alloc.get_width() * PANGO_SCALE);
-}
-
-void
-MixerStrip::output_button_resized (Gtk::Allocation& alloc)
-{
-	output_button.set_layout_ellipsize_width (alloc.get_width() * PANGO_SCALE);
-}
-
 void
 MixerStrip::name_button_resized (Gtk::Allocation& alloc)
 {
@@ -1994,7 +1320,7 @@ MixerStrip::width_button_pressed (GdkEventButton* ev)
 void
 MixerStrip::loudess_analysis_button_clicked ()
 {
-	PublicEditor::instance().measure_master_loudness (false);
+	PublicEditor::instance().loudness_assistant (false);
 }
 
 bool
