@@ -44,6 +44,7 @@
 #include <sigc++/signal.h>
 
 #include "pbd/statefuldestructible.h"
+#include "pbd/g_atomic_compat.h"
 
 #include "temporal/beats.h"
 
@@ -83,11 +84,11 @@ class AutomationLine;
 class AutomationTimeAxisView;
 class ControlPoint;
 class DragManager;
+class EditorCursor;
 class ArdourMarker;
 class MeterMarker;
 class MixerStrip;
 class MouseCursors;
-class PlaylistSelector;
 class RegionView;
 class RouteTimeAxisView;
 class Selection;
@@ -261,10 +262,10 @@ public:
 	/** Import existing media */
 	virtual void do_import (std::vector<std::string> paths, Editing::ImportDisposition, Editing::ImportMode mode, ARDOUR::SrcQuality,
 	                        ARDOUR::MidiTrackNameSource, ARDOUR::MidiTempoMapDisposition, samplepos_t&,
-	                        boost::shared_ptr<ARDOUR::PluginInfo> instrument=boost::shared_ptr<ARDOUR::PluginInfo>()) = 0;
-	virtual void do_embed (std::vector<std::string> paths, Editing::ImportDisposition, Editing::ImportMode mode,
-	                       samplepos_t&,
-	                       boost::shared_ptr<ARDOUR::PluginInfo> instrument=boost::shared_ptr<ARDOUR::PluginInfo>()) = 0;
+	                        boost::shared_ptr<ARDOUR::PluginInfo> instrument = boost::shared_ptr<ARDOUR::PluginInfo>(),
+	                        bool with_markers = false) = 0;
+	virtual void do_embed (std::vector<std::string> paths, Editing::ImportDisposition, Editing::ImportMode mode, samplepos_t&,
+	                       boost::shared_ptr<ARDOUR::PluginInfo> instrument = boost::shared_ptr<ARDOUR::PluginInfo>()) = 0;
 
 	/** Open main export dialog */
 	virtual void export_audio () = 0;
@@ -278,18 +279,28 @@ public:
 	/** Open export dialog with current range pre-selected */
 	virtual void export_range () = 0;
 
-	virtual void measure_master_loudness (bool) = 0;
+	virtual void loudness_assistant (bool) = 0;
 
 	virtual void register_actions () = 0;
 	virtual void set_zoom_focus (Editing::ZoomFocus) = 0;
 	virtual Editing::ZoomFocus get_zoom_focus () const = 0;
 	virtual samplecnt_t get_current_zoom () const = 0;
 	virtual void reset_zoom (samplecnt_t) = 0;
-	virtual PlaylistSelector& playlist_selector() const = 0;
 	virtual void clear_playlist (boost::shared_ptr<ARDOUR::Playlist>) = 0;
-	virtual void new_playlists (TimeAxisView*) = 0;
-	virtual void copy_playlists (TimeAxisView*) = 0;
-	virtual void clear_playlists (TimeAxisView*) = 0;
+	virtual void clear_grouped_playlists (RouteUI*) = 0;
+
+	virtual void mapped_select_playlist_matching (RouteUI&, boost::weak_ptr<ARDOUR::Playlist> pl) = 0;
+
+	virtual void mapover_grouped_routes (sigc::slot<void, RouteUI&> sl, RouteUI*, PBD::PropertyID) const = 0;
+	virtual void mapover_armed_routes (sigc::slot<void, RouteUI&> sl) const = 0;
+	virtual void mapover_selected_routes (sigc::slot<void, RouteUI&> sl) const = 0;
+	virtual void mapover_all_routes (sigc::slot<void, RouteUI&> sl) const = 0;
+
+	virtual void new_playlists_for_all_tracks(bool copy) = 0;
+	virtual void new_playlists_for_grouped_tracks(RouteUI* rui, bool copy) = 0;
+	virtual void new_playlists_for_selected_tracks(bool copy) = 0;
+	virtual void new_playlists_for_armed_tracks(bool copy) = 0;
+
 	virtual void select_all_visible_lanes () = 0;
 	virtual void select_all_tracks () = 0;
 	virtual void deselect_all () = 0;
@@ -364,6 +375,7 @@ public:
 	virtual void embed_audio_from_video (std::string, samplepos_t n = 0, bool lock_position_to_video = true) = 0;
 
 	virtual bool track_selection_change_without_scroll () const = 0;
+	virtual bool show_touched_automation () const = 0;
 
 	virtual StripableTimeAxisView* get_stripable_time_axis_by_id (const PBD::ID& id) const = 0;
 
@@ -472,10 +484,15 @@ public:
 	virtual MouseCursors const* cursors () const = 0;
 	virtual VerboseCursor* verbose_cursor () const = 0;
 
+	virtual EditorCursor* playhead_cursor () const = 0;
+	virtual EditorCursor* snapped_cursor () const = 0;
+
 	virtual bool get_smart_mode () const = 0;
 
 	virtual void get_pointer_position (double &, double &) const = 0;
 
+	virtual std::pair <samplepos_t, samplepos_t> session_gui_extents (bool use_extra = true) const = 0;
+
 	virtual ARDOUR::Location* find_location_from_marker (ArdourMarker*, bool&) const = 0;
 	virtual ArdourMarker* find_marker_from_location_id (PBD::ID const&, bool) const = 0;
 
@@ -516,13 +533,14 @@ protected:
 	friend class DisplaySuspender;
 	virtual void suspend_route_redisplay () = 0;
 	virtual void resume_route_redisplay () = 0;
-	gint _suspend_route_redisplay_counter;
+
+	GATOMIC_QUAL gint _suspend_route_redisplay_counter;
 };
 
 class DisplaySuspender {
 	public:
 		DisplaySuspender() {
-			if (g_atomic_int_add(&PublicEditor::instance()._suspend_route_redisplay_counter, 1) == 0) {
+			if (g_atomic_int_add (&PublicEditor::instance()._suspend_route_redisplay_counter, 1) == 0) {
 				PublicEditor::instance().suspend_route_redisplay ();
 			}
 		}
