@@ -33,9 +33,11 @@
 #include "ardour/playlist.h"
 #include "ardour/profile.h"
 #include "ardour/session.h"
+#include "ardour/source.h"
 
 #include "gtkmm2ext/colors.h"
 
+#include "canvas/arrow.h"
 #include "canvas/polygon.h"
 #include "canvas/debug.h"
 #include "canvas/pixbuf.h"
@@ -90,27 +92,47 @@ RegionView::RegionView (ArdourCanvas::Container*          parent,
 	, in_destructor(false)
 	, wait_for_data(false)
 	, _silence_text (0)
+	, _xrun_markers_visible (false)
+	, _cue_markers_visible (false)
 {
+	UIConfiguration::instance().ParameterChanged.connect (sigc::mem_fun (*this, &RegionView::parameter_changed));
+
+	for (SourceList::const_iterator s = _region->sources().begin(); s != _region->sources().end(); ++s) {
+		(*s)->CueMarkersChanged.connect (*this, invalidator (*this), boost::bind (&RegionView::update_cue_markers, this), gui_context());
+	}
 }
 
 RegionView::RegionView (const RegionView& other)
 	: sigc::trackable(other)
 	, TimeAxisViewItem (other)
 	, _silence_text (0)
+	, _xrun_markers_visible (false)
+	, _cue_markers_visible (false)
 {
+	UIConfiguration::instance().ParameterChanged.connect (sigc::mem_fun (*this, &RegionView::parameter_changed));
+
 	/* derived concrete type will call init () */
 
 	_region = other._region;
 	current_visible_sync_position = other.current_visible_sync_position;
 	valid = false;
 	_pixel_width = other._pixel_width;
+
+	for (SourceList::const_iterator s = _region->sources().begin(); s != _region->sources().end(); ++s) {
+		(*s)->CueMarkersChanged.connect (*this, invalidator (*this), boost::bind (&RegionView::update_cue_markers, this), gui_context());
+	}
 }
 
 RegionView::RegionView (const RegionView& other, boost::shared_ptr<Region> other_region)
 	: sigc::trackable(other)
 	, TimeAxisViewItem (other)
 	, _silence_text (0)
+	, _xrun_markers_visible (false)
+	, _cue_markers_visible (false)
 {
+	UIConfiguration::instance().ParameterChanged.connect (sigc::mem_fun (*this, &RegionView::parameter_changed));
+
+	/* derived concrete type will call init () */
 	/* this is a pseudo-copy constructor used when dragging regions
 	   around on the canvas.
 	*/
@@ -121,8 +143,13 @@ RegionView::RegionView (const RegionView& other, boost::shared_ptr<Region> other
 	current_visible_sync_position = other.current_visible_sync_position;
 	valid = false;
 	_pixel_width = other._pixel_width;
+
+	for (SourceList::const_iterator s = _region->sources().begin(); s != _region->sources().end(); ++s) {
+		(*s)->CueMarkersChanged.connect (*this, invalidator (*this), boost::bind (&RegionView::update_cue_markers, this), gui_context());
+	}
 }
 
+
 RegionView::RegionView (ArdourCanvas::Container*          parent,
                         TimeAxisView&                     tv,
                         boost::shared_ptr<ARDOUR::Region> r,
@@ -143,6 +170,11 @@ RegionView::RegionView (ArdourCanvas::Container*          parent,
 	, wait_for_data(false)
 	, _silence_text (0)
 {
+	UIConfiguration::instance().ParameterChanged.connect (sigc::mem_fun (*this, &RegionView::parameter_changed));
+
+	for (SourceList::const_iterator s = _region->sources().begin(); s != _region->sources().end(); ++s) {
+		(*s)->CueMarkersChanged.connect (*this, invalidator (*this), boost::bind (&RegionView::update_cue_markers, this), gui_context());
+	}
 }
 
 void
@@ -181,6 +213,29 @@ RegionView::init (bool wfd)
 		name_text->Event.connect (sigc::bind (sigc::mem_fun (PublicEditor::instance(), &PublicEditor::canvas_region_view_name_event), name_text, this));
 	}
 
+	XrunPositions xrp;
+	_region->captured_xruns (xrp, true);
+	int arrow_size = (int)(7.0 * UIConfiguration::instance ().get_ui_scale ()) & ~1;
+	for (XrunPositions::const_iterator x = xrp.begin (); x != xrp.end (); ++x) {
+		ArdourCanvas::Arrow* canvas_item = new ArdourCanvas::Arrow(group);
+		canvas_item->set_color (UIConfiguration::instance().color ("neutral:background"));
+		canvas_item->set_show_head (1, true);
+		canvas_item->set_show_head (0, false);
+		canvas_item->set_head_width (1, arrow_size);
+		canvas_item->set_head_height (1, arrow_size);
+		canvas_item->set_y0 (arrow_size);
+		canvas_item->set_y1 (arrow_size);
+		canvas_item->raise_to_top ();
+		canvas_item->hide ();
+		_xrun_markers.push_back (make_pair(*x, canvas_item));
+	}
+
+	_xrun_markers_visible = false;
+	update_xrun_markers ();
+
+	_cue_markers_visible = false;
+	update_cue_markers ();
+
 	if (wfd) {
 		_enable_display = true;
 	}
@@ -209,6 +264,14 @@ RegionView::~RegionView ()
 		delete *i;
 	}
 
+	for (list<std::pair<samplepos_t, ArdourCanvas::Arrow*> >::iterator i = _xrun_markers.begin(); i != _xrun_markers.end(); ++i) {
+		delete ((*i).second);
+	}
+
+	for (ViewCueMarkers::iterator c = _cue_markers.begin(); c != _cue_markers.end(); ++c) {
+		delete (*c);
+	}
+
 	drop_silent_frames ();
 
 	delete editor;
@@ -373,6 +436,7 @@ RegionView::region_changed (const PropertyChange& what_changed)
 	if (what_changed.contains (ARDOUR::bounds_change)) {
 		region_resized (what_changed);
 		region_sync_changed ();
+		update_cue_markers ();
 	}
 	if (what_changed.contains (ARDOUR::Properties::muted)) {
 		region_muted ();
@@ -421,10 +485,10 @@ RegionView::region_resized (const PropertyChange& what_changed)
 		unit_length = _region->length() / samples_per_pixel;
 
 		for (vector<GhostRegion*>::iterator i = ghosts.begin(); i != ghosts.end(); ++i) {
-
 			(*i)->set_duration (unit_length);
-
 		}
+
+		update_xrun_markers ();
 	}
 }
 
@@ -433,6 +497,143 @@ RegionView::reset_width_dependent_items (double pixel_width)
 {
 	TimeAxisViewItem::reset_width_dependent_items (pixel_width);
 	_pixel_width = pixel_width;
+
+	if (_xrun_markers_visible) {
+		const samplepos_t start = _region->start();
+		for (list<std::pair<samplepos_t, ArdourCanvas::Arrow*> >::iterator i = _xrun_markers.begin(); i != _xrun_markers.end(); ++i) {
+			float x_pos = trackview.editor().sample_to_pixel (i->first - start);
+			i->second->set_x (x_pos);
+		}
+	}
+}
+
+void
+RegionView::update_xrun_markers ()
+{
+	const bool show_xruns_markers = UIConfiguration::instance().get_show_region_xrun_markers();
+	if (_xrun_markers_visible == show_xruns_markers && !_xrun_markers_visible) {
+		return;
+	}
+
+	const samplepos_t start = _region->start();
+	const samplepos_t length = _region->length();
+	for (list<std::pair<samplepos_t, ArdourCanvas::Arrow*> >::iterator i = _xrun_markers.begin(); i != _xrun_markers.end(); ++i) {
+		float x_pos = trackview.editor().sample_to_pixel (i->first - start);
+		i->second->set_x (x_pos);
+		if (show_xruns_markers && (i->first >= start && i->first < start + length)) {
+			i->second->show ();
+		} else  {
+			i->second->hide ();
+		}
+	}
+	_xrun_markers_visible = show_xruns_markers;
+}
+
+RegionView::ViewCueMarker::~ViewCueMarker ()
+{
+	delete view_marker;
+}
+
+void
+RegionView::update_cue_markers ()
+{
+	const bool show_cue_markers = UIConfiguration::instance().get_show_region_cue_markers();
+
+	if (_cue_markers_visible == show_cue_markers && !_cue_markers_visible) {
+		return;
+	}
+
+	const samplepos_t start = region()->start();
+	const samplepos_t end = region()->start() + region()->length();
+	const Gtkmm2ext::SVAModifier alpha = UIConfiguration::instance().modifier (X_("region mark"));
+	const uint32_t color = Gtkmm2ext::HSV (UIConfiguration::instance().color ("region mark")).mod (alpha).color();
+
+	/* We assume that if the region has multiple sources, any of them will
+	 * be appropriate as the origin of cue markers. We use the first one.
+	 */
+
+	boost::shared_ptr<Source> source = region()->source (0);
+	CueMarkers const & model_markers (source->cue_markers());
+
+	/* Remove any view markers that are no longer present in the model cue
+	 * marker set
+	 */
+
+	for (ViewCueMarkers::iterator v = _cue_markers.begin(); v != _cue_markers.end(); ) {
+		if (model_markers.find ((*v)->model_marker) == model_markers.end()) {
+			delete *v;
+			v = _cue_markers.erase (v);
+		} else {
+			++v;
+		}
+	}
+
+	/* now check all the model markers and make sure we have view markers
+	 * for them. Note that because we use Source::cue_markers() above the
+	 * set will contain markers stamped with absolute, not region-relative,
+	 * timestamps and some of them may be outside the Region.
+	 */
+
+	for (CueMarkers::const_iterator c = model_markers.begin(); c != model_markers.end(); c++) {
+
+		ViewCueMarkers::iterator existing = _cue_markers.end();
+
+		for (ViewCueMarkers::iterator v = _cue_markers.begin(); v != _cue_markers.end(); ++v) {
+			if ((*v)->model_marker == *c) {
+				existing = v;
+				break;
+			}
+		}
+
+		if (existing == _cue_markers.end()) {
+
+			if (c->position() < start || c->position() >= end) {
+				/* not withing this region */
+				continue;
+			}
+
+			/* Create a new ViewCueMarker */
+
+			ArdourMarker* mark = new ArdourMarker (trackview.editor(), *group, color , c->text(), ArdourMarker::RegionCue, c->position() - start, true, this);
+			mark->set_points_color (color);
+			mark->set_show_line (true);
+			/* make sure the line has a clean end, before the frame
+			   of the region view
+			*/
+			mark->set_line_height (trackview.current_height() - (1.5 * UIConfiguration::instance ().get_ui_scale ()));
+			mark->the_item().raise_to_top ();
+
+			if (show_cue_markers) {
+				mark->show ();
+			} else  {
+				mark->hide ();
+			}
+
+			_cue_markers.push_back (new ViewCueMarker (mark, *c));
+
+		} else {
+
+			if (c->position() < start || c->position() >= end) {
+				/* not withing this region */
+				delete (*existing);
+				_cue_markers.erase (existing);
+				continue;
+			}
+
+			/* Move and control visibility for an existing ViewCueMarker */
+
+			if (show_cue_markers) {
+				(*existing)->view_marker->show ();
+			} else  {
+				(*existing)->view_marker->hide ();
+			}
+
+			(*existing)->view_marker->set_position (c->position() - start);
+			(*existing)->view_marker->the_item().raise_to_top ();
+		}
+	}
+
+	_cue_markers_visible = show_cue_markers;
 }
 
 void
@@ -515,6 +716,23 @@ RegionView::set_colors ()
 {
 	TimeAxisViewItem::set_colors ();
 	set_sync_mark_color ();
+
+	const Gtkmm2ext::SVAModifier alpha = UIConfiguration::instance().modifier (X_("region mark"));
+	const uint32_t color = Gtkmm2ext::HSV (UIConfiguration::instance().color ("region mark")).mod (alpha).color();
+
+	for (ViewCueMarkers::iterator c = _cue_markers.begin(); c != _cue_markers.end(); ++c) {
+		(*c)->view_marker->set_color_rgba (color);
+	}
+}
+
+void
+RegionView::parameter_changed (std::string const& p)
+{
+	if (p == "show-region-xrun-markers") {
+		update_xrun_markers ();
+	} else if (p == "show-region-cue-markers") {
+		update_cue_markers ();
+	}
 }
 
 void
@@ -752,6 +970,9 @@ RegionView::set_height (double h)
 		(*i)->set_y1 (h + 1);
 	}
 
+	for (ViewCueMarkers::iterator v = _cue_markers.begin(); v != _cue_markers.end(); ++v) {
+		(*v)->view_marker->set_line_height (h - (1.5 * UIConfiguration::instance().get_ui_scale()));
+	}
 }
 
 /** Remove old coverage frame and make new ones, if we're in a LayerDisplay mode
@@ -986,3 +1207,35 @@ RegionView::set_selected (bool yn)
 	_region->set_selected_for_solo(yn);
 	TimeAxisViewItem::set_selected(yn);
 }
+
+void
+RegionView::maybe_raise_cue_markers ()
+{
+	for (ViewCueMarkers::iterator v = _cue_markers.begin(); v != _cue_markers.end(); ++v) {
+		(*v)->view_marker->the_item().raise_to_top ();
+	}
+}
+
+CueMarker
+RegionView::find_model_cue_marker (ArdourMarker* m)
+{
+	for (ViewCueMarkers::iterator v = _cue_markers.begin(); v != _cue_markers.end(); ++v) {
+		if ((*v)->view_marker == m) {
+			return (*v)->model_marker;
+		}
+	}
+
+	return CueMarker (string(), 0); /* empty string signifies invalid */
+}
+
+void
+RegionView::drop_cue_marker (ArdourMarker* m)
+{
+	for (ViewCueMarkers::iterator v = _cue_markers.begin(); v != _cue_markers.end(); ++v) {
+		if ((*v)->view_marker == m) {
+			delete m;
+			_cue_markers.erase (v);
+			return;
+		}
+	}
+}
