@@ -28,6 +28,7 @@
 #include "pbd/file_utils.h"
 
 #include "ardour/audioengine.h"
+#include "ardour/audio_backend.h"
 #include "ardour/filename_extensions.h"
 #include "ardour/filesystem_paths.h"
 #include "ardour/profile.h"
@@ -69,6 +70,7 @@ using std::vector;
 
 StartupFSM::StartupFSM (EngineControl& amd)
 	: session_existing_sample_rate (0)
+	, session_engine_hints ("EngineHints")
 	, session_is_new (false)
 	, session_name_edited (false)
 	, new_user (NewUserWizard::required())
@@ -258,6 +260,7 @@ StartupFSM::dialog_response_handler (int response, StartupFSM::DialogID dialog_i
 					 * retry. Errors were addressed by
 					 * ::check_session_parameters()
 					 */
+					session_dialog->present ();
 					break;
 				}
 				break;
@@ -382,11 +385,10 @@ StartupFSM::show_plugin_scan_dialog ()
 {
 	set_state (WaitingForPlugins);
 
-	/* if the user does not ask to discover VSTs at startup, or if this is Mixbus, then the plugin scan
-	   that we run here, during startup, should only use the existing plugin cache (if any).
-	*/
-
-	const bool cache_only = (!Config->get_discover_vst_on_start() || Profile->get_mixbus());
+	/* if the user does not ask to discover AU/VSTs at startup, or if this is Mixbus, then the plugin scan
+	 * that we run here, during startup, should only use the existing plugin cache (if any).
+	 */
+	const bool cache_only = (!Config->get_discover_plugins_on_start () || Profile->get_mixbus ());
 	const bool verbose = new_user;
 
 	plugin_scan_dialog = new PluginScanDialog (cache_only, verbose);
@@ -398,7 +400,7 @@ StartupFSM::show_plugin_scan_dialog ()
 	 * See also comments in PluginScanDialog::start() to understand the absurd complexities behind this call.
 	 */
 
-	DEBUG_TRACE (DEBUG::GuiStartup, string_compose ("starting plugin dialog, cache only ? %1\n", !Config->get_discover_vst_on_start()));
+	DEBUG_TRACE (DEBUG::GuiStartup, string_compose ("starting plugin dialog, cache only ? %1\n", cache_only));
 	plugin_scan_dialog->start();
 	DEBUG_TRACE (DEBUG::GuiStartup, "plugin dialog done\n");
 }
@@ -435,9 +437,11 @@ StartupFSM::show_audiomidi_dialog ()
 void
 StartupFSM::start_audio_midi_setup ()
 {
+	BootMessage (_("Starting Audio/MIDI Engine"));
 	bool setup_required = false;
 
-	if (AudioEngine::instance()->current_backend() == 0) {
+	boost::shared_ptr<AudioBackend> backend = AudioEngine::instance()->current_backend();
+	if (!backend) {
 		/* backend is unknown ... */
 		setup_required = true;
 
@@ -457,10 +461,45 @@ StartupFSM::start_audio_midi_setup ()
 		}
 	}
 
+	bool try_autostart = (Config->get_try_autostart_engine () || g_getenv ("ARDOUR_TRY_AUTOSTART_ENGINE"));
+	if (session_is_new) {
+		try_autostart = false;
+	} else if (!backend) {
+		try_autostart = false;
+	} else if (try_autostart) {
+		/* if user has selected auto-start, check if autostart is possible */
+		bool ok = true;
+		std::string backend_name;
+		/* Allow auto-start if there is no backend information for the
+		 * given session. This can happen when loading an old (v6) session,
+		 * of if the user has been using externally started JACK.
+		 */
+		if (session_engine_hints.get_property ("backend", backend_name)) {
+			std::string input_device;
+			std::string output_device;
+			ok &= session_engine_hints.get_property ("input-device", input_device);
+			ok &= session_engine_hints.get_property ("output-device", output_device);
+			ok &= backend->name () == backend_name;
+			if (backend->use_separate_input_and_output_devices()) {
+				ok &= input_device == backend->input_device_name ();
+				ok &= output_device == backend->output_device_name ();
+			} else {
+				ok &= input_device == backend->device_name ();
+				ok &= output_device == backend->device_name ();
+			}
+		}
+		if (!ok) {
+			try_autostart = false;
+			ArdourMessageDialog msg (
+					_("Engine I/O device has changed since you last opened this session.\n"
+					  "Please verify that the new device has enough ports, or you may lose some i/o connections."),
+					false, Gtk::MESSAGE_WARNING, Gtk::BUTTONS_OK, true);
+			msg.run ();
+		}
+	}
+
 	if (setup_required) {
-
-		if (!session_is_new && (Config->get_try_autostart_engine () || g_getenv ("ARDOUR_TRY_AUTOSTART_ENGINE"))) {
-
+		if (try_autostart) {
 			AudioEngine::instance()->set_sample_rate(session_existing_sample_rate);
 			if (!AudioEngine::instance()->start ()) {
 				if (ARDOUR::AudioEngine::instance()->running()) {
@@ -552,7 +591,7 @@ StartupFSM::get_session_parameters_from_path (string const & path, string const
 		string program_version;
 
 		const string statefile_path = Glib::build_filename (session_path, session_name + ARDOUR::statefile_suffix);
-		if (Session::get_info_from_path (statefile_path, sr, fmt, program_version)) {
+		if (Session::get_info_from_path (statefile_path, sr, fmt, program_version, &session_engine_hints)) {
 			/* exists but we can't read it correctly */
 			error << string_compose (_("Cannot get existing session information from %1"), statefile_path) << endmsg;
 			return false;
@@ -671,10 +710,18 @@ StartupFSM::check_session_parameters (bool must_be_new)
 		if (rv < 0) {
 			ArdourMessageDialog msg (*session_dialog, string_compose (_("Extracting session-archive failed: %1"), inflate_error (rv)));
 			msg.run ();
-
 			return 1;
 		} else if (rv == 0) {
 			/* names are good (and session is unarchived/inflated */
+			float sr;
+			SampleFormat fmt;
+			string program_version;
+			const string statefile_path = Glib::build_filename (session_path, session_name + ARDOUR::statefile_suffix);
+			if (Session::get_info_from_path (statefile_path, sr, fmt, program_version, &session_engine_hints)) {
+				/* exists but we can't read it */
+				return -1;
+			}
+			session_existing_sample_rate = sr;
 			return 0;
 		}
 	}
@@ -786,7 +833,7 @@ StartupFSM::check_session_parameters (bool must_be_new)
 
 	if (!session_is_new) {
 
-		if (Session::get_info_from_path (statefile_path, sr, fmt, program_version)) {
+		if (Session::get_info_from_path (statefile_path, sr, fmt, program_version, &session_engine_hints)) {
 			/* exists but we can't read it */
 			return -1;
 		}
