@@ -343,18 +343,6 @@ StreamView::playlist_switched (boost::weak_ptr<Track> wtr)
 }
 
 
-void
-StreamView::diskstream_changed ()
-{
-	boost::shared_ptr<Track> t;
-
-	if ((t = _trackview.track()) != 0) {
-		Gtkmm2ext::UI::instance()->call_slot (invalidator (*this), boost::bind (&StreamView::display_track, this, t));
-	} else {
-		Gtkmm2ext::UI::instance()->call_slot (invalidator (*this), boost::bind (&StreamView::undisplay_track, this));
-	}
-}
-
 void
 StreamView::apply_color (Gdk::Color const& c, ColorTarget target)
 {
@@ -721,7 +709,7 @@ StreamView::setup_new_rec_layer_time (boost::shared_ptr<Region> region)
 	   top-layered region after the start of the region we are recording and make a note of it.
 	*/
 	if (_layer_display == Stacked) {
-		_new_rec_layer_time = _trackview.track()->playlist()->find_next_top_layer_position (region->start());
+		_new_rec_layer_time = _trackview.track()->playlist()->find_next_top_layer_position (region->position());
 	} else {
 		_new_rec_layer_time = max_samplepos;
 	}
