@@ -16,9 +16,13 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
+#ifndef __ardour_transpose_dialog_h__
+#define __ardour_transpose_dialog_h__
+
 #include <gtkmm/spinbutton.h>
 #include "ardour_dialog.h"
 
+
 /** A dialog box to select a transposition to apply to a MIDI region.
  *  It asks for octaves and semitones, with the transposition being
  *  the sum of the two.
@@ -37,3 +41,5 @@ private:
 	Gtk::SpinButton _octaves_spinner;
 	Gtk::SpinButton _semitones_spinner;
 };
+
+#endif /* __ardour_transpose_dialog_h__ */
