@@ -3,7 +3,7 @@
  * Copyright (C) 2005-2006 Taybin Rutkin <taybin@taybin.com>
  * Copyright (C) 2008-2011 David Robillard <d@drobilla.net>
  * Copyright (C) 2009-2011 Carl Hetherington <carl@carlh.net>
- * Copyright (C) 2014-2019 Robin Gareus <robin@gareus.org>
+ * Copyright (C) 2014-2021 Robin Gareus <robin@gareus.org>
  * Copyright (C) 2018 Ben Loftis <ben@harrisonconsoles.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -33,10 +33,16 @@
 #include <string>
 #include <set>
 #include <boost/utility.hpp>
+#include <boost/container/set.hpp>
 
 #include "ardour/libardour_visibility.h"
 #include "ardour/types.h"
 #include "ardour/plugin.h"
+#include "ardour/plugin_scan_result.h"
+
+#ifdef AUDIOUNIT_SUPPORT
+class CAComponentDescription;
+#endif
 
 namespace ARDOUR {
 
@@ -46,10 +52,21 @@ class Plugin;
 struct VST3Info;
 #endif
 
+#if (defined WINDOWS_VST_SUPPORT || defined MACVST_SUPPORT || defined LXVST_SUPPORT)
+struct VST2Info;
+#endif
+
+#ifdef AUDIOUNIT_SUPPORT
+struct AUv2Info;
+struct AUv2DescStr;
+#endif
+
+
 class LIBARDOUR_API PluginManager : public boost::noncopyable {
 public:
 	static PluginManager& instance();
-	static std::string scanner_bin_path;
+	static std::string auv2_scanner_bin_path;
+	static std::string vst2_scanner_bin_path;
 	static std::string vst3_scanner_bin_path;
 
 	~PluginManager ();
@@ -64,8 +81,13 @@ public:
 	const ARDOUR::PluginInfoList& vst3_plugin_info ();
 
 	void refresh (bool cache_only = false);
-	void cancel_plugin_scan();
-	void cancel_plugin_timeout();
+
+	void cancel_scan_all ();
+	void cancel_scan_one ();
+	void cancel_scan_timeout_all ();
+	void cancel_scan_timeout_one ();
+	void enable_scan_timeout ();
+
 	void clear_vst_cache ();
 	void clear_vst_blacklist ();
 	void clear_au_cache ();
@@ -76,6 +98,19 @@ public:
 	const std::string get_default_windows_vst_path() const { return windows_vst_path; }
 	const std::string get_default_lxvst_path() const { return lxvst_path; }
 
+	static uint32_t cache_version ();
+	bool cache_valid () const;
+
+	void scan_log (std::vector<boost::shared_ptr<PluginScanLogEntry> >&) const;
+	void clear_stale_log ();
+
+	bool whitelist (ARDOUR::PluginType, std::string const&, bool force);
+	void blacklist (ARDOUR::PluginType, std::string const&);
+	static std::string cache_file (ARDOUR::PluginType, std::string const&);
+
+	bool rescan_plugin (ARDOUR::PluginType, std::string const&, size_t num = 0, size_t den = 1);
+	void rescan_faulty ();
+
 	/* always return LXVST for any VST subtype */
 	static PluginType to_generic_vst (const PluginType);
 
@@ -84,8 +119,7 @@ public:
 	 */
 	static std::string plugin_type_name (const PluginType, bool short_name = true);
 
-	bool cancelled () const { return _cancel_scan; }
-	bool no_timeout () const { return _cancel_timeout; }
+	bool cancelled () const { return _cancel_scan_all || _cancel_scan_one; }
 
 	void reset_stats ();
 	void stats_use_plugin (PluginInfoPtr const&);
@@ -101,11 +135,13 @@ public:
 
 	std::string user_plugin_metadata_dir () const;
 	void save_statuses ();
-	void set_status (ARDOUR::PluginType type, std::string unique_id, PluginStatusType status);
+	void set_status (ARDOUR::PluginType type, std::string const& unique_id, PluginStatusType status);
 	PluginStatusType get_status (const PluginInfoPtr&) const;
 
 	void save_tags ();
 
+	std::string dump_untagged_plugins ();
+
 	bool load_plugin_order_file (XMLNode &n) const;  //returns TRUE if the passed-in node has valid info
 	void save_plugin_order_file (XMLNode &elem) const;
 
@@ -127,12 +163,17 @@ public:
 	};
 	std::vector<std::string> get_all_tags (enum TagFilter) const;
 
-	/** plugins were added to or removed from one of the PluginInfoLists */
+	/** plugins were added to or removed from one of the PluginInfoLists
+	 * This implies PluginScanLogChanged.
+	 */
 	PBD::Signal0<void> PluginListChanged;
 
 	/** Plugin Statistics (use-count, recently-used) changed */
 	PBD::Signal0<void> PluginStatsChanged;
 
+	/** Plugin ScanLog changed */
+	PBD::Signal0<void> PluginScanLogChanged;
+
 	/** A single plugin's Hidden/Favorite status changed */
 	PBD::Signal3<void, ARDOUR::PluginType, std::string, PluginStatusType> PluginStatusChanged; //PluginType t, string id, string tag
 
@@ -140,6 +181,26 @@ public:
 	PBD::Signal3<void, ARDOUR::PluginType, std::string, std::string> PluginTagChanged; //PluginType t, string id, string tag
 
 private:
+	typedef boost::shared_ptr<PluginScanLogEntry> PSLEPtr;
+
+	struct PSLEPtrSort {
+		bool operator() (PSLEPtr const& a, PSLEPtr const& b) const {
+			return *a < *b;
+		}
+	};
+
+	typedef boost::container::set<PSLEPtr, PSLEPtrSort> PluginScanLog;
+	PluginScanLog _plugin_scan_log;
+
+	PSLEPtr scan_log_entry (PluginType const type, std::string const& path) {
+		PSLEPtr psl = PSLEPtr (new PluginScanLogEntry (type, path));
+		PluginScanLog::iterator i = _plugin_scan_log.find (psl);
+		if (i == _plugin_scan_log.end ()) {
+			_plugin_scan_log.insert (psl);
+			i = _plugin_scan_log.find (psl);
+		}
+		return *i;
+	}
 
 	struct PluginTag {
 		PluginType const  type;
@@ -229,27 +290,39 @@ private:
 	std::string windows_vst_path;
 	std::string lxvst_path;
 
-	bool _cancel_scan;
-	bool _cancel_timeout;
+	bool _cancel_scan_one;
+	bool _cancel_scan_all;
+	bool _cancel_scan_timeout_one;
+	bool _cancel_scan_timeout_all;
+	bool _enable_scan_timeout;
+
+	void reset_scan_cancel_state (bool single = false);
+
+	bool no_timeout () const { return _cancel_scan_timeout_one || _cancel_scan_timeout_all; }
 
 	void detect_name_ambiguities (ARDOUR::PluginInfoList*);
 	void detect_type_ambiguities (ARDOUR::PluginInfoList&);
 
+	void detect_ambiguities ();
+
 	void conceal_duplicates (ARDOUR::PluginInfoList*, ARDOUR::PluginInfoList*);
 
 	void load_statuses ();
 	void load_tags ();
 	void load_stats ();
 
+	void load_scanlog ();
+	void save_scanlog ();
+
 	std::string sanitize_tag (const std::string) const;
 
 	void ladspa_refresh ();
 	void lua_refresh ();
 	void lua_refresh_cb ();
-	void windows_vst_refresh (bool cache_only = false);
-	void mac_vst_refresh (bool cache_only = false);
-	void lxvst_refresh (bool cache_only = false);
-	void vst3_refresh (bool cache_only = false);
+	void windows_vst_refresh (bool cache_only);
+	void mac_vst_refresh (bool cache_only);
+	void lxvst_refresh (bool cache_only);
+	void vst3_refresh (bool cache_only);
 
 	void add_lrdf_data (const std::string &path);
 	void add_ladspa_presets ();
@@ -258,26 +331,32 @@ private:
 	void add_lxvst_presets ();
 	void add_presets (std::string domain);
 
+#ifdef AUDIOUNIT_SUPPORT
 	void au_refresh (bool cache_only = false);
+	void auv2_plugin (CAComponentDescription const&, AUv2Info const&);
+	int  auv2_discover (AUv2DescStr const&, bool);
+	bool run_auv2_scanner_app (CAComponentDescription const&, AUv2DescStr const&, PSLEPtr) const;
+#endif
 
+	void lv2_plugin (std::string const&, PluginScanLogEntry::PluginScanResult, std::string const&, bool);
 	void lv2_refresh ();
 
 	int windows_vst_discover_from_path (std::string path, bool cache_only = false);
-	int windows_vst_discover (std::string path, bool cache_only = false);
-
 	int mac_vst_discover_from_path (std::string path, bool cache_only = false);
-	int mac_vst_discover (std::string path, bool cache_only = false);
+	int lxvst_discover_from_path (std::string path, bool cache_only = false);
+#if (defined WINDOWS_VST_SUPPORT || defined MACVST_SUPPORT || defined LXVST_SUPPORT)
+	bool vst2_plugin (std::string const& module_path, ARDOUR::PluginType, VST2Info const&);
+	bool run_vst2_scanner_app (std::string bundle_path, PSLEPtr) const;
+	int vst2_discover (std::string path, ARDOUR::PluginType, bool cache_only = false);
+#endif
 
 	int vst3_discover_from_path (std::string const& path, bool cache_only = false);
 	int vst3_discover (std::string const& path, bool cache_only = false);
 #ifdef VST3_SUPPORT
-	void vst3_plugin (std::string const& module_path, VST3Info const&);
-	bool run_vst3_scanner_app (std::string bundle_path) const;
+	void vst3_plugin (std::string const&, std::string const&, VST3Info const&);
+	bool run_vst3_scanner_app (std::string bundle_path, PSLEPtr) const;
 #endif
 
-	int lxvst_discover_from_path (std::string path, bool cache_only = false);
-	int lxvst_discover (std::string path, bool cache_only = false);
-
 	int ladspa_discover (std::string path);
 
 	std::string get_ladspa_category (uint32_t id);
