@@ -36,12 +36,12 @@
 
 #include "pbd/basename.h"
 #include "pbd/xml++.h"
-#include "pbd/stacktrace.h"
 #include "pbd/enumwriter.h"
 #include "pbd/convert.h"
 
 #include "evoral/Curve.h"
 
+#include "ardour/analysis_graph.h"
 #include "ardour/audioregion.h"
 #include "ardour/session.h"
 #include "ardour/dB.h"
@@ -1390,14 +1390,6 @@ AudioRegion::set_scale_amplitude (gain_t g)
 
 	_scale_amplitude = g;
 
-	/* tell the diskstream we're in */
-
-	if (pl) {
-		pl->ContentsChanged();
-	}
-
-	/* tell everybody else */
-
 	send_change (PropertyChange (Properties::scale_amplitude));
 }
 
@@ -1479,6 +1471,37 @@ AudioRegion::rms (Progress* p) const
 	return sqrt (2. * rms / (double)(total * n_chan));
 }
 
+bool
+AudioRegion::loudness (float& tp, float& i, float& s, float& m, Progress* p) const
+{
+	ARDOUR::AnalysisGraph ag (&_session);
+	tp = i = s = m = -200;
+
+	ag.set_total_samples (_length);
+	ag.analyze_region (this, true, p);
+
+	if (p && p->cancelled ()) {
+		return false;
+	}
+
+	AnalysisResults const& ar (ag.results ());
+	if (ar.size() != 1) {
+		return false;
+	}
+	ExportAnalysisPtr eap (ar.begin ()->second);
+
+	if (eap->have_dbtp) {
+		tp = eap->truepeak;
+	}
+	if (eap->have_loudness) {
+		i = eap->integrated_loudness;
+		s = eap->max_loudness_short;
+		m = eap->max_loudness_momentary;
+	}
+
+	return eap->have_dbtp || eap->have_loudness;
+}
+
 /** Normalize using a given maximum amplitude and target, so that region
  *  _scale_amplitude becomes target / max_amplitude.
  */
@@ -1670,7 +1693,7 @@ AudioRegion::remove_transient (samplepos_t where)
 	if (!_onsets.empty ()) {
 		const samplepos_t p = where - _position;
 		AnalysisFeatureList::iterator i = std::find (_onsets.begin (), _onsets.end (), p);
-		if (i != _transients.end ()) {
+		if (i != _onsets.end ()) {
 			_onsets.erase (i);
 			changed = true;
 		}
