@@ -51,7 +51,6 @@ using namespace PBD;
 Auditioner::Auditioner (Session& s)
 	: Track (s, "auditioner", PresentationInfo::Auditioner)
 	, current_sample (0)
-	, _auditioning (0)
 	, length (0)
 	, _seek_sample (-1)
 	, _seeking (false)
@@ -61,6 +60,7 @@ Auditioner::Auditioner (Session& s)
 	, _queue_panic (false)
 	, _import_position (0)
 {
+	g_atomic_int_set (&_auditioning, 0);
 }
 
 int
@@ -74,7 +74,6 @@ Auditioner::init ()
 		return -1;
 	}
 
-	_output->add_port ("", this, DataType::MIDI);
 	use_new_playlist (DataType::MIDI);
 
 	if (!audition_synth_info) {
@@ -131,6 +130,14 @@ void
 Auditioner::load_synth (bool need_lock)
 {
 	unload_synth(need_lock);
+
+	if (!audition_synth_info) {
+		lookup_fallback_synth ();
+	}
+
+	if (!audition_synth_info) {
+		return;
+	}
 	
 	boost::shared_ptr<Plugin> p = audition_synth_info->load (_session);
 	if (p) {
@@ -304,7 +311,7 @@ Auditioner::audition_region (boost::shared_ptr<Region> region)
 		_import_position = 0;
 
 		/* copy it */
-		the_region = boost::dynamic_pointer_cast<AudioRegion> (RegionFactory::create (region));
+		the_region = boost::dynamic_pointer_cast<AudioRegion> (RegionFactory::create (region, false));
 		the_region->set_position (0);
 
 		_disk_reader->midi_playlist()->drop_regions ();
@@ -330,7 +337,7 @@ Auditioner::audition_region (boost::shared_ptr<Region> region)
 		_import_position = region->position();
 
 		/* copy it */
-		midi_region = (boost::dynamic_pointer_cast<MidiRegion> (RegionFactory::create (region)));
+		midi_region = (boost::dynamic_pointer_cast<MidiRegion> (RegionFactory::create (region, false)));
 		midi_region->set_position (_import_position);
 
 		_disk_reader->audio_playlist()->drop_regions();
