@@ -46,26 +46,28 @@ using namespace std;
 ARDOUR::samplecnt_t DiskWriter::_chunk_samples = DiskWriter::default_chunk_samples ();
 PBD::Signal0<void> DiskWriter::Overrun;
 
-DiskWriter::DiskWriter (Session& s, string const & str, DiskIOProcessor::Flag f)
-	: DiskIOProcessor (s, str, f)
-	, _record_enabled (0)
-	, _record_safe (0)
-	, _capture_start_sample (0)
+DiskWriter::DiskWriter (Session& s, Track& t, string const & str, DiskIOProcessor::Flag f)
+	: DiskIOProcessor (s, t, X_("recorder:") + str, f)
 	, _capture_captured (0)
 	, _was_recording (false)
+	, _xrun_flag (false)
 	, _first_recordable_sample (max_samplepos)
 	, _last_recordable_sample (max_samplepos)
 	, _last_possibly_recording (0)
 	, _alignment_style (ExistingMaterial)
 	, _note_mode (Sustained)
-	, _samples_pending_write (0)
-	, _num_captured_loops (0)
 	, _accumulated_capture_offset (0)
 	, _transport_looped (false)
 	, _transport_loop_sample (0)
 	, _gui_feed_buffer(AudioEngine::instance()->raw_buffer_size (DataType::MIDI))
 {
 	DiskIOProcessor::init ();
+	_xruns.reserve (128);
+
+	g_atomic_int_set (&_record_enabled, 0);
+	g_atomic_int_set (&_record_safe, 0);
+	g_atomic_int_set (&_samples_pending_write, 0);
+	g_atomic_int_set (&_num_captured_loops, 0);
 }
 
 DiskWriter::~DiskWriter ()
@@ -88,7 +90,7 @@ DiskWriter::default_chunk_samples ()
 std::string
 DiskWriter::display_name () const
 {
-	return std::string (_ ("recorder"));
+	return std::string (_("Recorder"));
 }
 
 void
@@ -127,18 +129,33 @@ DiskWriter::set_write_source_name (string const & str)
 	return true;
 }
 
+std::string
+DiskWriter::write_source_name () const
+{
+	if (!_write_source_name.empty ()) {
+		return _write_source_name;
+	}
+
+	std::string const& n (name ());
+	if (n.find (X_("recorder:")) == 0 && n.size () > 9) {
+		return n.substr (9);
+	}
+	return n;
+}
+
 void
 DiskWriter::check_record_status (samplepos_t transport_sample, double speed, bool can_record)
 {
-	int possibly_recording;
-	const int transport_rolling = 0x4;
-	const int track_rec_enabled = 0x2;
-	const int global_rec_enabled = 0x1;
-	const int fully_rec_enabled = (transport_rolling |track_rec_enabled | global_rec_enabled);
+	static const int transport_rolling = 0x4;
+	static const int track_rec_enabled = 0x2;
+	static const int global_rec_enabled = 0x1;
+
+	static const int rec_ready = (track_rec_enabled | global_rec_enabled);
+	static const int fully_rec_enabled = (transport_rolling |track_rec_enabled | global_rec_enabled);
 
 	/* merge together the 3 factors that affect record status, and compute what has changed. */
 
-	possibly_recording = (speed != 0.0f ? 4 : 0)  | (record_enabled() ? 2 : 0) | (can_record ? 1 : 0);
+	int possibly_recording = (speed != 0.0f ? 4 : 0)  | (record_enabled() ? 2 : 0) | (can_record ? 1 : 0);
 
 	if (possibly_recording == _last_possibly_recording) {
 		return;
@@ -153,17 +170,19 @@ DiskWriter::check_record_status (samplepos_t transport_sample, double speed, boo
 		Location* loc;
 		if  (_session.config.get_punch_in () && 0 != (loc = _session.locations()->auto_punch_location ())) {
 			_capture_start_sample = loc->start ();
+		} else if (_loop_location) {
+			_capture_start_sample = _loop_location->start ();
+			if (_last_possibly_recording & transport_rolling) {
+				_accumulated_capture_offset = _playback_offset + transport_sample - _session.transport_sample (); // + rec_offset;
+			}
 		} else {
 			_capture_start_sample = _session.transport_sample ();
 		}
 
-		_first_recordable_sample = _capture_start_sample;
+		_first_recordable_sample = _capture_start_sample.value ();
 
 		if (_alignment_style == ExistingMaterial) {
 			_first_recordable_sample += _capture_offset + _playback_offset;
-			if (_accumulated_capture_offset == 0) {
-				_accumulated_capture_offset = _playback_offset;
-			}
 		}
 
 		if  (_session.config.get_punch_out () && 0 != (loc = _session.locations()->auto_punch_location ())) {
@@ -184,7 +203,7 @@ DiskWriter::check_record_status (samplepos_t transport_sample, double speed, boo
 		                                                      name(),
 		                                                      transport_sample,
 		                                                      _session.transport_sample(),
-																													_capture_start_sample,
+																													_capture_start_sample.value (),
 																													_first_recordable_sample,
 																													_last_recordable_sample,
 		                                                      _input_latency,
@@ -195,8 +214,20 @@ DiskWriter::check_record_status (samplepos_t transport_sample, double speed, boo
 		                                                      _session.worst_input_latency()));
 
 
-		prepare_record_status (_capture_start_sample);
-
+	} else if  (!_capture_start_sample) {
+		/* set _capture_start_sample early on to calculate MIDI _accumulated_capture_offset */
+		Location* loc;
+		if  (_session.config.get_punch_in () && 0 != (loc = _session.locations()->auto_punch_location ())) {
+			_capture_start_sample = loc->start ();
+		} else if (_loop_location) {
+			_capture_start_sample = _loop_location->start ();
+		} else if ((possibly_recording & rec_ready) == rec_ready) {
+			/* count-in, pre-roll */
+			_capture_start_sample = _session.transport_sample ();
+		} else if (possibly_recording) {
+			/* already rolling, manual punch rec-arm/rec-en */
+			_accumulated_capture_offset = _playback_offset;
+		}
 	}
 
 	_last_possibly_recording = possibly_recording;
@@ -283,12 +314,30 @@ DiskWriter::get_capture_start_sample (uint32_t n) const
 	if (capture_info.size() > n) {
 		/* this is a completed capture */
 		return capture_info[n]->start;
-	} else {
+	} else if (_capture_start_sample) {
 		/* this is the currently in-progress capture */
-		return _capture_start_sample;
+		return _capture_start_sample.value ();
+	} else {
+		/* pre-roll, count-in etc */
+		return _session.transport_sample(); /* mild lie */
 	}
 }
 
+samplepos_t
+DiskWriter::current_capture_start () const
+{
+	if (!_capture_start_sample) {
+		return _session.transport_sample(); /* mild lie */
+	}
+	return _capture_start_sample.value ();
+}
+
+samplepos_t
+DiskWriter::current_capture_end () const
+{
+	return current_capture_start () + _capture_captured;
+}
+
 ARDOUR::samplecnt_t
 DiskWriter::get_captured_samples (uint32_t n) const
 {
@@ -321,7 +370,7 @@ DiskWriter::state ()
 {
 	XMLNode& node (DiskIOProcessor::state ());
 	node.set_property (X_("type"), X_("diskwriter"));
-	node.set_property (X_("record-safe"), (_record_safe ? X_("yes" : "no")));
+	node.set_property (X_("record-safe"), record_safe ());
 	return node;
 }
 
@@ -332,9 +381,9 @@ DiskWriter::set_state (const XMLNode& node, int version)
 		return -1;
 	}
 
-	if (!node.get_property (X_("record-safe"), _record_safe)) {
-		_record_safe = false;
-	}
+	int rec_safe = 0;
+	node.get_property (X_("record-safe"), rec_safe);
+	g_atomic_int_set (&_record_safe, rec_safe);
 
 	reset_write_sources (false, true);
 
@@ -352,11 +401,6 @@ DiskWriter::non_realtime_locate (samplepos_t position)
 }
 
 
-void
-DiskWriter::prepare_record_status (samplepos_t _capture_start_sample)
-{
-}
-
 /** Do some record stuff [not described in this comment!]
  *
  *  Also:
@@ -370,6 +414,7 @@ DiskWriter::run (BufferSet& bufs, samplepos_t start_sample, samplepos_t end_samp
                  double speed, pframes_t nframes, bool result_required)
 {
 	if (!_active && !_pending_active) {
+		_xrun_flag = false;
 		return;
 	}
 	_active = _pending_active;
@@ -421,6 +466,7 @@ DiskWriter::run (BufferSet& bufs, samplepos_t start_sample, samplepos_t end_samp
 	check_record_status (start_sample, speed, can_record);
 
 	if (nframes == 0) {
+		_xrun_flag = false;
 		return;
 	}
 
@@ -444,6 +490,7 @@ DiskWriter::run (BufferSet& bufs, samplepos_t start_sample, samplepos_t end_samp
 
 		if (rec_nframes && !_was_recording) {
 			_capture_captured = 0;
+			_xrun_flag = false;
 
 			if (loop_loc) {
 				/* Loop recording, so pretend the capture started at the loop
@@ -451,9 +498,14 @@ DiskWriter::run (BufferSet& bufs, samplepos_t start_sample, samplepos_t end_samp
 				   at the loop start and can handle time wrapping around.
 				   Otherwise, start the source right now as usual.
 				*/
-				_capture_captured     = start_sample - loop_start;
+				_capture_captured     = start_sample - loop_start + rec_offset;
 				_capture_start_sample = loop_start;
 				_first_recordable_sample = loop_start;
+
+				if (_alignment_style == ExistingMaterial) {
+					_capture_captured  -= _playback_offset + _capture_offset;
+				}
+
 				if (_capture_captured > 0) {
 					/* when enabling record while already looping,
 					 * zero fill region back to loop-start.
@@ -468,11 +520,12 @@ DiskWriter::run (BufferSet& bufs, samplepos_t start_sample, samplepos_t end_samp
 			}
 
 			if (_midi_write_source) {
-				_midi_write_source->mark_write_starting_now (_capture_start_sample, _capture_captured, loop_length);
+				assert (_capture_start_sample);
+				_midi_write_source->mark_write_starting_now (_capture_start_sample.value (), _capture_captured, loop_length);
 			}
 
-			g_atomic_int_set (const_cast<gint*> (&_samples_pending_write), 0);
-			g_atomic_int_set (const_cast<gint*> (&_num_captured_loops), 0);
+			g_atomic_int_set (&_samples_pending_write, 0);
+			g_atomic_int_set (&_num_captured_loops, 0);
 
 			_was_recording = true;
 
@@ -484,7 +537,7 @@ DiskWriter::run (BufferSet& bufs, samplepos_t start_sample, samplepos_t end_samp
 		 */
 		if (rec_nframes) {
 			_accumulated_capture_offset += rec_offset;
-		} else if (start_sample >= _capture_start_sample) {
+		} else if (_capture_start_sample && start_sample >= _capture_start_sample.value ()) {
 			_accumulated_capture_offset += nframes;
 		}
 
@@ -520,6 +573,8 @@ DiskWriter::run (BufferSet& bufs, samplepos_t start_sample, samplepos_t end_samp
 					DEBUG_TRACE (DEBUG::Butler, string_compose ("%1 overrun in %2, rec_nframes = %3 total space = %4\n",
 					                                            DEBUG_THREAD_SELF, name(), rec_nframes, total));
 					Overrun ();
+					_xruns.push_back (_capture_captured);
+					_xrun_flag = false;
 					return;
 				}
 
@@ -540,7 +595,7 @@ DiskWriter::run (BufferSet& bufs, samplepos_t start_sample, samplepos_t end_samp
 
 			// Pump entire port buffer into the ring buffer (TODO: split cycles?)
 			MidiBuffer& buf    = bufs.get_midi (0);
-			boost::shared_ptr<MidiTrack> mt = boost::dynamic_pointer_cast<MidiTrack>(_track);
+			MidiTrack* mt = dynamic_cast<MidiTrack*>(&_track);
 			MidiChannelFilter* filter = mt ? &mt->capture_filter() : 0;
 
 			assert (buf.size() == 0 || _midi_buf);
@@ -574,7 +629,7 @@ DiskWriter::run (BufferSet& bufs, samplepos_t start_sample, samplepos_t end_samp
 				   reconstruct their actual time; future clever MIDI looping should
 				   probably be implemented in the source instead of here.
 				*/
-				const samplecnt_t loop_offset = _num_captured_loops * loop_length;
+				const samplecnt_t loop_offset = g_atomic_int_get (&_num_captured_loops) * loop_length;
 				const samplepos_t event_time = start_sample + loop_offset - _accumulated_capture_offset + ev.time();
 				if (event_time < 0 || event_time < _first_recordable_sample) {
 					/* Event out of range, skip */
@@ -600,7 +655,7 @@ DiskWriter::run (BufferSet& bufs, samplepos_t start_sample, samplepos_t end_samp
 				}
 			}
 
-			g_atomic_int_add (const_cast<gint*>(&_samples_pending_write), nframes);
+			g_atomic_int_add (&_samples_pending_write, nframes);
 
 			if (buf.size() != 0) {
 				Glib::Threads::Mutex::Lock lm (_gui_feed_buffer_mutex, Glib::Threads::TRY_LOCK);
@@ -613,9 +668,12 @@ DiskWriter::run (BufferSet& bufs, samplepos_t start_sample, samplepos_t end_samp
 
 					for (MidiBuffer::iterator i = buf.begin(); i != buf.end(); ++i) {
 						/* This may fail if buf is larger than _gui_feed_buffer, but it's not really
-						   the end of the world if it does.
-						*/
-						_gui_feed_buffer.push_back ((*i).time() + start_sample, Evoral::MIDI_EVENT, (*i).size(), (*i).buffer());
+						 * the end of the world if it does.
+						 */
+						samplepos_t mpos = (*i).time() + start_sample - _accumulated_capture_offset;
+						if (mpos >= _first_recordable_sample) {
+							_gui_feed_buffer.push_back (mpos, Evoral::MIDI_EVENT, (*i).size(), (*i).buffer());
+						}
 					}
 				}
 
@@ -623,6 +681,12 @@ DiskWriter::run (BufferSet& bufs, samplepos_t start_sample, samplepos_t end_samp
 			}
 		}
 
+		if (_xrun_flag) {
+			/* There still are `Port::resampler_quality () -1` samples in the resampler
+			 * buffer from before the xrun. */
+			_xruns.push_back (_capture_captured + Port::resampler_quality () - 1);
+		}
+
 		_capture_captured += rec_nframes;
 		DEBUG_TRACE (DEBUG::CaptureAlignment, string_compose ("%1 now captured %2 (by %3)\n", name(), _capture_captured, rec_nframes));
 
@@ -633,9 +697,14 @@ DiskWriter::run (BufferSet& bufs, samplepos_t start_sample, samplepos_t end_samp
 		if (_was_recording) {
 			finish_capture (c);
 			_accumulated_capture_offset = 0;
+			_capture_start_sample.reset ();
+			_last_possibly_recording = 0; // re-init
 		}
 	}
 
+	/* clear xrun flag */
+	_xrun_flag = false;
+
 	/* AUDIO BUTLER REQUIRED CODE */
 
 	if (_playlists[DataType::AUDIO] && !c->empty()) {
@@ -657,6 +726,7 @@ void
 DiskWriter::finish_capture (boost::shared_ptr<ChannelList> c)
 {
 	_was_recording = false;
+	_xrun_flag = false;
 	_first_recordable_sample = max_samplepos;
 	_last_recordable_sample = max_samplepos;
 
@@ -664,22 +734,25 @@ DiskWriter::finish_capture (boost::shared_ptr<ChannelList> c)
 		return;
 	}
 
-	CaptureInfo* ci = new CaptureInfo;
+	CaptureInfo* ci = new CaptureInfo ();
 
-	ci->start =  _capture_start_sample;
+	assert (_capture_start_sample);
+	ci->start   =  _capture_start_sample.value ();
 	ci->samples = _capture_captured;
+	ci->xruns   = _xruns;
+	_xruns.clear ();
 
 	if (_loop_location) {
 		samplepos_t loop_start  = 0;
 		samplepos_t loop_end    = 0;
 		samplepos_t loop_length = 0;
 		get_location_times (_loop_location, &loop_start, &loop_end, &loop_length);
-		ci->loop_offset = _num_captured_loops * loop_length;
+		ci->loop_offset = g_atomic_int_get (&_num_captured_loops) * loop_length;
 	} else {
 		ci->loop_offset = 0;
 	}
 
-	DEBUG_TRACE (DEBUG::CaptureAlignment, string_compose ("Finish capture, add new CI, %1 + %2\n", ci->start, ci->samples));
+	DEBUG_TRACE (DEBUG::CaptureAlignment, string_compose ("Finish capture, add new CI, %1 + %2 Loop-off %3\n", ci->start, ci->samples, ci->loop_offset));
 
 	/* XXX theoretical race condition here. Need atomic exchange ?
 	   However, the circumstances when this is called right
@@ -707,6 +780,12 @@ DiskWriter::get_gui_feed_buffer () const
 	return b;
 }
 
+void
+DiskWriter::mark_capture_xrun ()
+{
+	_xrun_flag = true;
+}
+
 void
 DiskWriter::set_record_enabled (bool yn)
 {
@@ -813,7 +892,14 @@ DiskWriter::configuration_changed ()
 }
 
 int
-DiskWriter::seek (samplepos_t sample, bool /*complete_refill*/)
+DiskWriter::seek (samplepos_t /*sample*/, bool /*complete_refill*/)
+{
+	reset_capture ();
+	return 0;
+}
+
+void
+DiskWriter::reset_capture ()
 {
 	uint32_t n;
 	ChannelList::iterator chan;
@@ -826,12 +912,9 @@ DiskWriter::seek (samplepos_t sample, bool /*complete_refill*/)
 	if (_midi_buf) {
 		_midi_buf->reset ();
 	}
-	g_atomic_int_set(&_samples_read_from_ringbuffer, 0);
-	g_atomic_int_set(&_samples_written_to_ringbuffer, 0);
 
-	playback_sample = sample;
-
-	return 0;
+	_accumulated_capture_offset = 0;
+	_capture_start_sample.reset ();
 }
 
 int
@@ -906,7 +989,7 @@ DiskWriter::do_flush (RunContext ctxt, bool force_flush)
 
 	if (_midi_write_source && _midi_buf) {
 
-		const samplecnt_t total = g_atomic_int_get(const_cast<gint*> (&_samples_pending_write));
+		const samplecnt_t total = g_atomic_int_get(&_samples_pending_write);
 
 		if (total == 0 ||
 		    _midi_buf->read_space() == 0 ||
@@ -942,7 +1025,7 @@ DiskWriter::do_flush (RunContext ctxt, bool force_flush)
 				error << string_compose(_("MidiDiskstream %1: cannot write to disk"), id()) << endmsg;
 				return -1;
 			}
-			g_atomic_int_add(const_cast<gint*> (&_samples_pending_write), -to_write);
+			g_atomic_int_add(&_samples_pending_write, -to_write);
 		}
 	}
 
@@ -1061,11 +1144,9 @@ DiskWriter::transport_stopped_wallclock (struct tm& when, time_t twhen, bool abo
 {
 	bool more_work = true;
 	int err = 0;
-	samplecnt_t total_capture;
 	SourceList audio_srcs;
 	SourceList midi_srcs;
 	ChannelList::iterator chan;
-	vector<CaptureInfo*>::iterator ci;
 	boost::shared_ptr<ChannelList> c = channels.reader();
 	uint32_t n = 0;
 	bool mark_write_completed = false;
@@ -1098,6 +1179,7 @@ DiskWriter::transport_stopped_wallclock (struct tm& when, time_t twhen, bool abo
 	}
 
 	if (abort_capture) {
+		_xruns.clear ();
 
 		for (ChannelList::iterator chan = c->begin(); chan != c->end(); ++chan) {
 
@@ -1120,10 +1202,6 @@ DiskWriter::transport_stopped_wallclock (struct tm& when, time_t twhen, bool abo
 		goto out;
 	}
 
-	for (total_capture = 0, ci = capture_info.begin(); ci != capture_info.end(); ++ci) {
-		total_capture += (*ci)->samples;
-	}
-
 	/* figure out the name for this take */
 
 	for (n = 0, chan = c->begin(); chan != c->end(); ++chan, ++n) {
@@ -1133,7 +1211,7 @@ DiskWriter::transport_stopped_wallclock (struct tm& when, time_t twhen, bool abo
 		if (as) {
 			audio_srcs.push_back (as);
 			as->update_header (capture_info.front()->start, when, twhen);
-			as->set_captured_for (_track->name());
+			as->set_captured_for (_track.name());
 			as->mark_immutable ();
 
 			Glib::DateTime tm (Glib::DateTime::create_now_local (mktime (&when)));
@@ -1148,13 +1226,11 @@ DiskWriter::transport_stopped_wallclock (struct tm& when, time_t twhen, bool abo
 
 		if (_midi_write_source) {
 			midi_srcs.push_back (_midi_write_source);
-			_midi_write_source->set_captured_for (_track->name());
+			_midi_write_source->set_captured_for (_track.name());
 		}
 
 		(*chan)->write_source->stamp (twhen);
-
-		/* "re-announce the source to the world */
-		Source::SourcePropertyChanged ((*chan)->write_source);
+		(*chan)->write_source->set_captured_xruns (capture_info.front()->xruns);
 	}
 
 
@@ -1185,15 +1261,19 @@ DiskWriter::transport_stopped_wallclock (struct tm& when, time_t twhen, bool abo
 		midi_srcs.push_back (_midi_write_source);
 
 		_midi_write_source->set_natural_position (capture_info.front()->start);
-		_midi_write_source->set_captured_for (_name);
+		_midi_write_source->set_captured_for (_track.name ());
 
 		Glib::DateTime tm (Glib::DateTime::create_now_local (mktime (&when)));
 		_midi_write_source->set_take_id (tm.format ("%F %H.%M.%S"));
 
 		/* set length in beats to entire capture length */
 
-		BeatsSamplesConverter converter (_session.tempo_map(), capture_info.front()->start);
-		const Temporal::Beats total_capture_beats = converter.from (total_capture);
+
+		Temporal::Beats total_capture_beats;
+		for (vector<CaptureInfo*>::iterator ci = capture_info.begin(); ci != capture_info.end(); ++ci) {
+			BeatsSamplesConverter converter (_session.tempo_map(), (*ci)->start);
+			total_capture_beats += converter.from ((*ci)->samples);
+		}
 		_midi_write_source->set_length_beats (total_capture_beats);
 
 		/* flush to disk: this step differs from the audio path,
@@ -1207,22 +1287,20 @@ DiskWriter::transport_stopped_wallclock (struct tm& when, time_t twhen, bool abo
 	_last_capture_sources.insert (_last_capture_sources.end(), midi_srcs.begin(), midi_srcs.end());
 
 
-	if (_track) {
-		_track->use_captured_sources (audio_srcs, capture_info);
-		_track->use_captured_sources (midi_srcs, capture_info);
-	}
+	_track.use_captured_sources (audio_srcs, capture_info);
+	_track.use_captured_sources (midi_srcs, capture_info);
 
 	mark_write_completed = true;
 
   out:
 	reset_write_sources (mark_write_completed);
 
-	for (ci = capture_info.begin(); ci != capture_info.end(); ++ci) {
+	for (vector<CaptureInfo*>::iterator ci = capture_info.begin(); ci != capture_info.end(); ++ci) {
 		delete *ci;
 	}
 
 	capture_info.clear ();
-	_capture_start_sample = 0;
+	reset_capture ();
 }
 
 void
@@ -1261,7 +1339,7 @@ DiskWriter::loop (samplepos_t transport_sample)
 	   the Source and/or entirely after the capture is finished.
 	*/
 	if (_was_recording) {
-		g_atomic_int_add(const_cast<gint*> (&_num_captured_loops), 1);
+		g_atomic_int_add (&_num_captured_loops, 1);
 	}
 }
 
@@ -1307,7 +1385,7 @@ DiskWriter::steal_write_source_name ()
 		 */
 
 		try {
-			string new_path = _session.new_midi_source_path (name());
+			string new_path = _session.new_midi_source_path (write_source_name ());
 
 			if (_midi_write_source->rename (new_path)) {
 				return string();
