@@ -138,7 +138,7 @@ ExportHandler::add_export_config (ExportTimespanPtr timespan, ExportChannelConfi
 	return true;
 }
 
-void
+int
 ExportHandler::do_export ()
 {
 	/* Count timespans */
@@ -164,28 +164,29 @@ ExportHandler::do_export ()
 	/* Start export */
 
 	Glib::Threads::Mutex::Lock l (export_status->lock());
-	start_timespan ();
+	return start_timespan ();
 }
 
-void
+int
 ExportHandler::start_timespan ()
 {
-	export_status->timespan++;
-
 	/* stop freewheeling and wait for latency callbacks */
 	if (AudioEngine::instance()->freewheeling ()) {
 		AudioEngine::instance()->freewheel (false);
 		do {
 			Glib::usleep (AudioEngine::instance()->usecs_per_cycle ());
 		} while (AudioEngine::instance()->freewheeling ());
+		session.reset_xrun_count ();
 	}
 
 	if (config_map.empty()) {
 		// freewheeling has to be stopped from outside the process cycle
 		export_status->set_running (false);
-		return;
+		return -1;
 	}
 
+	export_status->timespan++;
+
 	/* finish_timespan pops the config_map entry that has been done, so
 	   this is the timespan to do this time
 	*/
@@ -227,7 +228,7 @@ ExportHandler::start_timespan ()
 	session.ProcessExport.connect_same_thread (process_connection, boost::bind (&ExportHandler::process, this, _1));
 	process_position = current_timespan->get_start();
 	// TODO check if it's a RegionExport.. set flag to skip  process_without_events()
-	session.start_audio_export (process_position, realtime, region_export);
+	return session.start_audio_export (process_position, realtime, region_export);
 }
 
 void
@@ -237,15 +238,19 @@ ExportHandler::handle_duplicate_format_extensions()
 
 	ExtCountMap counts;
 	for (ConfigMap::iterator it = timespan_bounds.first; it != timespan_bounds.second; ++it) {
+		std::string pfx;
+		if (it->second.filename->include_timespan) {
+			pfx = it->first->name();
+		}
 		if (it->second.filename->include_channel_config && it->second.channel_config) {
 			/* stem-export has multiple files in the same timestamp, but a different channel_config for each.
 			 * However channel_config is only set in ExportGraphBuilder::Encoder::init_writer()
 			 * so we cannot yet use   it->second.filename->get_path(it->second.format).
 			 * We have to explicily check uniqueness of "channel-config + extension" here:
 			 */
-			counts[it->second.channel_config->name() + it->second.format->extension()]++;
+			counts[pfx + it->second.channel_config->name() + it->second.format->extension()]++;
 		} else {
-			counts[it->second.format->extension()]++;
+			counts[pfx + it->second.format->extension()]++;
 		}
 	}
 
@@ -256,6 +261,7 @@ ExportHandler::handle_duplicate_format_extensions()
 
 	// Set this always, as the filenames are shared...
 	for (ConfigMap::iterator it = timespan_bounds.first; it != timespan_bounds.second; ++it) {
+		assert (it->second.filename->include_format_name == duplicates_found);
 		it->second.filename->include_format_name = duplicates_found;
 	}
 }
@@ -350,6 +356,9 @@ ExportHandler::command_output(std::string output, size_t size)
 void*
 ExportHandler::start_timespan_bg (void* eh)
 {
+	char name[64];
+	snprintf (name, 64, "Export-TS-%p", (void*)DEBUG_THREAD_SELF);
+	pthread_set_name (name);
 	ExportHandler* self = static_cast<ExportHandler*> (eh);
 	self->process_connection.disconnect ();
 	Glib::Threads::Mutex::Lock l (self->export_status->lock());
@@ -364,6 +373,10 @@ ExportHandler::finish_timespan ()
 
 	while (config_map.begin() != timespan_bounds.second) {
 
+		// XXX single timespan+format may produce multiple files
+		// e.g export selection == session
+		// -> TagLib::FileRef is null
+
 		ExportFormatSpecPtr fmt = config_map.begin()->second.format;
 		std::string filename = config_map.begin()->second.filename->get_path(fmt);
 		if (fmt->with_cue()) {
