@@ -888,6 +888,46 @@ ExportProfileManager::get_warnings ()
 
 	/*** Check files ***/
 
+	/* handle_duplicate_format_extensions */
+	for (TimespanList::iterator t1 = timespans->begin(); t1 != timespans->end(); ++t1) {
+
+		typedef std::map<std::string, int> ExtCountMap;
+		ExtCountMap counts;
+
+		FormatStateList::const_iterator format_it;
+		FilenameStateList::const_iterator filename_it;
+
+		for (format_it = formats.begin(), filename_it = filenames.begin();
+		     format_it != formats.end() && filename_it != filenames.end();
+		     ++format_it, ++filename_it) {
+
+			ExportFilenamePtr filename = (*filename_it)->filename;
+			filename->include_channel_config = (_type == StemExport) || (channel_configs.size() > 1);
+
+			for(ChannelConfigStateList::iterator cc_it = channel_configs.begin(); cc_it != channel_configs.end(); ++cc_it) {
+				if (filename->include_channel_config && (*cc_it)->config) {
+					counts[(*cc_it)->config->name() + (*format_it)->format->extension()]++;
+				} else {
+					counts[(*format_it)->format->extension()]++;
+				}
+			}
+		}
+
+		bool duplicates_found = false;
+		for (ExtCountMap::iterator it = counts.begin(); it != counts.end(); ++it) {
+			if (it->second > 1) {
+				duplicates_found = true;
+			}
+		}
+
+		for (format_it = formats.begin(), filename_it = filenames.begin();
+		     format_it != formats.end() && filename_it != filenames.end();
+		     ++format_it, ++filename_it) {
+			ExportFilenamePtr filename = (*filename_it)->filename;
+			filename->include_format_name = duplicates_found;
+		}
+	}
+
 	bool folder_ok = true;
 
 	if (channel_config_state) {
