@@ -159,7 +159,7 @@ InternalSend::use_target (boost::shared_ptr<Route> sendto, bool update_name)
 
 	_send_to->add_send_to_internal_return (this);
 
-	mixbufs.ensure_buffers (_send_to->internal_return ()->input_streams (), _session.get_block_size ());
+	ensure_mixbufs ();
 	mixbufs.set_count (_send_to->internal_return ()->input_streams ());
 
 	_meter->configure_io (ChanCount (DataType::AUDIO, pan_outs ()), ChanCount (DataType::AUDIO, pan_outs ()));
@@ -186,7 +186,7 @@ void
 InternalSend::target_io_changed ()
 {
 	assert (_send_to);
-	mixbufs.ensure_buffers (_send_to->internal_return ()->input_streams (), _session.get_block_size ());
+	ensure_mixbufs ();
 	mixbufs.set_count (_send_to->internal_return ()->input_streams ());
 	reset_panner ();
 }
@@ -344,11 +344,20 @@ out:
 	_active = _pending_active;
 }
 
+void
+InternalSend::ensure_mixbufs ()
+{
+	for (DataType::iterator t = DataType::begin (); t != DataType::end (); ++t) {
+		size_t size = (*t == DataType::MIDI) ? _session.engine ().raw_buffer_size (*t) : _session.get_block_size ();
+		mixbufs.ensure_buffers (*t, _send_to->internal_return ()->input_streams ().get (*t), size);
+	}
+}
+
 int
-InternalSend::set_block_size (pframes_t nframes)
+InternalSend::set_block_size (pframes_t)
 {
 	if (_send_to) {
-		mixbufs.ensure_buffers (_send_to->internal_return ()->input_streams (), nframes);
+		ensure_mixbufs ();
 	}
 
 	return 0;
@@ -357,6 +366,9 @@ InternalSend::set_block_size (pframes_t nframes)
 void
 InternalSend::set_allow_feedback (bool yn)
 {
+	if (is_foldback ()) {
+		return;
+	}
 	_allow_feedback = yn;
 	_send_from->processors_changed (RouteProcessorChange ()); /* EMIT SIGNAL */
 }
@@ -392,6 +404,15 @@ InternalSend::set_state (const XMLNode& node, int version)
 {
 	init_gain ();
 
+	/* Allow Delivery::set_state() to restore pannable state when
+	 * copy/pasting Aux sends.
+	 *
+	 * At this point in time there is no target-bus. So when
+	 * Delivery::set_state() calls reset_panner(), the pannable
+	 * is dropped, before the panner state can be restored.
+	 */
+	defer_pan_reset ();
+
 	Send::set_state (node, version);
 
 	if (node.get_property ("target", _send_to_id)) {
@@ -406,8 +427,13 @@ InternalSend::set_state (const XMLNode& node, int version)
 			connect_when_legal ();
 		}
 	}
+	allow_pan_reset ();
 
-	node.get_property (X_("allow-feedback"), _allow_feedback);
+	if (!is_foldback ()) {
+		node.get_property (X_("allow-feedback"), _allow_feedback);
+	} else {
+		_allow_feedback = false;
+	}
 
 	return 0;
 }
