@@ -77,7 +77,6 @@ IO::IO (Session& s, const string& name, Direction dir, DataType default_type, bo
 	, _sendish (sendish)
 {
 	_active = true;
-	Port::PostDisconnect.connect_same_thread (*this, boost::bind (&IO::disconnect_check, this, _1, _2));
 	pending_state_node = 0;
 	setup_bundle ();
 }
@@ -90,7 +89,6 @@ IO::IO (Session& s, const XMLNode& node, DataType dt, bool sendish)
 {
 	_active = true;
 	pending_state_node = 0;
-	Port::PostDisconnect.connect_same_thread (*this, boost::bind (&IO::disconnect_check, this, _1, _2));
 
 	set_state (node, Stateful::loading_state_version);
 	setup_bundle ();
@@ -111,7 +109,7 @@ IO::~IO ()
 }
 
 void
-IO::disconnect_check (boost::shared_ptr<Port> a, boost::shared_ptr<Port> b)
+IO::connection_change (boost::shared_ptr<Port> a, boost::shared_ptr<Port> b)
 {
 	if (_session.deletion_in_progress ()) {
 		return;
@@ -505,6 +503,15 @@ IO::ensure_ports (ChanCount count, bool clear, void* src)
 	return 0;
 }
 
+void
+IO::reestablish_port_subscriptions ()
+{
+	_port_connections.drop_connections ();
+	for (PortSet::iterator i = _ports.begin(); i != _ports.end(); ++i) {
+		i->ConnectedOrDisconnected.connect_same_thread (*this, boost::bind (&IO::connection_change, this, _1, _2));
+	}
+}
+
 /** Caller must hold process lock */
 int
 IO::ensure_io (ChanCount count, bool clear, void* src)
@@ -615,7 +622,7 @@ IO::set_state (const XMLNode& node, int version)
 		 */
 		PortSet::iterator i = _ports.begin();
 		XMLNodeConstIterator x = node.children().begin();
-		for (; i != _ports.end(), x != node.children().end(); ++i, ++x) {
+		for (; i != _ports.end() && x != node.children().end(); ++i, ++x) {
 			if ((*x)->name() == "Port") {
 				(*x)->remove_property (X_("name"));
 				(*x)->set_property (X_("name"), i->name());
@@ -650,7 +657,6 @@ int
 IO::set_state_2X (const XMLNode& node, int version, bool in)
 {
 	XMLProperty const * prop;
-	XMLNodeConstIterator iter;
 	LocaleGuard lg;
 
 	/* force use of non-localized representation of decimal point,
@@ -1289,6 +1295,7 @@ IO::latency () const
 	return max_latency;
 }
 
+#if 0 // not used, but may some day be handy for debugging
 samplecnt_t
 IO::public_latency () const
 {
@@ -1312,6 +1319,7 @@ IO::public_latency () const
 	                                               ((_direction == Output) ? "PLAYBACK" : "CAPTURE")));
 	return max_latency;
 }
+#endif
 
 samplecnt_t
 IO::connected_latency (bool for_playback) const
@@ -1550,6 +1558,8 @@ IO::setup_bundle ()
 
 	}
 
+	reestablish_port_subscriptions ();
+
 	_bundle->resume_signals ();
 }
 
