@@ -201,10 +201,6 @@ MidiPort::read_and_parse_entire_midi_buffer_with_no_speed_adjustment (pframes_t
 
 		parser.set_timestamp (now + timestamp);
 
-		if (_trace_parser) {
-			_trace_parser->set_timestamp (now + timestamp);
-		}
-
 		/* During this parsing stage, signals will be emitted from the
 		 * Parser, which will update anything connected to it.
 		 *
@@ -218,20 +214,10 @@ MidiPort::read_and_parse_entire_midi_buffer_with_no_speed_adjustment (pframes_t
 			parser.scanner (buf[1]);
 			parser.scanner (0x40);  /* default (off) velocity */
 
-			if (_trace_parser) {
-				_trace_parser->scanner (0x80 | (buf[0] & 0x0F));
-				_trace_parser->scanner (buf[1]);
-				_trace_parser->scanner (0x40);
-			}
 		} else {
 			for (size_t n = 0; n < size; ++n) {
 				parser.scanner (buf[n]);
 			}
-			if (_trace_parser) {
-				for (size_t n = 0; n < size; ++n) {
-					_trace_parser->scanner (buf[n]);
-				}
-			}
 		}
 	}
 }
