@@ -84,6 +84,7 @@ MidiTrack::MidiTrack (Session& sess, string name, TrackMode mode)
 	, _note_mode (Sustained)
 	, _step_editing (false)
 	, _input_active (true)
+	, _restore_pgm_on_load (true)
 {
 	_session.SessionLoaded.connect_same_thread (*this, boost::bind (&MidiTrack::restore_controls, this));
 
@@ -173,6 +174,10 @@ MidiTrack::set_state (const XMLNode& node, int version)
 		set_input_active (yn);
 	}
 
+	if (node.get_property ("restore-pgm", yn)) {
+		set_restore_pgm_on_load (yn);
+	}
+
 	ChannelMode playback_channel_mode = AllChannels;
 	ChannelMode capture_channel_mode = AllChannels;
 
@@ -227,6 +232,7 @@ MidiTrack::state(bool save_template)
 
 		freeze_node = new XMLNode (X_("freeze-info"));
 		freeze_node->set_property ("playlist", _freeze_record.playlist->name());
+		freeze_node->set_property ("playlist-id", _freeze_record.playlist->id().to_s());
 		freeze_node->set_property ("state", _freeze_record.state);
 
 		for (vector<FreezeRecordProcessorInfo*>::iterator i = _freeze_record.processor_info.begin(); i != _freeze_record.processor_info.end(); ++i) {
@@ -250,6 +256,7 @@ MidiTrack::state(bool save_template)
 	root.set_property ("note-mode", _note_mode);
 	root.set_property ("step-editing", _step_editing);
 	root.set_property ("input-active", _input_active);
+	root.set_property ("restore-pgm", _restore_pgm_on_load);
 
 	for (Controls::const_iterator c = _controls.begin(); c != _controls.end(); ++c) {
 		if (boost::dynamic_pointer_cast<MidiTrack::MidiControl>(c->second)) {
@@ -266,6 +273,7 @@ void
 MidiTrack::set_state_part_two ()
 {
 	XMLNode* fnode;
+	XMLProperty const * prop;
 
 	/* This is called after all session state has been restored but before
 	   have been made ports and connections are established.
@@ -284,23 +292,28 @@ MidiTrack::set_state_part_two ()
 		}
 		_freeze_record.processor_info.clear ();
 
-		std::string str;
-		if (fnode->get_property (X_("playlist"), str)) {
-			boost::shared_ptr<Playlist> pl = _session.playlists()->by_name (str);
-			if (pl) {
-				_freeze_record.playlist = boost::dynamic_pointer_cast<MidiPlaylist> (pl);
-			} else {
-				_freeze_record.playlist.reset();
-				_freeze_record.state = NoFreeze;
-				return;
-			}
+		boost::shared_ptr<Playlist> freeze_pl;
+		if ((prop = fnode->property (X_("playlist-id"))) != 0) {
+			freeze_pl = _session.playlists()->by_id (prop->value());
+		} else if ((prop = fnode->property (X_("playlist"))) != 0) {
+			freeze_pl = _session.playlists()->by_name (prop->value());
 		}
+		if (freeze_pl) {
+			_freeze_record.playlist = boost::dynamic_pointer_cast<MidiPlaylist> (freeze_pl);
+			_freeze_record.playlist->use();
+		} else {
+			_freeze_record.playlist.reset ();
+			_freeze_record.state = NoFreeze;
+			return;
+		}
+
 
 		fnode->get_property (X_("state"), _freeze_record.state);
 
 		XMLNodeConstIterator citer;
 		XMLNodeList clist = fnode->children();
 
+		std::string str;
 		for (citer = clist.begin(); citer != clist.end(); ++citer) {
 			if ((*citer)->name() != X_("processor")) {
 				continue;
@@ -323,10 +336,22 @@ MidiTrack::set_state_part_two ()
 void
 MidiTrack::restore_controls ()
 {
-	// TODO order events (CC before PGM to set banks)
+	/* first CC (bank select) */
 	for (Controls::const_iterator c = _controls.begin(); c != _controls.end(); ++c) {
 		boost::shared_ptr<MidiTrack::MidiControl> mctrl = boost::dynamic_pointer_cast<MidiTrack::MidiControl>(c->second);
-		if (mctrl) {
+		if (mctrl && mctrl->parameter().type () != MidiPgmChangeAutomation) {
+			mctrl->restore_value();
+		}
+	}
+
+	if (!_restore_pgm_on_load) {
+		return;
+	}
+
+	/* then restore PGM */
+	for (Controls::const_iterator c = _controls.begin(); c != _controls.end(); ++c) {
+		boost::shared_ptr<MidiTrack::MidiControl> mctrl = boost::dynamic_pointer_cast<MidiTrack::MidiControl>(c->second);
+		if (mctrl && mctrl->parameter().type () == MidiPgmChangeAutomation) {
 			mctrl->restore_value();
 		}
 	}
@@ -589,24 +614,24 @@ MidiTrack::midi_panic()
 	DEBUG_TRACE (DEBUG::MidiIO, string_compose ("%1 delivers panic data\n", name()));
 	for (uint8_t channel = 0; channel <= 0xF; channel++) {
 		uint8_t ev[3] = { ((uint8_t) (MIDI_CMD_CONTROL | channel)), ((uint8_t) MIDI_CTL_SUSTAIN), 0 };
-		write_immediate_event(3, ev);
+		write_immediate_event (Evoral::MIDI_EVENT, 3, ev);
 		ev[1] = MIDI_CTL_ALL_NOTES_OFF;
-		write_immediate_event(3, ev);
+		write_immediate_event (Evoral::MIDI_EVENT, 3, ev);
 		ev[1] = MIDI_CTL_RESET_CONTROLLERS;
-		write_immediate_event(3, ev);
+		write_immediate_event (Evoral::MIDI_EVENT, 3, ev);
 	}
 }
 
 /** \return true on success, false on failure (no buffer space left)
  */
 bool
-MidiTrack::write_immediate_event(size_t size, const uint8_t* buf)
+MidiTrack::write_immediate_event(Evoral::EventType event_type, size_t size, const uint8_t* buf)
 {
 	if (!Evoral::midi_event_is_valid(buf, size)) {
 		cerr << "WARNING: Ignoring illegal immediate MIDI event" << endl;
 		return false;
 	}
-	return (_immediate_events.write (0, Evoral::MIDI_EVENT, size, buf) == size);
+	return (_immediate_events.write (0, event_type, size, buf) == size);
 }
 
 void
@@ -657,8 +682,14 @@ MidiTrack::MidiControl::actually_set_value (double val, PBD::Controllable::Group
 		return;
 	}
 
+	if (_session.loading ()) {
+		/* send events later in MidiTrack::restore_controls */
+		AutomationControl::actually_set_value (val, group_override);
+		return;
+	}
+
 	assert(val <= desc.upper);
-	if ( ! _list || ! automation_playback()) {
+	if (!_list || !automation_playback ()) {
 		size_t size = 3;
 		uint8_t ev[3] = { parameter.channel(), uint8_t (val), 0 };
 		switch(parameter.type()) {
@@ -696,7 +727,7 @@ MidiTrack::MidiControl::actually_set_value (double val, PBD::Controllable::Group
 			size = 0;
 			assert(false);
 		}
-		_route->write_immediate_event(size,  ev);
+		_route->write_immediate_event(Evoral::LIVE_MIDI_EVENT, size, ev);
 	}
 
 	AutomationControl::actually_set_value(val, group_override);
@@ -759,6 +790,22 @@ MidiTrack::midi_playlist ()
 	return boost::dynamic_pointer_cast<MidiPlaylist> (_playlists[DataType::MIDI]);
 }
 
+void
+MidiTrack::set_restore_pgm_on_load (bool yn)
+{
+	if (_restore_pgm_on_load == yn) {
+		return;
+	}
+	_restore_pgm_on_load = yn;
+	_session.set_dirty();
+}
+
+bool
+MidiTrack::restore_pgm_on_load () const
+{
+	return _restore_pgm_on_load;
+}
+
 bool
 MidiTrack::input_active () const
 {
@@ -834,7 +881,7 @@ MidiTrack::act_on_mute ()
 
 				DEBUG_TRACE (DEBUG::MidiIO, string_compose ("%1 delivers mute message to channel %2\n", name(), channel+1));
 				uint8_t ev[3] = { ((uint8_t) (MIDI_CMD_CONTROL | channel)), MIDI_CTL_SUSTAIN, 0 };
-				write_immediate_event (3, ev);
+				write_immediate_event (Evoral::MIDI_EVENT, 3, ev);
 
 				/* Note we do not send MIDI_CTL_ALL_NOTES_OFF here, since this may
 				   silence notes that came from another non-muted track. */
