@@ -39,7 +39,6 @@ using namespace std;
 using namespace ARDOUR;
 using namespace PBD;
 
-PBD::Signal2<void,boost::shared_ptr<Port>, boost::shared_ptr<Port> > Port::PostDisconnect;
 PBD::Signal0<void> Port::PortDrop;
 PBD::Signal0<void> Port::PortSignalDrop;
 
@@ -157,20 +156,16 @@ Port::drop ()
 void
 Port::port_connected_or_disconnected (boost::weak_ptr<Port> w0, boost::weak_ptr<Port> w1, bool con)
 {
-	if (con) {
-		/* we're only interested in disconnect */
-		return;
-	}
 	boost::shared_ptr<Port> p0 = w0.lock ();
 	boost::shared_ptr<Port> p1 = w1.lock ();
 	/* a cheaper, less hacky way to do boost::shared_from_this() ...  */
 	boost::shared_ptr<Port> pself = AudioEngine::instance()->get_port_by_name (name());
 
 	if (p0 == pself) {
-		PostDisconnect (p0, p1); // emit signal
+		ConnectedOrDisconnected (p0, p1, con); // emit signal
 	}
 	if (p1 == pself) {
-		PostDisconnect (p1, p0); // emit signal
+		ConnectedOrDisconnected (p1, p0, con); // emit signal
 	}
 }
 
@@ -201,7 +196,7 @@ Port::disconnect_all ()
 		for (vector<string>::const_iterator c = connections.begin(); c != connections.end() && pself; ++c) {
 			boost::shared_ptr<Port> pother = AudioEngine::instance()->get_port_by_name (*c);
 			if (pother) {
-				PostDisconnect (pself, pother); // emit signal
+				ConnectedOrDisconnected (pself, pother, false); // emit signal
 			}
 		}
 	}
@@ -295,7 +290,7 @@ Port::disconnect (std::string const & other)
 		   a check on whether this may affect anything that we
 		   need to know about.
 		*/
-		PostDisconnect (pself, pother); // emit signal
+		ConnectedOrDisconnected (pself, pother, false); // emit signal
 	}
 
 	return r;
@@ -433,9 +428,9 @@ Port::private_latency_range (bool playback) const
 LatencyRange
 Port::public_latency_range (bool /*playback*/) const
 {
+	/*Note: this method is no longer used. It exists purely for debugging reasons */
 	LatencyRange r;
 
-
 	if (_port_handle) {
 		r = port_engine.get_latency_range (_port_handle, sends_output() ? true : false);
 		if (externally_connected () && 0 == (_flags & TransportSyncPort)) {
@@ -443,10 +438,13 @@ Port::public_latency_range (bool /*playback*/) const
 			r.min /= _speed_ratio;
 			r.max /= _speed_ratio;
 #endif
+#if 0
+			/* use value as set by set_public_latency_range */
 			if (type () == DataType::AUDIO) {
 				r.min += (_resampler_quality - 1);
 				r.max += (_resampler_quality - 1);
 			}
+#endif
 		}
 
 		DEBUG_TRACE (DEBUG::LatencyIO, string_compose (
@@ -562,15 +560,23 @@ Port::reconnect ()
 {
 	/* caller must hold process lock; intended to be used only after reestablish() */
 
-	DEBUG_TRACE (DEBUG::Ports, string_compose ("Connect %1 to %2 destinations\n",name(), _connections.size()));
+	DEBUG_TRACE (DEBUG::Ports, string_compose ("Port::reconnect() Connect %1 to %2 destinations\n",name(), _connections.size()));
 
-	for (std::set<string>::iterator i = _connections.begin(); i != _connections.end(); ++i) {
-		if (connect (*i)) {
-			return -1;
+	int count = 0;
+	std::set<string>::iterator i = _connections.begin();
+
+	while (i != _connections.end()) {
+		std::set<string>::iterator current = i++;
+		if (connect (*current)) {
+			DEBUG_TRACE (DEBUG::Ports, string_compose ("Port::reconnect() failed to connect %1 to %2\n", name(), (*current)));
+			_connections.erase (current);
+		}
+		else {
+			++count;
 		}
 	}
 
-	return 0;
+	return count == 0 ? -1 : 0;
 }
 
 /** @param n Short port name (no port-system client name) */
