@@ -86,7 +86,7 @@ namespace ARDOUR {
 	}
 }
 
-PBD::Signal2<void,boost::shared_ptr<ARDOUR::Region>,const PropertyChange&> Region::RegionPropertyChanged;
+PBD::Signal2<void,boost::shared_ptr<ARDOUR::RegionList>,const PropertyChange&> Region::RegionsPropertyChanged;
 
 void
 Region::make_property_quarks ()
@@ -262,6 +262,7 @@ Region::Region (Session& s, samplepos_t start, samplecnt_t length, const string&
 	, _last_position (0)
 	, _first_edit (EditChangesNothing)
 	, _layer (0)
+	, _changemap (0)
 {
 	register_properties ();
 
@@ -277,6 +278,7 @@ Region::Region (const SourceList& srcs)
 	, _last_position (0)
 	, _first_edit (EditChangesNothing)
 	, _layer (0)
+	, _changemap (0)
 {
 	register_properties ();
 
@@ -297,6 +299,7 @@ Region::Region (boost::shared_ptr<const Region> other)
 	, _last_position(other->_last_position) \
 	, _first_edit (EditChangesNothing)
 	, _layer (other->_layer)
+	, _changemap (other->_changemap)
 {
 	register_properties ();
 
@@ -311,7 +314,7 @@ Region::Region (boost::shared_ptr<const Region> other)
 	use_sources (other->_sources);
 	set_master_sources (other->_master_sources);
 
-	_position_lock_style = other->_position_lock_style;
+	_position_lock_style = other->_position_lock_style.val();
 	_first_edit = other->_first_edit;
 
 	_start = other->_start;
@@ -359,6 +362,7 @@ Region::Region (boost::shared_ptr<const Region> other, MusicSample offset)
 	, _last_position(other->_last_position) \
 	, _first_edit (EditChangesNothing)
 	, _layer (other->_layer)
+	, _changemap (other->_changemap)
 {
 	register_properties ();
 
@@ -415,6 +419,7 @@ Region::Region (boost::shared_ptr<const Region> other, const SourceList& srcs)
 	, _last_position (other->_last_position)
 	, _first_edit (EditChangesID)
 	, _layer (other->_layer)
+	, _changemap (other->_changemap)
 {
 	register_properties ();
 
@@ -1490,12 +1495,18 @@ Region::send_change (const PropertyChange& what_changed)
 	if (!Stateful::property_changes_suspended()) {
 
 		/* Try and send a shared_pointer unless this is part of the constructor.
-		   If so, do nothing.
-		*/
+		 * If so, do nothing.
+		 */
 
 		try {
 			boost::shared_ptr<Region> rptr = shared_from_this();
-			RegionPropertyChanged (rptr, what_changed);
+			if (_changemap) { 
+				(*_changemap)[what_changed].push_back (rptr);
+			} else {
+				boost::shared_ptr<RegionList> rl (new RegionList);
+				rl->push_back (rptr);
+				RegionsPropertyChanged (rl, what_changed);
+			}
 		} catch (...) {
 			/* no shared_ptr available, relax; */
 		}
@@ -1538,12 +1549,6 @@ Region::size_equivalent (boost::shared_ptr<const Region> other) const
 		_length == other->_length;
 }
 
-bool
-Region::region_list_equivalent (boost::shared_ptr<const Region> other) const
-{
-	return size_equivalent (other) && source_equivalent (other) && _name == other->_name;
-}
-
 void
 Region::source_deleted (boost::weak_ptr<Source>)
 {
@@ -1587,7 +1592,6 @@ Region::set_master_sources (const SourceList& srcs)
 
 	for (SourceList::const_iterator i = _master_sources.begin (); i != _master_sources.end(); ++i) {
 		(*i)->inc_use_count ();
-//		Source::SourcePropertyChanged( *i );
 	}
 }
 
@@ -1891,6 +1895,59 @@ Region::merge_features (AnalysisFeatureList& result, const AnalysisFeatureList&
 	}
 }
 
+void
+Region::captured_xruns (XrunPositions& xruns, bool abs) const
+{
+	bool was_empty = xruns.empty ();
+	for (SourceList::const_iterator i = _sources.begin (); i != _sources.end(); ++i) {
+		XrunPositions const& x = (*i)->captured_xruns ();
+		for (XrunPositions::const_iterator p = x.begin (); p != x.end (); ++p) {
+			if (abs) {
+				xruns.push_back (*p);
+			} else if (*p >= _start && *p < _start + _length) {
+				xruns.push_back (*p - _start);
+			}
+		}
+	}
+	if (_sources.size () > 1 || !was_empty) {
+		sort (xruns.begin (), xruns.end ());
+		xruns.erase (unique (xruns.begin (), xruns.end ()), xruns.end ());
+	}
+}
+
+void
+Region::get_cue_markers (CueMarkers& cues, bool abs) const
+{
+	for (SourceList::const_iterator s = _sources.begin (); s != _sources.end(); ++s) {
+		CueMarkers const& x = (*s)->cue_markers ();
+		for (CueMarkers::const_iterator p = x.begin (); p != x.end (); ++p) {
+			if (p->position() >= _start && p->position() < _start + _length) {
+				if (abs) {
+					cues.insert (*p);
+				} else {
+					cues.insert (CueMarker (p->text(), p->position() - _start));
+				}
+			}
+		}
+	}
+}
+
+void
+Region::move_cue_marker (CueMarker const & cm, samplepos_t region_relative_position)
+{
+	for (SourceList::const_iterator s = _sources.begin (); s != _sources.end(); ++s) {
+		(*s)->move_cue_marker (cm, start() + region_relative_position);
+	}
+}
+
+void
+Region::rename_cue_marker (CueMarker& cm, std::string const & str)
+{
+	for (SourceList::const_iterator s = _sources.begin (); s != _sources.end(); ++s) {
+		(*s)->rename_cue_marker (cm, str);
+	}
+}
+
 void
 Region::drop_sources ()
 {
