@@ -46,11 +46,6 @@ TestUI::TestUI ()
 	m_test_receiver.listen_to (PBD::warning);
 	m_test_receiver.listen_to (PBD::error);
 	m_test_receiver.listen_to (PBD::fatal);
-
-	/* We can't use VSTs here as we have a stub instead of the
-	   required bits in gtk2_ardour.
-	*/
-	Config->set_use_lxvst (false);
 }
 
 TestUI::~TestUI ()
