@@ -37,6 +37,29 @@ using namespace PBD;
 
 Pool* TransportFSM::Event::pool = 0;
 
+/*
+
+autoplay off (hint: state remains the same):
+
+[STOPPED]  -> LOCATE -> [STOPPED]
+[ROLLING]  -> LOCATE -> [ROLLING]
+[RWD|FFWD] -> LOCATE -> [RWD|FFWD]
+
+autoplay on (hint: state is always rolling):
+
+[STOPPED]  -> LOCATE -> [ROLLING]
+[ROLLING]  -> LOCATE -> [ROLLING]
+[RWD|FFWD] -> LOCATE -> [ROLLING]
+
+the problem child is the last one. The final ROLLING state is intended to be
+forwards at the default speed. But if we were rewinding, we need a reverse.
+
+if autoplay is the determining factor in differentiating these two, we must
+make it available via TransportAPI. but let's rename it as something slightly
+more on-topic.
+
+*/
+
 void
 TransportFSM::Event::init_pool ()
 {
@@ -56,10 +79,16 @@ TransportFSM::Event::operator delete (void *ptr, size_t /*size*/)
 }
 
 TransportFSM::TransportFSM (TransportAPI& tapi)
-	: _last_locate (Locate, 0, MustRoll, false, false, false) /* all but first argument don't matter */
-	, last_speed_request (SetSpeed, 0, false, false, false) /* ditto */
+	: _motion_state (Stopped)
+	, _butler_state (NotWaitingForButler)
+	, _direction_state (Forwards)
+	, _transport_speed (0)
+	, _last_locate (Locate, max_samplepos, MustRoll, false, false) /* all but first two argument don't matter */
 	, api (&tapi)
 	, processing (0)
+	, most_recently_requested_speed (std::numeric_limits<double>::max())
+	, _default_speed (1.0)
+	, _reverse_after_declick (0)
 {
 	init ();
 }
@@ -67,10 +96,6 @@ TransportFSM::TransportFSM (TransportAPI& tapi)
 void
 TransportFSM::init ()
 {
-	_motion_state = Stopped;
-	_butler_state = NotWaitingForButler;
-	_direction_state = Forwards;
-	_last_locate.target = max_samplepos;
 }
 
 void
@@ -82,6 +107,7 @@ TransportFSM::process_events ()
 
 		MotionState oms = _motion_state;
 		ButlerState obs = _butler_state;
+		DirectionState ods = _direction_state;
 
 		Event* ev = &queued_events.front();
 		bool deferred;
@@ -97,23 +123,33 @@ TransportFSM::process_events ()
 
 		if (process_event (*ev, false, deferred)) { /* event processed successfully */
 
-			if (oms != _motion_state || obs != _butler_state) {
+			if (oms != _motion_state || obs != _butler_state || ods != _direction_state) {
 
 				/* state changed, so now check deferred events
 				 * to see if they can be processed now
 				 */
 
 				if (!deferred_events.empty() ){
+
 					DEBUG_TRACE (DEBUG::TFSMEvents, string_compose ("processing %1 deferred events\n", deferred_events.size()));
 
 					for (EventList::iterator e = deferred_events.begin(); e != deferred_events.end(); ) {
 						Event* deferred_ev = &(*e);
 						bool deferred2;
-						if (process_event (*e, true, deferred2)) { /* event processed, remove from deferred */
-							e = deferred_events.erase (e);
-							delete deferred_ev;
-						} else {
+
+						if (process_event (*e, true, deferred2)) {
+							if (!deferred2) { /* event processed and not deferred again, remove from deferred */
+								DEBUG_TRACE (DEBUG::TFSMEvents, string_compose ("processed deferred event %1, now deleting\n", enum_2_string (deferred_ev->type)));
+								e = deferred_events.erase (e);
+								delete deferred_ev;
+							} else {
+								DEBUG_TRACE (DEBUG::TFSMEvents, string_compose ("Re-Defer deferred event %1\n", enum_2_string (deferred_ev->type)));
+								++e;
+							}
+						} else { /* process error */
+							DEBUG_TRACE (DEBUG::TFSMEvents, string_compose ("deferred event %1 failed, deleting\n", enum_2_string (deferred_ev->type)));
 							++e;
+							delete deferred_ev;
 						}
 					}
 				}
@@ -192,7 +228,6 @@ TransportFSM::bad_transition (Event const & ev)
 {
 	error << "bad transition, current state = " << current_state() << " event = " << enum_2_string (ev.type) << endmsg;
 	std::cerr << "bad transition, current state = " << current_state() << " event = " << enum_2_string (ev.type) << std::endl;
-	PBD::stacktrace (std::cerr, 30);
 }
 
 bool
@@ -209,8 +244,8 @@ TransportFSM::process_event (Event& ev, bool already_deferred, bool& deferred)
 		case Reversing:
 			if (!already_deferred) {
 				defer (ev);
-				deferred = true;
 			}
+			deferred = true;
 			break;
 		default:
 			switch (_motion_state) {
@@ -221,8 +256,8 @@ TransportFSM::process_event (Event& ev, bool already_deferred, bool& deferred)
 			default:
 				if (!already_deferred) {
 					defer (ev);
-					deferred = true;
 				}
+				deferred = true;
 			}
 			break;
 		}
@@ -240,14 +275,14 @@ TransportFSM::process_event (Event& ev, bool already_deferred, bool& deferred)
 		case WaitingForLocate:
 			if (!already_deferred) {
 				defer (ev);
-				deferred = true;
 			}
+			deferred = true;
 			break;
 		case DeclickToStop:
 			if (!already_deferred) {
 				defer (ev);
-				deferred = true;
 			}
+			deferred = true;
 			break;
 		default:
 			bad_transition (ev); return false;
@@ -267,8 +302,8 @@ TransportFSM::process_event (Event& ev, bool already_deferred, bool& deferred)
 		case WaitingForLocate:
 			if (!already_deferred) {
 				defer (ev);
-				deferred = true;
 			}
+			deferred = true;
 			break;
 		case DeclickToStop:
 			/* already doing it */
@@ -280,9 +315,8 @@ TransportFSM::process_event (Event& ev, bool already_deferred, bool& deferred)
 		break;
 
 	case Locate:
-		DEBUG_TRACE (DEBUG::TFSMEvents, string_compose ("locate, ltd = %1 flush = %2 target = %3 loop %4 force %5\n",
+		DEBUG_TRACE (DEBUG::TFSMEvents, string_compose ("locate, ltd = %1 target = %2 for loop end %3 force %4\n",
 		                                                enum_2_string (ev.ltd),
-		                                                ev.with_flush,
 		                                                ev.target,
 		                                                ev.for_loop_end,
 		                                                ev.force));
@@ -360,17 +394,11 @@ TransportFSM::process_event (Event& ev, bool already_deferred, bool& deferred)
 					transition (Backwards);
 				}
 
-				if (fabs (most_recently_requested_speed) > 0.) {
-
-					transition (Rolling);
-
-					api->set_transport_speed (last_speed_request.speed, last_speed_request.abort_capture, last_speed_request.clear_state, last_speed_request.as_default);
-
-					if (most_recently_requested_speed != 0.0) {
-						roll_after_locate ();
-					}
-				} else {
+				if (should_not_roll_after_locate()) {
 					transition (Stopped);
+				} else {
+					transition (Rolling);
+					roll_after_locate ();
 				}
 
 			} else {
@@ -392,12 +420,17 @@ TransportFSM::process_event (Event& ev, bool already_deferred, bool& deferred)
 	case DeclickDone:
 		switch (_motion_state) {
 		case DeclickToLocate:
+			if (_reverse_after_declick) {
+				transition (Reversing);
+			}
 			transition (WaitingForLocate);
 			start_locate_after_declick ();
 			break;
 		case DeclickToStop:
-			transition (Stopped);
-			/* transport already stopped */
+			if (!maybe_reset_speed ()) {
+				transition (Stopped);
+				/* transport already stopped */
+			}
 			break;
 		default:
 			bad_transition (ev); return false;
@@ -442,7 +475,16 @@ TransportFSM::start_playback ()
 	_last_locate.target = max_samplepos;
 	current_roll_after_locate_status = boost::none;
 
-	api->start_transport();
+	if (most_recently_requested_speed == std::numeric_limits<double>::max()) {
+		/* we started rolling without ever setting speed; that's an implicit
+		 * call to set_speed (1.0)
+		 */
+		most_recently_requested_speed = 1.0;
+	} else {
+		api->set_transport_speed (most_recently_requested_speed);
+	}
+
+	api->start_transport (false);
 }
 
 void
@@ -456,18 +498,42 @@ TransportFSM::stop_playback (Event const & s)
 	api->stop_transport (s.abort_capture, s.clear_state);
 }
 
-void
-TransportFSM::set_roll_after (bool with_roll) const
+bool
+TransportFSM::maybe_reset_speed ()
 {
-	current_roll_after_locate_status = with_roll;
+	bool state_changed = false;
+
+	if (Config->get_reset_default_speed_on_stop()) {
+
+		if (most_recently_requested_speed != 1.0 || default_speed() != 1.0) {
+			set_default_speed(1.0);
+			set_speed (Event (1.0));
+			state_changed = true;
+		}
+
+	} else {
+
+		/* We're not resetting back to 1.0, but we may need to handle a
+		 * speed change from whatever we have been rolling at to
+		 * whatever the current default is. We could have been
+		 * rewinding at -4.5 ... when we restart, we need to play at
+		 * the current _default_speed
+		 */
+
+		if (most_recently_requested_speed != _default_speed) {
+			state_changed = set_speed (Event (_default_speed));
+		}
+	}
+
+	return state_changed;
 }
 
 void
 TransportFSM::start_declick_for_locate (Event const & l)
 {
 	assert (l.type == Locate);
-	DEBUG_TRACE (DEBUG::TFSMEvents, string_compose ("start_declick_for_locate, crals %1 ltd %2 speed %3 sral %4\n", (bool) current_roll_after_locate_status,
-	                                                enum_2_string (l.ltd), api->speed(), api->should_roll_after_locate()));
+	DEBUG_TRACE (DEBUG::TFSMEvents, string_compose ("start_declick_for_locate, crals %1 ltd %2 sral %3\n", (bool) current_roll_after_locate_status,
+	                                                enum_2_string (l.ltd), api->should_roll_after_locate()));
 	_last_locate = l;
 
 	if (!current_roll_after_locate_status) {
@@ -476,17 +542,6 @@ TransportFSM::start_declick_for_locate (Event const & l)
 	api->stop_transport (false, false);
 }
 
-void
-TransportFSM::start_locate_while_stopped (Event const & l) const
-{
-	assert (l.type == Locate);
-	DEBUG_TRACE (DEBUG::TFSMEvents, "start_locate_while_stopped\n");
-
-	set_roll_after (compute_should_roll (l.ltd));
-
-	api->locate (l.target, current_roll_after_locate_status.get(), l.with_flush, l.for_loop_end, l.force);
-}
-
 bool
 TransportFSM::compute_should_roll (LocateTransportDisposition ltd) const
 {
@@ -495,19 +550,37 @@ TransportFSM::compute_should_roll (LocateTransportDisposition ltd) const
 		return true;
 	case MustStop:
 		return false;
-	case RollIfAppropriate:
-		/* by the time we call this, if we were rolling before the
-		   locate, we've already transitioned into DeclickToLocate
-		*/
-		if (_motion_state == DeclickToLocate) {
-			return true;
-		} else {
-			return api->should_roll_after_locate ();
-		}
+	default:
 		break;
 	}
-	/*NOTREACHED*/
-	return true;
+
+	/* case RollIfAppropriate */
+
+	/* by the time we call this, if we were rolling before the
+	   locate, we've already transitioned into DeclickToLocate,
+	   so DeclickToLocate is essentially a synonym for "was Rolling".
+	*/
+	if (_motion_state == DeclickToLocate) {
+		return true;
+	}
+
+	return api->should_roll_after_locate ();
+}
+
+void
+TransportFSM::set_roll_after (bool with_roll) const
+{
+	current_roll_after_locate_status = with_roll;
+}
+
+void
+TransportFSM::start_locate_while_stopped (Event const & l) const
+{
+	assert (l.type == Locate);
+	DEBUG_TRACE (DEBUG::TFSMEvents, "start_locate_while_stopped\n");
+
+	set_roll_after (compute_should_roll (l.ltd));
+	api->locate (l.target, l.for_loop_end, l.force);
 }
 
 void
@@ -516,20 +589,49 @@ TransportFSM::locate_for_loop (Event const & l)
 	assert (l.type == Locate);
 	DEBUG_TRACE (DEBUG::TFSMEvents, string_compose ("locate_for_loop, wl = %1\n", l.for_loop_end));
 
-	const bool should_roll = compute_should_roll (l.ltd);
-	current_roll_after_locate_status = should_roll;
 	_last_locate = l;
-	api->locate (l.target, should_roll, l.with_flush, l.for_loop_end, l.force);
+	set_roll_after (compute_should_roll (l.ltd));
+	api->locate (l.target, l.for_loop_end, l.force);
 }
 
 void
-TransportFSM::start_locate_after_declick () const
+TransportFSM::start_locate_after_declick ()
 {
 	DEBUG_TRACE (DEBUG::TFSMEvents, string_compose ("start_locate_after_declick, have crals ? %1 roll will be %2\n", (bool) current_roll_after_locate_status,
 	                                                current_roll_after_locate_status ? current_roll_after_locate_status.get() : compute_should_roll (_last_locate.ltd)));
 
-	const bool roll = current_roll_after_locate_status ? current_roll_after_locate_status.get() : compute_should_roll (_last_locate.ltd);
-	api->locate (_last_locate.target, roll, _last_locate.with_flush, _last_locate.for_loop_end, _last_locate.force);
+	/* we only get here because a locate request arrived while we were rolling. We declicked and that is now finished */
+
+	/* Special case: we were rolling. If the user has set auto-play, then
+	   post-locate, we should roll at the default speed, which may involve
+	   a reversal and that needs to be setup before we actually locate.
+	*/
+
+	double post_locate_speed;
+
+	if (api->user_roll_after_locate() && !_reverse_after_declick) {
+		post_locate_speed = _default_speed;
+	} else {
+		post_locate_speed = most_recently_requested_speed;
+	}
+
+	if (post_locate_speed * most_recently_requested_speed < 0) {
+		/* different directions */
+		transition (Reversing);
+	}
+
+	DEBUG_TRACE (DEBUG::TFSMEvents, string_compose ("post-locate speed will be %1 based on user-roll-after %2 and r-a-dc %3\n", post_locate_speed, api->user_roll_after_locate(), _reverse_after_declick));
+
+	if (_reverse_after_declick) {
+		_reverse_after_declick--;
+	}
+
+	if (api->user_roll_after_locate()) {
+		most_recently_requested_speed = post_locate_speed;
+	}
+
+
+	api->locate (_last_locate.target, _last_locate.for_loop_end, _last_locate.force);
 }
 
 void
@@ -558,7 +660,7 @@ TransportFSM::interrupt_locate (Event const & l) const
 	 * we are interrupting the locate to start a new one.
 	 */
 	_last_locate = l;
-	api->locate (l.target, false, l.with_flush, l.for_loop_end, l.force);
+	api->locate (l.target, l.for_loop_end, l.force);
 }
 
 void
@@ -586,9 +688,19 @@ TransportFSM::should_roll_after_locate () const
 void
 TransportFSM::roll_after_locate () const
 {
-	DEBUG_TRACE (DEBUG::TFSMEvents, string_compose ("rolling after locate, was for_loop ? %1\n", _last_locate.for_loop_end));
+	bool for_loop = _last_locate.for_loop_end;
+	DEBUG_TRACE (DEBUG::TFSMEvents, string_compose ("rolling after locate, was for_loop ? %1\n", for_loop));
 	current_roll_after_locate_status = boost::none;
-	api->start_transport ();
+
+	if (most_recently_requested_speed == std::numeric_limits<double>::max()) {
+		/* we started rolling without ever setting speed; that's an implicit
+		 * call to set_speed (1.0)
+		 */
+		most_recently_requested_speed = 1.0;
+	}
+
+	api->set_transport_speed (most_recently_requested_speed);
+	api->start_transport (for_loop);
 }
 
 void
@@ -603,6 +715,7 @@ TransportFSM::transition (MotionState ms)
 {
 	const MotionState old = _motion_state;
 	_motion_state = ms;
+	_transport_speed = compute_transport_speed ();
 	DEBUG_TRACE (DEBUG::TFSMState, string_compose ("Leave %1, enter %2\n", enum_2_string (old), current_state()));
 }
 
@@ -611,6 +724,7 @@ TransportFSM::transition (ButlerState bs)
 {
 	const ButlerState old = _butler_state;
 	_butler_state = bs;
+	_transport_speed = compute_transport_speed ();
 	DEBUG_TRACE (DEBUG::TFSMState, string_compose ("Leave %1, enter %2\n", enum_2_string (old), current_state()));
 }
 
@@ -619,6 +733,7 @@ TransportFSM::transition (DirectionState ds)
 {
 	const DirectionState old = _direction_state;
 	_direction_state = ds;
+	_transport_speed = compute_transport_speed ();
 	DEBUG_TRACE (DEBUG::TFSMState, string_compose ("Leave %1, enter %2\n", enum_2_string (old), current_state()));
 }
 
@@ -632,55 +747,92 @@ TransportFSM::enqueue (Event* ev)
 	}
 }
 
-void
-TransportFSM::set_speed (Event const & ev)
+int
+TransportFSM::compute_transport_speed () const
 {
-	if ((rolling() && ev.speed * most_recently_requested_speed < 0.0) ||
-	    (stopped() && ev.speed < 0.0) ||
-	    (rolling() && most_recently_requested_speed < 0.0 && ev.speed == 0.0)) {
-
-		/* Transport was rolling, and new speed has opposite sign to
-		 * the last requested speed.
-		 *
-		 * OR
-		 *
-		 * Transport was stopped, and new speed is negative.
-		 *
-		 * OR
-		 *
-		 * new speed is zero, last requested speed was negative
-		 *
-		 * SO ... we need to reverse.
-		 *
-		 * The plan: stop normally (with a declick, and schedule a
-		 * locate to our current position, with "force" set to true,
-		 * and roll right after it is complete.
-		 */
-
-		most_recently_requested_speed = ev.speed;
-		last_speed_request = ev;
-		transition (Reversing);
-
-		DEBUG_TRACE (DEBUG::TFSMState, string_compose ("reverse, target speed %1 MRRS %2 state %3\n", ev.speed, most_recently_requested_speed, current_state()));
-
-		Event lev (Locate, api->position(), MustRoll, true, false, true);
-
-		transition (DeclickToLocate);
-		start_declick_for_locate (lev);
-
-	} else {
-
-		most_recently_requested_speed = ev.speed;
-		api->set_transport_speed (ev.speed, ev.abort_capture, ev.clear_state, ev.as_default);
-
+	if (_motion_state != Rolling || _direction_state == Reversing) {
+		return 0;
 	}
 
+	if (_direction_state == Backwards) {
+		return -1;
+	}
+
+	return 1;
+}
+
+bool
+TransportFSM::set_speed (Event const & ev)
+{
+	assert (ev.speed != 0.0);
+
+	bool initial_reverse = false;
+	bool must_reverse = false;
+
+	DEBUG_TRACE (DEBUG::TFSMState, string_compose ("::set_speed(): target speed %1 MRRS %2 state %3\n", ev.speed, most_recently_requested_speed, current_state()));
+
+	/* This "must_roll" value is a bit complicated to explain. If we were moving at anything other than normal speed+direction,
+	   and then stop, and reset-default-speed-on-stop is enabled, Session::stop_transport() will queue a SetSpeed
+	   request to restore the default speed to 1.0. However, because of the stop in progress, the SetSpeed event
+	   will be deferred during the stop. Eventually, we will handle it, and call this method. If we need to reverse
+	   direction, the locate will end up using compute_should_roll(), with rolling state already set to DeclickToLocate.
+
+	   If we pass in RollIfAppropriate as the after-locate roll disposition, the logic there will conclude that we should roll.
+	   This logic is correct in cases where we do a normal locate, but not for a locate-for-reverse when stopping.
+
+	   So, instead of using RollIfAppropriate, determine here is our after-locate state should be rolling or stopped, and pass
+	   MustRoll or MustStop to the locate request. This will get compute_should_roll() to do the right thing, and once the locate
+	   is complete, we will be in the correct state.
+	*/
+
+
+	const bool must_roll = rolling();
+
+	if (most_recently_requested_speed == std::numeric_limits<double>::max()) {
+		/* have never rolled yet */
+		initial_reverse = true;
+	}
+
+	if (ev.speed * most_recently_requested_speed < 0.0 || initial_reverse) {
+		must_reverse = true;
+	}
+
+	api->set_transport_speed (ev.speed);
+
+	/* corner case: first call to ::set_speed() has a negative
+	 * speed
+	 */
+
+	most_recently_requested_speed = ev.speed;
+
+	if (must_reverse) {
+
+		/* direction change */
+
+		DEBUG_TRACE (DEBUG::TFSMState, string_compose ("switch-directions, target speed %1 state %2 IR %3\n", ev.speed, current_state(), initial_reverse));
+
+		Event lev (Locate, api->position(), must_roll ? MustRoll : MustStop, false, true);
+
+		if (_transport_speed) {
+			_reverse_after_declick++;
+			transition (DeclickToLocate);
+			start_declick_for_locate (lev);
+		} else {
+			transition (Reversing);
+			transition (WaitingForLocate);
+			start_locate_while_stopped (lev);
+		}
+
+		return true;
+	}
+
+	return false;
 }
 
 bool
 TransportFSM::will_roll_fowards () const
 {
-	if (reversing()) {
+	if (reversing() || _reverse_after_declick) {
 		return most_recently_requested_speed >= 0; /* note: future speed of zero is equivalent to Forwards */
 	}
 	return (_direction_state == Forwards);
