@@ -35,6 +35,7 @@
 
 #include "ardour/vst_types.h"
 #include "ardour/vst_plugin.h"
+#include "ardour/vst2_scan.h"
 #include "ardour/vestige/vestige.h"
 #include "ardour/session.h"
 #include "ardour/filesystem_paths.h"
@@ -187,7 +188,7 @@ VSTPlugin::get_parameter (uint32_t which) const
 }
 
 void
-VSTPlugin::set_parameter (uint32_t which, float newval)
+VSTPlugin::set_parameter (uint32_t which, float newval, sampleoffset_t when)
 {
 	if (which == UINT32_MAX - 1) {
 		// ardour uses enable-semantics: 1: enabled, 0: bypassed
@@ -217,15 +218,15 @@ VSTPlugin::set_parameter (uint32_t which, float newval)
 
 	if (!PBD::floateq (curval, oldval, 1)) {
 		/* value has changed, follow rest of the notification path */
-		Plugin::set_parameter (which, newval);
+		Plugin::set_parameter (which, newval, when);
 	}
 }
 
 void
-VSTPlugin::parameter_changed_externally (uint32_t which, float value )
+VSTPlugin::parameter_changed_externally (uint32_t which, float value)
 {
 	ParameterChangedExternally (which, value); /* EMIT SIGNAL */
-	Plugin::set_parameter (which, value);
+	Plugin::set_parameter (which, value, 0);
 }
 
 
@@ -323,7 +324,6 @@ VSTPlugin::set_state (const XMLNode& node, int version)
 			_plugin->dispatcher (_plugin, effSetProgram, 0, pgm, NULL, 0);
 		};
 
-		XMLPropertyList::const_iterator i;
 		XMLNodeList::const_iterator n;
 
 		for (n = child->children ().begin (); n != child->children ().end (); ++n) {
@@ -427,9 +427,8 @@ VSTPlugin::get_parameter_descriptor (uint32_t which, ParameterDescriptor& desc)
 
 	if (_parameter_defaults.find (which) == _parameter_defaults.end ()) {
 		_parameter_defaults[which] = get_parameter (which);
-	} else {
-		desc.normal = _parameter_defaults[which];
 	}
+	desc.normal = _parameter_defaults[which];
 
 	return 0;
 }
@@ -531,7 +530,7 @@ VSTPlugin::load_user_preset (PresetRecord r)
 						assert (false);
 					}
 
-					set_parameter (index, value);
+					set_parameter (index, value, 0);
 					PresetPortSetValue (index, value); /* EMIT SIGNAL */
 				}
 			}
@@ -944,23 +943,25 @@ VSTPlugin::presets_file () const
 }
 
 
-VSTPluginInfo::VSTPluginInfo (VSTInfo* nfo)
+VSTPluginInfo::VSTPluginInfo (VST2Info const& nfo)
 {
 
 	char buf[32];
-	snprintf (buf, sizeof (buf), "%d", nfo->UniqueID);
+	snprintf (buf, sizeof (buf), "%d", nfo.id);
 	unique_id = buf;
 
 	index = 0;
 
-	name = nfo->name;
-	creator = nfo->creator;
-	n_inputs.set_audio  (nfo->numInputs);
-	n_outputs.set_audio (nfo->numOutputs);
-	n_inputs.set_midi  ((nfo->wantMidi & 1) ? 1 : 0);
-	n_outputs.set_midi ((nfo->wantMidi & 2) ? 1 : 0);
+	name     = nfo.name;
+	creator  = nfo.creator;
+	category = nfo.category;
 
-	_is_instrument = nfo->isInstrument;
+	n_inputs.set_audio  (nfo.n_inputs);
+	n_outputs.set_audio (nfo.n_outputs);
+	n_inputs.set_midi   (nfo.n_midi_inputs);
+	n_outputs.set_midi  (nfo.n_midi_outputs);
+
+	_is_instrument = nfo.is_instrument;
 }
 
 bool
