@@ -978,14 +978,17 @@ JACKAudioBackend::process_thread ()
 
         while (1) {
                 GET_PRIVATE_JACK_POINTER_RET(_priv_jack,0);
-
+                dsp_stats[AudioBackend::DeviceWait].start ();
                 pframes_t nframes = jack_cycle_wait (_priv_jack);
+                dsp_stats[AudioBackend::DeviceWait].update ();
+                dsp_stats[RunLoop].start();
 
                 if (engine.process_callback (nframes)) {
                         return 0;
                 }
 
 		jack_cycle_signal (_priv_jack, 0);
+                dsp_stats[AudioBackend::RunLoop].update ();
         }
 
         return 0;
