@@ -562,7 +562,8 @@ ContourDesignControlProtocol::prev_marker_keep_rolling ()
 	samplepos_t pos = session->locations()->first_mark_before (session->transport_sample());
 
 	if (pos >= 0) {
-		session->request_locate (pos, RollIfAppropriate);
+
+		session->request_locate (pos);
 	} else {
 		session->goto_start ();
 	}
@@ -574,7 +575,7 @@ ContourDesignControlProtocol::next_marker_keep_rolling ()
 	samplepos_t pos = session->locations()->first_mark_after (session->transport_sample());
 
 	if (pos >= 0) {
-		session->request_locate (pos, RollIfAppropriate);
+		session->request_locate (pos);
 	} else {
 		session->goto_end();
 	}
