@@ -114,7 +114,6 @@ LaunchControlXL::~LaunchControlXL ()
 	DEBUG_TRACE (DEBUG::LaunchControlXL, "Launch Control XL  control surface object being destroyed\n");
 
 	/* do this before stopping the event loop, so that we don't get any notifications */
-	port_reg_connection.disconnect ();
 	port_connection.disconnect ();
 	session_connections.drop_connections ();
 	stripable_connections.drop_connections ();
@@ -800,6 +799,7 @@ LaunchControlXL::set_state (const XMLNode & node, int version)
 	if ((child = node.child (X_("Input"))) != 0) {
 		XMLNode* portnode = child->child (Port::state_node_name.c_str());
 		if (portnode) {
+			portnode->remove_property ("name");
 			_async_in->set_state (*portnode, version);
 		}
 	}
@@ -807,6 +807,7 @@ LaunchControlXL::set_state (const XMLNode & node, int version)
 	if ((child = node.child (X_("Output"))) != 0) {
 		XMLNode* portnode = child->child (Port::state_node_name.c_str());
 		if (portnode) {
+			portnode->remove_property ("name");
 			_async_out->set_state (*portnode, version);
 		}
 	}
