@@ -17,6 +17,7 @@
 
 #include <glibmm.h>
 
+#include "pbd/basename.h"
 #include "pbd/debug.h"
 #include "pbd/error.h"
 #include "pbd/event_loop.h"
@@ -25,6 +26,7 @@
 #include "pbd/reallocpool.h"
 #include "pbd/receiver.h"
 #include "pbd/transmitter.h"
+#include "pbd/win_console.h"
 
 #include "ardour/ardour.h"
 #include "ardour/audioengine.h"
@@ -92,6 +94,7 @@ protected:
 		std::cout << prefix << str << std::endl;
 
 		if (chn == Transmitter::Fatal) {
+			console_madness_end ();
 			::exit (9);
 		}
 	}
@@ -141,8 +144,9 @@ static MyEventLoop* event_loop = NULL;
 static void
 init ()
 {
-	if (!ARDOUR::init (false, true, localedir)) {
+	if (!ARDOUR::init (true, localedir)) {
 		cerr << "Ardour failed to initialize\n" << endl;
+		console_madness_end ();
 		::exit (EXIT_FAILURE);
 	}
 
@@ -243,12 +247,19 @@ _create_session (string dir, string state, uint32_t rate) // throws
 }
 
 static Session*
-_load_session (string dir, string state) // throws
+_load_session (string const& dir, string state) // throws
 {
 	if (prepare_engine ()) {
 		return 0;
 	}
 
+	if (state.empty ()) {
+		state = Session::get_snapshot_from_instant (dir);
+	}
+	if (state.empty ()) {
+		state = PBD::basename_nosuffix (dir);
+	}
+
 	float        sr;
 	SampleFormat sf;
 	std::string  v;
@@ -386,6 +397,7 @@ engine_halted (const char* err)
 {
 	if (terminate_when_halted) {
 		cerr << "Engine halted: " << err << "\n";
+		console_madness_end ();
 		::exit (EXIT_FAILURE);
 	}
 }
@@ -536,7 +548,7 @@ usage ()
   -i, --interactive          enter interactive mode after executing 'script',\n\
                              force the interpreter to run interactively\n\
   -X, --exit-when-halted     terminate when the audio-engine halts\n\
-                             unexpectedly (disconnect, or too many x-runs)\n\
+                             unexpectedly (disconnect, or too many xruns)\n\
   -V, --version              print version information and exit\n\
 \n");
 	printf ("\n\
@@ -544,6 +556,7 @@ Ardour at your finger tips...\n\
 \n");
 	printf ("Report bugs to <http://tracker.ardour.org/>\n"
 	        "Website: <http://ardour.org/>\n");
+	console_madness_end ();
 	::exit (EXIT_SUCCESS);
 }
 
@@ -560,6 +573,7 @@ main (int argc, char** argv)
 	};
 
 	bool interactive = false;
+	console_madness_begin ();
 
 	int c = 0;
 	while (EOF != (c = getopt_long (argc, argv,
@@ -576,6 +590,7 @@ main (int argc, char** argv)
 			case 'V':
 				printf ("ardour-lua version %s\n\n", VERSIONSTRING);
 				printf ("Copyright (C) GPL 2015-2020 Robin Gareus <robin@gareus.org>\n");
+				console_madness_end ();
 				exit (EXIT_SUCCESS);
 				break;
 
@@ -585,6 +600,7 @@ main (int argc, char** argv)
 
 			default:
 				cerr << "Error: unrecognized option. See --help for usage information.\n";
+				console_madness_end ();
 				::exit (EXIT_FAILURE);
 				break;
 		}
@@ -604,8 +620,10 @@ main (int argc, char** argv)
 		lua_setglobal (L, "arg");
 	}
 
+	int res = 0;
+
 	if (argc > optind && 0 != strcmp (argv[optind], "-")) {
-		lua->do_file (argv[optind]);
+		res = lua->do_file (argv[optind]);
 		if (!interactive) {
 			keep_running = false;
 		}
@@ -616,7 +634,7 @@ main (int argc, char** argv)
 	} else if (is_tty () || interactive) {
 		interactive_interpreter ();
 	} else {
-		luaL_dofile (lua->getState (), NULL);
+		res = luaL_dofile (lua->getState (), NULL);
 	}
 
 	if (session) {
@@ -634,5 +652,6 @@ main (int argc, char** argv)
 	ARDOUR::cleanup ();
 	delete event_loop;
 	pthread_cancel_all ();
-	return 0;
+	console_madness_end ();
+	return res;
 }
