@@ -1,7 +1,7 @@
 ardour {
 	["type"]    = "EditorHook",
 	name        = "Make all Regions Transparent",
-	author      = "Ardour Lua Task Force",
+	author      = "Ardour Team",
 	description = "While the transport is looping, all regions become transparent.",
 }
 
@@ -25,4 +25,4 @@ function factory ()
 		end
 		::next::
 	end
-end end
\ No newline at end of file
+end end
